/**
 * \file gre/datatipgeneral.h
 * \brief GRE::DATATIPGENERAL definitions
 *
 * \if NODOC
 * $Id: datatipgeneral.h_v 1.1 2013/10/01 16:28:11 mju Exp $
 *
 * $Log: datatipgeneral.h_v $
 * Revision 1.1  2013/10/01 16:28:11  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_DATATIPGENERAL_H
#define	INC_GRE_DATATIPGENERAL_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

namespace GRE {

//=====================================================================================================================

//! General (process-wide) DataTip settings.
class DATATIPGENERAL {
	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class FORM;
		#endif

		//! Dialog to modify settings.
		static void DlgSettings (
			MDLGPARENT dlgparent
			);

		//! Get shared class instance.
		static const DATATIPGENERAL& GetShared ()
			{ return (AccessShared()); }

		bool GetDefaultHighlightElems () const
			{ return (m_bDefaultHighlightElems); }

		bool GetEnableHighlightElems () const
			{ return (m_bEnableHighlightElems); }

		INT16 GetMaxLines () const
			{ return (m_MaxLines); }

		bool GetShowObjectNames () const
			{ return (m_bShowObjectNames); }

		char GetThousandSeparator () const
			{ return (m_Sep000); }

		void SetDefaultHighlightElems (
			bool bDefaultHighlightElems
			) { m_bDefaultHighlightElems = bDefaultHighlightElems; }

		void SetEnableHighlightElems (
			bool bEnableHighlightElems
			) { m_bEnableHighlightElems = bEnableHighlightElems; }

		void SetMaxLines (
			INT16 MaxLines
			) { m_MaxLines = MaxLines; }

		void SetShowObjectNames (
			bool bShowObjectNames
			) { m_bShowObjectNames = bShowObjectNames; }

		void SetThousandSeparator (
			char Sep000
			) { m_Sep000 = Sep000; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class DLG;

		INT16 m_MaxLines;
		char m_Sep000;
		bool m_bEnableHighlightElems;
		bool m_bDefaultHighlightElems;
		bool m_bShowObjectNames;

		static DATATIPGENERAL& AccessShared ();

		DATATIPGENERAL ();

		void IniRead ();
		void IniWrite () const;

		// Unimplemented.
		DATATIPGENERAL (DATATIPGENERAL&);
		DATATIPGENERAL& operator= (DATATIPGENERAL&);
		#endif
	};

//=====================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_DATATIPGENERAL_H
