/**
 * \file gre/dlgmaskedit.h
 *
 * \if NODOC
 * $Id: dlgmaskedit.h_v 1.31 2013/09/27 17:02:50 bcolombe Exp $
 *
 * $Log: dlgmaskedit.h_v $
 * Revision 1.31  2013/09/27 17:02:50  bcolombe
 * Remove unneeded methods
 *
 * Revision 1.30  2013/09/19 22:31:18  bcolombe
 * Add OnApply
 *
 * Revision 1.29  2013/08/08 15:48:39  bcolombe
 * Use PRIV
 *
 * Revision 1.28  2013/07/03 19:41:15  bcolombe
 * Resume and Suspend
 *
 * Revision 1.27  2013/06/11 13:13:58  mju
 * Remove unnecessary hdrs.
 *
 * Revision 1.26  2013/06/11 12:59:22  mju
 * Use formcolorselector hdr.
 *
 * Revision 1.25  2013/04/02 16:49:18  bcolombe
 * Don't override Suspend and Resume
 *
 * Revision 1.24  2012/12/04 22:20:09  mju
 * Make some methods non-static so can use with drawgen delegates.
 *
 * Revision 1.23  2012/08/13 19:38:49  bcolombe
 * Added fillrect
 *
 * Revision 1.22  2011/12/05 17:44:49  bcolombe
 * Add GetLocatorView
 *
 * Revision 1.21  2011/08/26 13:23:37  mju
 * Use formradiobuttons hdr.
 *
 * Revision 1.20  2010/01/14 17:26:56  bcolombe
 * Use single toolbar
 *
 * Revision 1.19  2009/06/23 20:01:10  mju
 * Use getObjItem.
 *
 * Revision 1.18  2008/10/16 13:13:16  mju
 * Remove mxtools hdr.
 *
 * Revision 1.17  2008/10/07 16:34:52  bcolombe
 * Added RCB_Mark
 *
 * Revision 1.16  2008/09/08 20:05:12  bcolombe
 * Add pane to tooldata
 *
 * Revision 1.15  2008/07/18 21:45:33  bcolombe
 * Added ability to expand/shrink marked area with tool
 *
 * Revision 1.14  2008/07/17 21:38:16  bcolombe
 * Removed ObjectViewSetRast
 *
 * Revision 1.13  2008/06/24 16:51:45  bcolombe
 * Use new view and tools
 *
 * Revision 1.12  2007/11/12 20:28:38  bcolombe
 * Use GRE::TOOL
 *
 * Revision 1.11  2007/11/12 15:03:15  mju
 * Remove unneeded legacy hdrs.
 *
 * Revision 1.10  2007/10/02 15:23:42  scowan
 * Changed createDialog to createMaskDialog because under mfc, it conflicts
 * with windows macro.
 *
 * Revision 1.8  2007/08/01 22:08:52  bcolombe
 * Moved clear mask button to mask edit toolbar
 *
 * Revision 1.7  2007/07/30 20:56:23  bcolombe
 * Added onchange callback for tools
 *
 * Revision 1.6  2007/07/20 15:40:35  bcolombe
 * Added read/write for mask that takes void* and converts to INT8
 * Added own fillseeded function
 *
 * Revision 1.5  2007/07/18 22:39:12  bcolombe
 * Removed some unused extents code
 *
 * Revision 1.4  2007/07/17 21:21:19  bcolombe
 * Change get/setfillval to work with multi-band rasters
 *
 * Revision 1.3  2007/07/16 17:28:08  bcolombe
 * Added 'tolerance' settings to floodfill
 *
 * Revision 1.2  2007/07/02 19:00:34  bcolombe
 * Added private function for inverting mark layer
 *
 * Revision 1.1  2007/06/18 17:30:38  bcolombe
 * Initial revision
 * \endif
****************************************************************************/

#ifndef	INC_GRE_DLGMASKEDIT_H
#define	INC_GRE_DLGMASKEDIT_H

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
namespace GRE {
	class LAYER_RASTER;
	}
#endif

namespace GRE {

//! Class to create a dialog for editing a raster mask
class DLG_MASKEDIT : public MGUI::DLGSHELL {
	public:

		DLG_MASKEDIT (
			MDLGPARENT DlgParent = NULL,			//!< DlgParent of process, null if process is standalone
			GRE::LAYER_RASTER *RasterLayer = NULL	//!< Raster layer to use, null if needs to be created
			);

		~DLG_MASKEDIT ();

		//! Create the dialog
		void Create ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		typedef DLG_MASKEDIT THISCLASS;

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnCloseRequest ();
		#endif
	};
} //end GRE namespace

#endif //INC_MI32_DLGMASKEDIT_H