/**
 * \file gre/legendparms.h
 * \brief GRE LEGENDPARMS definitions
 *
 * \if NODOC
 * $Id: legendparms.h_v 1.2 2015/06/08 20:54:21 mju Exp $
 *
 * $Log: legendparms.h_v $
 * Revision 1.2  2015/06/08 20:54:21  mju
 * Add member for sort table/field.
 *
 * Revision 1.1  2006/09/28 13:14:51  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LEGENDPARMS_H
#define	INC_GRE_LEGENDPARMS_H

#ifndef	INC_RVC_OBJECTNAME_H
	#include <rvc/objectname.h>
#endif

#ifndef	INC_MI32_MG2ENUMS_H
	#include <mi32/mg2enums.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class DBASE;
	class DBTABLE;
	}
#endif

namespace GRE {

//! Parameters for Legend associated with element type.
struct LEGENDPARMS {
	public:

		RVC::OBJECTNAME TableName;					//!< Table for legend labels
		RVC::OBJECTNAME FieldName;					//!< Field for legend labels
		MISTRING m_SortTableField;					//!< Table.Field used for sorting, blank to use same as label
		MGD::STYLESAMPLEFLAGS SampleFlags;		//!< Flags for legend element drawing

		//! Default constructor.
		LEGENDPARMS ();

		//! Destructor.
		~LEGENDPARMS ();

		//! Get table and field for label.
		//! @return Field number, -1 if none specified and not using default
		int GetLabelTableField (
			RVC::DBASE &ObjDb,						//!< Open database object
			RVC::DBTABLE &LabelTable,				//!< Label table passed/returned
			bool bDftUseRelated = true				//!< Default to use style relation field if none set
			) const;

		//! Get table and field for sorting.
		int GetSortTableField (
			RVC::DBASE &ObjDb,						//!< Open database object
			RVC::DBTABLE &ObjTableSort				//!< Sort table passed/returned
			) const;
	};

}	// End namespace GRE

#endif	// INC_GRE_LEGENDPARMS_H
