/**
 * \file gre/lrtilesetpublic.h
 * \brief GRE LAYER_TILESET_PUBLIC definitions
 *
 * \if NODOC
 * $Id: lrtilesetpublic.h_v 1.5 2014/06/20 21:23:57 mju Exp $
 *
 * $Log: lrtilesetpublic.h_v $
 * Revision 1.5  2014/06/20 21:23:57  mju
 * Add v_hasProperty.
 *
 * Revision 1.4  2013/12/23 17:18:12  bcolombe
 * Changes for reverse geocode datatip
 *
 * Revision 1.3  2013/09/30 16:59:56  mju
 * Add view parm to dataTipHasEnabled.
 *
 * Revision 1.2  2013/08/30 15:51:54  bcolombe
 * Add static Create method
 *
 * Revision 1.1  2013/08/26 19:52:26  bcolombe
 * Initial revision
 * *
 * \endif
**/

#ifndef	INC_GRE_LRTILESETPUBLIC_H
#define	INC_GRE_LRTILESETPUBLIC_H

#ifndef	INC_GRE_LRIMAGE_H
#include <gre/lrimage.h>
#endif

#ifndef INC_SPATMOD_IMAGESOURCETILESET_PUBLIC_H
#include <spatmod/imagesourcetilesetpublic.h>
#endif

#ifndef INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#include <spatmod/imageresamplemethod.h>
#endif

namespace GRE {
//===================================================================================================================

//! Tileset Layer class
class LAYER_TILESET_PUBLIC : public GRE::LAYER_IMAGE {
	public:

		// LAYER_TILESET::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:
				DISPPARM ();

				DISPPARM (
					const DISPPARM& rhs
					);

				virtual ~DISPPARM ();

				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Copy(rhs); } return (*this); }

				const SPATMOD::IMAGE::SOURCE_TILESET_PUBLIC::MAPSTYLE& GetMapStyle (
					) const { return (m_MapStyle); }

				SPATMOD::IMAGE::RESAMPLEMETHOD GetResampleMethod (
					) const { return (m_ResampleMethod); }

				bool GetShowDatatipTileInfo (
					) const { return (m_bShowDatatipTileInfo); }

				bool GetShowDatatipImageValue (
					) const { return (m_bShowDatatipImageValue); }

				bool GetShowDatatipDate (
					) const { return (m_bShowDatatipDate); }

				bool GetShowLabels (
					) const { return (m_bShowLabels); }

				//! Determine if has been validated.
				bool IsValidated (
					) const { return (m_IsValidated); }

				void SetMapStyle (
					const SPATMOD::IMAGE::SOURCE_TILESET_PUBLIC::MAPSTYLE& MapStyle
					) { m_MapStyle = MapStyle; }

				void SetResampleMethod (
					SPATMOD::IMAGE::RESAMPLEMETHOD ResampleMethod
					) { m_ResampleMethod = ResampleMethod; }

				void SetShowDatatipTileInfo (
					bool ShowDatatipTileInfo
					) { m_bShowDatatipTileInfo = ShowDatatipTileInfo; }

				void SetShowDatatipImageValue (
					bool ShowDatatipImageValue
					) { m_bShowDatatipImageValue = ShowDatatipImageValue; }

				void SetShowDatatipDate (
					bool ShowDatatipDate
					) { m_bShowDatatipDate = ShowDatatipDate; }

				void SetShowLabels (
					bool ShowLabels
					) { m_bShowLabels = ShowLabels; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				bool m_IsValidated;
				bool m_bShowDatatipTileInfo;
				bool m_bShowDatatipImageValue;
				bool m_bShowDatatipDate;
				bool m_bShowLabels;
				SPATMOD::IMAGE::SOURCE_TILESET_PUBLIC::MAPSTYLE m_MapStyle;
				SPATMOD::IMAGE::RESAMPLEMETHOD m_ResampleMethod;

				void Copy (const DISPPARM&);

				// SERIALIZABLE overrides
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

				// LAYER::DISPPARM overrides
				virtual ERRVALUE v_Validate ();

				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_TILESET::DISPPARM

		#ifdef USE_MGUI
		//! Show bing maps terms of use agreement.  Returns 0 if agreed, EUserCancel if not
		static ERRVALUE DlgShowTermsOfUse (
			SPATMOD::IMAGE::SOURCE_TILESET_PUBLIC::MAPSTYLE MapStyle,
			MDLGPARENT dlgparent,
			bool bForceToShow = false	//!< True to ignore if terms have been agreed to before and show dialog
			);

		//! Get string with reverse geocoded location information
		static ERRVALUE BingDataTipReverseGeocode (
			MISTRING& string,			//!< String to return
			const DPOINT2D& point,		//!< DataTip point in disp coords
			GRE::VIEW *view				//!< View
			);
		#endif

		//! Construct layer from mapstyle
		static ERRVALUE Create (
			SPATMOD::IMAGE::SOURCE_TILESET_PUBLIC::MAPSTYLE MapStyle,
			GROUP *group = NULL
			);

		//! Register this layer type
		static void RegisterType (
			);

		//! Construct layer and insert into list
		explicit LAYER_TILESET_PUBLIC (
			GRE::GROUP *group,							//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,	//!< Position in list
			LAYER *reflayer = 0							//!< Reference layer for insertion
			);

		//! Construct layer and insert into list
		explicit LAYER_TILESET_PUBLIC (
			GRE::GROUP *group,							//!< Group to contain layer
			SPATMOD::IMAGE::SOURCE_TILESET_PUBLIC::MAPSTYLE MapStyle, //!< Map Style
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,	//!< Position in list
			LAYER *reflayer = 0							//!< Reference layer for insertion
			);

		virtual ~LAYER_TILESET_PUBLIC ();

		//! Retrieve layer-specific display parameters.
		//! @return current parameters
		const DISPPARM& GetDispParm (
			) const {
			return (m_dispparm);
			}

		//! Set layer-specific display parameters.
		void SetDispParm (
			const DISPPARM& dispparm,					//!< Parameters to set
			bool bUpdate = true							//!< Redraw view and set flags
			);

	protected:
		DISPPARM m_dispparm;								//!< Layer-specific display parameters
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef LAYER_IMAGE BASECLASS;
		
		class DLG;
		#ifdef USE_MGUI
			DLG *m_pDlgControlPanel;
		#endif

		SPATMOD::IMAGE::SOURCE_TILESET_PUBLIC* m_source;
		SIMPLE_ARRAY<INT32> m_ZoomLevelArray;
		bool m_HasError;

		ERRVALUE Draw2D (LAYERDC& layerdc);
		static ERRVALUE GetBingDatatip (const DPOINT2D& pt, MISTRINGLIST& strings, MISTRING& datatipstring);

		#ifdef USE_MGUI
		void OnMenuTool (MGUI::ID, bool);
		#endif

		static void GRECB_Layer (const GRE::MSG *msg, void *vpCommonParm);

		// Overrides from LAYER.
		#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
		virtual void v_MenuAddLayerItems (MGUI::MENU& menu);
		#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual void v_DataTipGetObjectNamesString (const DPOINT2D& point, MISTRING& string, int indent, GRE::VIEW *view) const;
		virtual int v_DataTipGetString (const DPOINT2D& point, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY& images, GRE::VIEW *view);
		virtual bool v_DataTipHasEnabled (GRE::VIEW *view) const;
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual ERRVALUE v_DrawOverlay (LAYERDC& layerdc);
		virtual double v_GetScaleMaxDetail (const DPOINT2D& point, const SPATREF::COORDREFSYS& crs, double DisplayPixelSizeMM) const;
		virtual bool v_HasProperty (PROPERTY property) const;
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual ERRVALUE v_UpdateExtents (bool forceupdate);
		
		LAYER_TILESET_PUBLIC (const LAYER_TILESET_PUBLIC&);
		LAYER_TILESET_PUBLIC& operator= (const LAYER_TILESET_PUBLIC&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER_TILESET_PUBLIC


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRTILESET_PUBLIC_H
