/**
 * \file gre/lrtin.h
 * \brief GRE_LAYER_TIN class definition.
 *
 * \if NODOC
 * $Id: lrtin.h_v 1.43 2014/11/21 20:54:03 mju Exp $
 *
 * $Log: lrtin.h_v $
 * Revision 1.43  2014/11/21 20:54:03  mju
 * Add v_getInfoDisplay.
 *
 * Revision 1.42  2014/06/20 21:24:02  mju
 * Add v_hasProperty.
 *
 * Revision 1.41  2014/06/10 14:20:20  mju
 * Remove legendMsgHandler.
 *
 * Revision 1.40  2013/10/11 19:47:05  mju
 * Fix dispparm virtual method names.
 *
 * Revision 1.39  2013/10/10 22:21:33  mju
 * Add copySettings.
 *
 * Revision 1.38  2013/02/04 20:39:24  mju
 * Add m_enableFill3D.
 *
 * Revision 1.37  2010/08/19 17:40:45  mju
 * Use elemnum type.
 *
 * Revision 1.36  2010/04/23 16:54:55  mju
 * Use unsignedset.
 *
 * Revision 1.34  2008/05/12 16:59:57  mju
 * Add parm to constructLayerSection.
 *
 * Revision 1.33  2008/03/17 21:33:47  mju
 * Change v_getLayerView to v_getLayerSection.
 *
 * Revision 1.32  2007/03/12 17:42:56  mju
 * Add v_setObjectDescriptor.
 *
 * Revision 1.18  2005/02/25 23:22:48  scowan
 * Added copy virtual method implementation.
 *
 * Revision 1.1  2002/09/09 20:37:35  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRTIN_H
#define	INC_GRE_LRTIN_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_GRE_ELEMPARMS_H
#include <gre/elemparms.h>
#endif

#ifndef	INC_GRE_LEGEND_H
#include <gre/legend.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class TIN;
	}
#endif

namespace GRE {
//===================================================================================================================

//! GRE layer class for viewing TIN (Triangulated Irregular Network) spatial object.
class LAYER_TIN : public LAYER {
	public:

		// LAYER_TIN::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:

				class DLG;										//!< Layer settings dialog, defined in <gre/uitin.h>

				struct CONTOUR {
					UINT8 Selected;
					LINESTYLE Style;
					double Interval;
					};

				RVC::OBJITEM m_ObjItem;
				RVC::OBJECTNAME m_GeorefName;				//!< Name of georeference object
				RVC::OBJITEM m_ObjItemStyle;
				POINTPARMS Node;
				LINEPARMS Edge;
				POLYPARMS Triangle;
				CONTOUR Contour[2];
				UINT8 ContourUnitCode;
				bool m_IsEditable;
				bool m_HasSaveInfo;
				bool m_EnableFill3D;
				mutable RVC::OBJITEM m_SaveObjItem;
				mutable RVC::OBJITEM m_SaveObjItemStyle;

				DISPPARM ();
				DISPPARM (
					const DISPPARM& rhs
					): LAYER::DISPPARM(rhs) { Copy(rhs); }
				virtual ~DISPPARM (
					) { Free(); }

				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Free(); Copy(rhs); } return (*this); }

				// Copy settings without changing object.
				// Does not validate.
				void CopySettingsFrom (
					const DISPPARM& other
					);

				//! Set to use specified object.
				ERRVALUE SetObject (
					const RVC::OBJITEM& objitem,		//!< Object to set
					bool keepsettings = false		//!< Retain current display settings
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				void Copy (const DISPPARM&);
				void Free ();
				static ERRVALUE SerializerCB_StyleFileName (SERIALIZER&, const SERIALIZERITEM *, void *, SERIALIZERITEM::ACTION);
				void UpdateSaveInfo () const;

				// Overrides from SERIALIZABLE.
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

				// Overrides from LAYER::DISPPARM.
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();

				friend class LAYER_TIN;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_TIN::DISPPARM

		//! Register this layer type
		static void RegisterType ();

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context
			);

		//! Construct layer and insert into list
		explicit LAYER_TIN (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_TIN ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm () const
			{ return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIVDRAWDATA;
		#endif // GENERATING_DOXYGEN_OUTPUT

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef LAYER BASECLASS;

		GRE::LEGEND_POINT m_NodeLegend;
		GRE::LEGEND_LINE m_EdgeLegend;
		GRE::LEGEND_POLYGON m_TriangleLegend;
		GRE::LEGEND_LINE m_ContourLegend;
		COMPINFO m_NodeCompInfo;
		COMPINFO m_EdgeCompInfo;
		COMPINFO m_TriangleCompInfo;
	#ifdef USE_MGUI
		DISPPARM::DLG *m_pDlgControlPanel;
	#endif
		RVC::TIN *m_pObjTIN;

		ERRVALUE OpenPrivateObject () const;
		void SetupLegend ();
		int SetupRegion ();

		// Overrides from LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		virtual GRE::LAYERSECTION* v_ConstructLayerSection (GRE::VIEW* view);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_CopySettingsFrom (const LAYER *other);
		virtual const DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual ERRVALUE v_DataTipSetParms (int component, const DATATIPPARMS& DataTipParms);
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual ERRVALUE v_DrawElementLow (LAYERDC& layerdc, ELEMTYPE ElemType, ELEMNUM ElemNum, const void *style);
		virtual ERRVALUE v_DrawElementsLow (LAYERDC& layerdc, ELEMTYPE ElemType, const UNSIGNEDSET *pElemSet, const void *style);
		virtual ELEMNUM v_FindClosestElement (const DPOINT2D& point, double searchdist, double *distret, int component, const UNSIGNEDSET *pSearchSet, GRE::FINDFLAGS findflags);
		virtual int v_GetComponent (ELEMTYPE ElemType) const;
		virtual MICONSTRUCT* v_GetComponentIcon (int component) const;
		virtual COMPINFO* v_GetComponentInfo (int component);
		virtual MISTRING v_GetComponentName (int component) const;
		virtual const void* v_GetElemDefaultStyle (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawSelectionMode (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawStyleMode (ELEMTYPE ElemType) const;
		virtual ERRVALUE v_GetElemExtents (ELEMTYPE ElemType, ELEMNUM ElemNum, DRECT3D& Extents);
		virtual ERRVALUE v_GetElemSetByExtent (ELEMTYPE ElemType, const DRECT2D& Extent, UNSIGNEDSET& ElemSet);
		virtual ELEMTYPE v_GetElemType (int component) const;
		virtual bool v_GetGeorefName (RVC::OBJECTNAME& name, int component = 0) const;
		virtual MISTRING v_GetInfoDisplay (INFODISPLAY infodisplay, int component) const;
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElements (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual ERRVALUE v_GetStyleInfo (ELEMTYPE ElemType, RVC::OBJITEM& ObjItem, RVC::OBJECTNAME& TableName);
		virtual bool v_HasProperty (PROPERTY property) const;
		virtual bool v_IsTableUsableForComponent (int component, const RVC::DBTABLE& table) const;
	#ifdef USE_MGUI
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual ERRVALUE v_ReportRawElemInfo (MISTRING& infostr, ELEMTYPE ElemType, ELEMNUM ElemNum);
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetElemDrawSelectionMode (ELEMTYPE ElemType, UINT16 SelectMode);
		virtual void v_SetElemDrawStyleMode (ELEMTYPE ElemType, UINT16 StyleMode);
		virtual void v_SetLegendSampleFlags (MGD::STYLESAMPLEFLAGS SampleFlags);
		virtual void v_SetObjectDescriptor (int component, const RVC::DESCRIPTOR& descriptor);
		virtual ERRVALUE v_SetStyleObjItem (const RVC::OBJITEM& objitem);
		virtual int v_TestElementByRegion (ELEMTYPE ElemType, ELEMNUM ElemNum, const REGION2D& Region, TESTCRITERIA TestMode);
	#ifdef USE_MGUI
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
		virtual void v_TreeViewAddUpdateDetails (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual int v_UpdateExtents (bool forceupdate);

		LAYER_TIN (const LAYER_TIN&);
		LAYER_TIN& operator= (const LAYER_TIN&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_LRTIN_H
