/**
 * \file gre/uiformcontrastparm.h
 * \brief GRE::UI::FORM_CONTRASTPARM definitions
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
 * \endif
**/

#ifndef	INC_GRE_UIFORMCONTRASTPARM_H
#define	INC_GRE_UIFORMCONTRASTPARM_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class CONTRASTPARM;
#endif

namespace GRE {
namespace UI {

//=====================================================================================================================

//! Form for adjusting contrast parameters (CONTRASTPARM).
class FORM_CONTRASTPARM : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_CONTRASTPARM ();

		//! Destructor.
		~FORM_CONTRASTPARM ();

		//! Add component.
		//! @return Component index (0-based).
		//! Must add components before Create().
		int AddComponent (
			const MISTRING& label,				//!< Label for user interface
			const COLOR& color					//!< Color for graph
			);

		//! Create form.
		//! Must add at least one component via AddComponent first.
		void Create (
			MGUI::LAYOUT_PANE& paneParent		//!< Parent pane
			);

		//! Get contrast values for component.
		void GetValues (
			int component,							//!< Component index
			CONTRASTPARM& values					//!< Contrast values returned.
			) const;

		//! Set delegate for change notification.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate	//!< Delegate to call
			);

		//! Set delegate for save/save-as request.
		void SetDelegateOnSave (
			DELEGATE_ERRVALUE_BOOL delegate	//!< Delegate, bool value is true for "save-as", false for "save"
			);

		//! Set contrast values for component.
		void SetValues (
			int component,							//!< Component index
			const CONTRASTPARM& values,		//!< Values to set
			bool notify								//!< Notify delegate of change
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace GRE

#endif	// INC_GRE_UIFORMCONTRASTPARM_H
