/**
 * \file mgui/dlgeditname.h
 * \brief MGUI::DLG_EDIT_NAME definition
 *
 * \if NODOC
 * $Id: dlgeditname.h_v 1.1 2013/10/21 17:34:19 mju Exp $
 *
 * $Log: dlgeditname.h_v $
 * Revision 1.1  2013/10/21 17:34:19  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGEDITNAME_H
#define	INC_MGUI_DLGEDITNAME_H

#ifndef	INC_MGUI_DLGSHELL_H
	#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_EDIT_H
	#include <mgui/edit.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Dialog for editing "name" with possible checks for in-use and validity.
class DLG_EDIT_NAME : public DLGSHELL {
	public:

		//! Delegate to check name.
		typedef fastdelegate::FastDelegate<bool(const MISTRING&)> DELEGATE_CHECKNAME;

		//! Constructor.
		DLG_EDIT_NAME (
			const MISTRING& dlgtitle,					//!< Title for dialog
			const MISTRING& dftname,					//!< Initial default name
			int maxlength = 200							//!< Maximum name length
			);

		//! Destructor.
		virtual ~DLG_EDIT_NAME ();

		//! Get specified name.
		const MISTRING& GetName ()
			{ return (m_EditName.GetValue()); }

		//! Set delegate to check if name is in use.
		void SetDelegateCheckNameInUse (
			DELEGATE_CHECKNAME delegate				//!< Delegate, must return true if name is in use, false if not
			) { m_DelegateCheckNameInUse = delegate; }

		//! Set delegate to check if name is valid.
		void SetDelegateCheckNameValid (
			DELEGATE_CHECKNAME delegate				//!< Delegate, must return true if name is valid, false if not
			) { m_DelegateCheckNameValid = delegate; }

		//! Set valid or invalid characters for name.
		void SetValidChars (
			const MISTRING& ValidChars,				//!< String containing valid (or invalid if bInvert is true) characters
			bool bInvert = false							//!< Whether to invert (ie, specifying invalid characters)
			) { m_EditName.SetValidChars(ValidChars,bInvert); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_EDIT_NAME THISCLASS;

		FORM_EDIT_STRING m_EditName;
		CTRL_LABEL m_LblMsg;
		DELEGATE_CHECKNAME m_DelegateCheckNameInUse;
		DELEGATE_CHECKNAME m_DelegateCheckNameValid;
		int m_maxlength;

		void OnEditName ();

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOK ();
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_DLGEDITNAME_H
