/**
 * \file mgui/dlgselectbandnums.h
 * \brief Functions to prompt user to select band numbers from larger set
 *
 * \if NODOC
 * $Id: dlgselectbandnums.h_v 1.3 2014/01/24 16:45:10 mju Exp $
 *
 * $Log: dlgselectbandnums.h_v $
 * Revision 1.3  2014/01/24 16:45:10  mju
 * Add delegate to get source band name.
 *
 * Revision 1.1  2009/04/30 15:03:45  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_DLGSELECTBANDNUMS_H
#define	INC_MGUI_DLGSELECTBANDNUMS_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

namespace MGUI {
class FORM_COMBOBOX;

//! Dialog to allow user to choose limited set of bands from larger set.
class DLG_SELECTBANDNUMS : public MGUI::DLGSHELL {
	public:

		enum FLAGS {
			FLAG_None =					0x00,
			FLAG_AllowNone =			0x01,		//!< Include "none", will return band number as -1
			FLAG_AllowDuplicate =	0x02,		//!< Allow same source band to be used more than once
			};

		//! Delegate to get source band name.
		//! Parameter is 0-based band index.
		typedef fastdelegate::FastDelegate<MISTRING(int)> DELEGATE_GETSRCBANDNAME;

		//! Construct with band names from TEXTID array.
		DLG_SELECTBANDNUMS (
			const SIMPLE_ARRAY<INT16>& BandNums,	//!< Initial band numbers
			const TEXTID *BandNames,					//!< Band names, must at least as many entries as BandNums
			int NumSrcBands,								//!< Number of source bands to select from
			FLAGS flags = FLAG_None
			);

		virtual ~DLG_SELECTBANDNUMS ();

		//! Get selected band numbers.
		const SIMPLE_ARRAY<INT16> GetBandNums () const
			{ return (m_BandNums); }

		//! Set delegate to get source band name.
		//! If no delegate is specified or delegate returns empty string the band number+1 will be displayed.
		void SetDelegateGetSrcBandName (
			DELEGATE_GETSRCBANDNAME delegate
			) { m_DelegateGetSrcBandName = delegate; }

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		typedef DLG_SELECTBANDNUMS THISCLASS;
		SIMPLE_ARRAY<INT16> m_BandNums;
		SIMPLE_ARRAY<TEXTID> m_BandNames;
		UINT32 m_NumSrcBands;
		FLAGS m_flags;
		MGUI::FORM_COMBOBOX *m_CbxBandNums;
		DELEGATE_GETSRCBANDNAME m_DelegateGetSrcBandName;

		void OnCbxBandNum ();

		virtual ERRVALUE v_CreateContent ();
		#endif
	};

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLG_SELECTBANDNUMS::FLAGS)
#endif


}	// End namespace MGUI

#endif	// INC_MGUI_DLGSELECTBANDNUMS_H
