/**
 * \file mgui/dlgselectunits2.h
 * \brief MGUI::DLG_SELECT_UNITS2 class definition
 *
 * \if NODOC
 * $Id: dlgselectunits2.h_v 1.2 2014/10/01 20:24:14 mju Exp $
 *
 * $Log: dlgselectunits2.h_v $
 * Revision 1.2  2014/10/01 20:24:14  mju
 * Add dlgstyle for showing numeric and string dates.
 *
 * Revision 1.1  2014/09/25 15:43:07  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGSELECTUNITS2_H
#define	INC_MGUI_DLGSELECTUNITS2_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MI32_UOM_H
#include <mi32/uom.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Dialog to select measure and units.
class DLG_SELECT_UNITS2 : public DLGSHELL {
	public:

		//! Dialog style.
		enum DLGSTYLE {
			DLGSTYLE_Default =				0,
			DLGSTYLE_AllowDate_Numeric =	0x01,
			DLGSTYLE_AllowDate_String =	0x02,
			DLGSTYLE_AllowDate_MASK =		0x03,
			};

		//! Construct with title and default units.
		DLG_SELECT_UNITS2 (
			const MISTRING& dlgtitle,					//!< Text for title
			const UOM::UNIT& unitDft,					//!< Initial default units
			DLGSTYLE dlgStyle = DLGSTYLE_Default	//!< Dialog style flags
			);

		//! Destructor.
		~DLG_SELECT_UNITS2 ();

		//! Get selected units.
		const UOM::UNIT& GetSelectedUnits () const
			{ return (m_Unit); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_SELECT_UNITS2 THISCLASS;

		UOM::UNIT m_Unit;
		FORM_COMBOBOX m_CbxMeasure;
		FORM_COMBOBOX m_CbxUnits;
		DLGSTYLE m_dlgstyle;

		void OnCbxMeasure ();
		void OnCbxUnits ();
		void SetupCbxUnits ();

		virtual ERRVALUE v_CreateContent ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLG_SELECT_UNITS2::DLGSTYLE)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_DLGSELECTUNITS2_H
