/**
 * \file mgui/formbtxt.h
 * \brief MGUI::FORM_BUTTONTEXT class definitions
 *
 * \if NODOC
 * $Id: formbtxt.h_v 1.17 2015/08/17 17:27:37 mju Exp $
 *
 * $Log: formbtxt.h_v $
 * Revision 1.17  2015/08/17 17:27:37  mju
 * Add style enum so can put button on right.
 *
 * Revision 1.13  2006/08/25 19:17:51  mju
 * Change to use ctrl_edit_string for mfc and add getTextCtrl.
 *
 * Revision 1.12  2006/07/21 20:49:39  mju
 * Add setToolTipString.
 *
 * Revision 1.1  2002/10/09 16:20:16  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMBTXT_H
#define	INC_MGUI_FORMBTXT_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif


namespace MGUI {

//=====================================================================================================================

//! Form containing PUSHBUTTON and non-editable text field.
class FORM_BUTTONTEXT : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum STYLE {
			STYLE_ButtonLeft =		0x00,			//!< Place button on left (default)
			STYLE_ButtonRight =		0x01			//!< Place button on right
			};

		//! Constructor.
		FORM_BUTTONTEXT ();

		//! Clear the text field.
		void ClearText ()
			{ m_Text.ClearValue(false); }

		//! Create with default style.
		void Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,	//!< Parent pane
			const MISTRING &label,					//!< Button label string
			int width = 0,								//!< Width in 'em' characters
			STYLE style = STYLE_ButtonLeft,		//!< Form style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Create with default style.
		DEPRECATED void Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,	//!< Parent pane
			const MISTRING &label,					//!< Button label string
			int width,									//!< Width in 'em' characters
			MGUI::LAYOUT_SIZEALIGN sizealign
			);
	#endif

		//! Get reference to button control.
		CTRL_PUSHBUTTON& GetButton ()
			{ return (m_Button); }

		//! Get reference to text control.
		//! @return non-editable control reference.
		CTRL_EDIT_STRING& GetTextCtrl ()
			{ return (m_Text); }

		//! Set delegate to call when button pressed.
		void SetDelegateOnPressed (
			DELEGATE_VOID_NOPARMS delegate
			) { m_Button.SetDelegateOnPressed(delegate); }

		//! Set text to show in field.
		void SetText (
			const MISTRING &string
			);

		//! Set text to show as ToolTip.
		void SetToolTipString (
			const MISTRING &string
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_EDIT_STRING m_Text;
		CTRL_PUSHBUTTON m_Button;

		FORM_BUTTONTEXT (const FORM_BUTTONTEXT&);
		FORM_BUTTONTEXT& operator= (const FORM_BUTTONTEXT&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMBTXT_H
