/**
 * \file mgui/formcolorgenerator.h
 * \brief FORM_COLORGENERATOR class definitions
 *
 * \if NODOC
 * $Id: formcolorgenerator.h_v 1.2 2015/06/12 21:07:27 mju Exp $
 *
 * $Log: formcolorgenerator.h_v $
 * Revision 1.2  2015/06/12 21:07:27  mju
 * Change getColorArray to return false if not enough colors.
 *
 * Revision 1.1  2015/06/12 19:54:35  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMCOLORGENERATOR_H
#define	INC_MGUI_FORMCOLORGENERATOR_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_MI32_COLORARRAY_H
#include <mi32/colorarray.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Form for color generation.
class FORM_COLORGENERATOR : FORM_COMPOSITE {
	public:

		//! Form style.
		enum STYLE {
			STYLE_Default =				0,
			STYLE_AllowNone =				0x01,		//!< Allow "none" option
			STYLE_AllowAllSame =			0x02,		//!< Allow "all same" option
			STYLE_AllowTransparency =	0x04,		//!< Include "transparency" control
			};

		//! Default constructor.
		FORM_COLORGENERATOR ();

		//! Destructor.
		~FORM_COLORGENERATOR ();

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE &paneParent,							//!< Parent pane
			const MISTRING &LabelStr,										//!< Label for mode control, can be blank
			const MISTRING &ValueStr,										//!< String with settings (from GetValueStr)
			STYLE formstyle = STYLE_Default,								//!< Form style
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get color for "all same" mode.
		const COLOR& GetColorAllSame () const;

		//! Get generated color array.
		//! @return TRUE if source has enough colors, FALSE if not.
		int GetColorArray (
			COLOR_ARRAY &ColorArray,		//!< Array of colors returned
			UINT32 NumColors = 0				//!< Number of colors to generate, 0 to use from SetNumColors
			) const;

		//! Get string containing all value settings.
		MISTRING GetValueStr () const;

		//! Determine if generation mode set to "all same".
		bool IsAllSame () const;

		//! Determine if generation mode set to "none".
		bool IsNone () const;

		//! Set delegate to notify if any setting changed.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set number of colors to generate.
		void SetNumColors (
			UINT32 NumColors
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_COLORGENERATOR::STYLE)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORM_COLORGENERATOR_H
