/**
 * \file <mgui/formcolorpalette.h>
 * \brief MGUI::FORM_COLORPALETTE defns
 *
 * \if NODOC
 * $Id: formcolorpalette.h_v 1.1 2015/06/11 21:42:12 mju Exp $
 *
 * $Log: formcolorpalette.h_v $
 * Revision 1.1  2015/06/11 21:42:12  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMCOLORPALETTE_H
#define	INC_MGUI_FORMCOLORPALETTE_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_MI32_COLORPALETTETYPE_H
#include <mi32/colorpalettetype.h>
#endif

#ifndef	INC_MI32_COLORARRAY_H
#include <mi32/colorarray.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Form to display and select color palette.
class FORM_COLORPALETTE : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum STYLE {
			STYLE_Default =		0x00,
			STYLE_Sequential =	0x01,		//!< Display "discrete" palette sequentially instead of arranging as specified
			};

		//! Constructor.
		FORM_COLORPALETTE ();

		//! Destructor.
		virtual ~FORM_COLORPALETTE ();

		//! Create form.
		void Create (
			LAYOUT_PANE_BASE &PaneParent,
			const MISTRING &LabelStr,		//!< Label string, can be blank
			COLORPALETTETYPE ValidTypes = COLORPALETTETYPE_Discrete,
			int width = 128,					//!< Form width in pixels
			STYLE style = STYLE_Default,
			LAYOUT_SIZEALIGN SizeAlign = LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get palette.
		const COLOR_ARRAY& GetPalette () const;

		//! Get selected palette name.
		const MISTRING& GetPaletteName () const;

		//! Get selected palette type.
		COLORPALETTETYPE GetPaletteType () const;

		//! Set delegate to call when selected palette changed.
		void SetDelegateOnChangeSelection (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set palette by name and optional type.
		ERRVALUE SetPalette (
			const MISTRING &PaletteName,							//!< Palette name
			COLORPALETTETYPE type = COLORPALETTETYPE_None,	//!< Palette type if known
			bool notify = false										//!< Whether to notify delegate of change
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_COLORPALETTE::STYLE)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMCOLORPALETTE_H
