/**
 * \file mgui/formcolorselector.h
 * \brief MGUI::FORM_COLORSELECTOR definitions
 *
 * \if NODOC
 * $Id: formcolorselector.h_v 1.2 2013/06/10 16:42:20 mju Exp $
 *
 * $Log: formcolorselector.h_v $
 * Revision 1.2  2013/06/10 16:42:20  mju
 * Add formstyle_ToggleTransparent and setColorCapability.
 *
 * Revision 1.1  2013/06/10 13:23:20  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMCOLORSELECTOR_H
#define	INC_MGUI_FORMCOLORSELECTOR_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Form for color selection.
//! This form includes a "palette", optional sliders or pushbutton for choosing color, and an
//! optional "edit" control for tranparency.
class FORM_COLORSELECTOR : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum FORMSTYLE {
			FORMSTYLE_Default =				0x00,
			FORMSTYLE_EditTransparency =	0x01,	//!< Include edit controls for transparency
			FORMSTYLE_UseSliders =			0x02,	//!< Include sliders for setting color components in various ways
			FORMSTYLE_NoSample =				0x04,	//!< Don't include larger color sample (use when showing elsewhere)
			FORMSTYLE_CenterPaletteH =		0x08,	//!< Center palette horizontally
			FORMSTYLE_ToggleTransparent =	0x10,	//!< Include toggle for "transparent"
			};

		//! Constructor.
		FORM_COLORSELECTOR ();

		//! Destructor.
		virtual ~FORM_COLORSELECTOR ();

		//! Create the form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			FORMSTYLE formstyle = FORMSTYLE_Default
			);

		//! Get current color.
		const COLOR& GetColor () const;

		//! Set color.
		void SetColor (
			const COLOR& color,
			bool notify = true
			);

		//! Set capability for color.
		//! If capability of setting transparent/transparency is desired, appropriate FORMSTYLE must be specified on creation.
		void SetColorCapability (
			COLOR::CAPABILITY capability		//! Capability for color, affects whether controls for transparency are available
			);

		//! Set delegate to call when color changed.
		void SetDelegateOnChangeColor (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to call when color changing via slider.
		void SetDelegateOnChangingColor (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set transparency factor.
		void SetTransparency (
			UINT8 transparency,					//!< 0 = opaque, 255 = invisible.
			bool notify = true
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_COLORSELECTOR (const FORM_COLORSELECTOR&);
		FORM_COLORSELECTOR& operator= (const FORM_COLORSELECTOR&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_COLORSELECTOR::FORMSTYLE)
#endif

//=====================================================================================================================

}		// End namespace MGUI

#endif	// INC_MGUI_FORMCOLORSELECTOR_H
