/**
 * \file mgui/formgeocatalog.h
 *
 * \if NODOC
 * $Id: formgeocatalog.h_v 1.14 2015/09/18 22:00:22 bcolombe Exp $
 *
 * $Log: formgeocatalog.h_v $
 * Revision 1.14  2015/09/18 22:00:22  bcolombe
 * Changes for filters
 *
 * Revision 1.13  2014/09/26 17:37:54  bcolombe
 * Add select item delegate
 *
 * Revision 1.12  2013/11/11 17:52:18  bcolombe
 * Destructor should be virtual
 *
 * Revision 1.11  2013/08/09 21:51:53  bcolombe
 * AddCatalog
 *
 * Revision 1.10  2013/07/25 22:33:15  bcolombe
 * Allow disable of web
 *
 * Revision 1.9  2013/07/18 22:42:57  bcolombe
 * Major UI updates to use single pane
 *
 * Revision 1.8  2013/07/15 16:13:05  bcolombe
 * Add CheckCatalogs to give warning if no catalogs
 *
 * Revision 1.7  2013/07/01 17:25:41  bcolombe
 * Change geocatlist to filepathlist
 *
 * Revision 1.6  2011/11/29 22:22:46  mju
 * Remove redundant hdr.
 *
 * Revision 1.5  2010/10/27 20:44:21  bcolombe
 * Conver to use form_list
 *
 * Revision 1.4  2009/10/30 17:47:36  mju
 * Add hasResults/Selected and getObjItemListResults.
 *
 * Revision 1.3  2009/07/02 18:02:59  bcolombe
 * Changes for setting CRS for displaying extents
 *
 * Revision 1.2  2009/04/20 15:54:44  bcolombe
 * Changed ui for searching by extents, uses listbox with options
 * Added finding point by geonames
 *
 * Revision 1.1  2009/04/15 20:09:27  bcolombe
 * Initial revision
 * \endif
****************************************************************************/

#ifndef	INC_MGUI_FORMGEOCATALOG_H
#define	INC_MGUI_FORMGEOCATALOG_H

#ifndef INC_MGUI_FORMLIST_H
#include <mgui/formlist.h>
#endif

#ifndef INC_MGUI_FORM_EDIT_EXTENTS_H
#include <mgui/formextents.h>
#endif

#ifndef INC_MGUI_FORM_EDIT_COORD_H
#include <mgui/formcoord.h>
#endif

#ifndef INC_MGUI_MENU_H
#include <mgui/menu.h>
#endif

#ifndef INC_MGUI_LISTBOX_H
#include <mgui/listbox.h>
#endif

#ifndef	INC_MI32_GEOCAT_H
#include <mi32/geocat.h>
#endif

#ifndef INC_MI32_MICON_H
#include <mi32/micon.h>
#endif

#ifndef INC_RVC_OBJITEMLIST_H
#include <rvc/objitemlist.h>
#endif

#ifndef INC_MGUI_TOOLBAR_H
#include <mgui/toolbar.h>
#endif

namespace MGUI {

//! Form for geocatalog browse/search
class FORM_GEOCATALOG : public MGUI::FORM_COMPOSITE {
	public:
		FORM_GEOCATALOG (
			);

		virtual ~FORM_GEOCATALOG (
			);

		//! Add catalog to list
		void AddCatalog (const FILEPATH& path);

		//! Check if have catalogs, displays warning and returns false if it does not
		bool CheckCatalogs ();

		//! Create Form
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			bool bIncludeWeb = true
			);

		//! Perform search
		void DoSearch ();

		//! Get all results.
		void GetObjItemListResults (
			RVC::OBJITEMLIST& ObjItemList
			);

		//! Get selected objitems
		void GetObjItemListSelected (
			RVC::OBJITEMLIST& ObjItemList
			);

		//! Determine if has any results.
		bool HasResults (
			) const;

		//! Determine if has any items selected in results.
		bool HasSelected (
			) const;

		//! Set allowed objtypes
		void SetObjTypeSet (
			const RVC::OBJTYPESET& objtypeset
			);

		//! Set extensions to match
		void SetExtnPattern (
			const MISTRING& pattern
			);

		//! Set object names to match
		void SetNamePattern (
			const MISTRING& pattern,
			bool UseSimpleFilter = true
			);

		//! delegate when user adds item from search results
		void SetAddItemDelegate (
			FastDelegate<void(const RVC::OBJITEM&)> delegate
			);

		//! delegate when user selects item from search results
		void SetSelectItemDelegate (
			DELEGATE_VOID_NOPARMS delegate
			);

	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif
	};

} //end MGUI namespace

#endif //INC_MGUI_FORMGEOCATALOG_H
