/**
 * \file mgui/indexbar.h
 * \brief MGUI::CTRL_INDEXBAR definitions
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
 * \endif
**/

#ifndef	INC_MGUI_INDEXBAR_H
#define	INC_MGUI_INDEXBAR_H

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

namespace MGUI {
//===================================================================================================================

//! Index Bar control to allow rapid position indexing into long sorted list.
class CTRL_INDEXBAR : public MGUI::CTRL {
	public:

		enum ORIENTATION {
			ORIENTATION_Horizontal,
			ORIENTATION_Vertical
			};

		//! Constructor.
		CTRL_INDEXBAR (
			);

		//! Destructor.
		virtual ~CTRL_INDEXBAR (
			);

		//! Add item with ID and label.
		void AddItem (
			ID id,
			const MISTRING& label
			);

		//! Create scrollbar control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			ORIENTATION orientation						//!< Orientation
			);

		//! Delete all items.
		void DeleteAllItems ();

		//! Get selection.
		ID GetSelectedItemID (
			) const;

		//! Disable redraw.
		//! Use when changing items after initial creation.
		void RedrawDisable ();

		//! Enable redraw.
		//! Use when changing items after initial creation.
		void RedrawEnable ();

		//! Set delegate to call while user is "browsing" (holding button down while dragging pointer along control).
		void SetDelegateOnBrowse (
			DELEGATE_VOID_ID delegate
			) { m_DelegateOnBrowse = delegate; }

		//! Set delegate to call after selection made.
		void SetDelegateOnSelection (
			DELEGATE_VOID_ID delegate
			) { m_DelegateOnSelection = delegate; }

		//! Set selection.
		void SetSelectedItemID (
			ID id
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DELEGATE_VOID_ID m_DelegateOnBrowse;
		DELEGATE_VOID_ID m_DelegateOnSelection;

		CTRL_INDEXBAR (const CTRL_INDEXBAR&);
		CTRL_INDEXBAR& operator= (const CTRL_INDEXBAR&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
} // End namespace MGUI

#endif	// INC_MGUI_INDEXBAR_H

