/**
 * \file mi32/fftfunc.h
 * \brief one dimensional Fast Fourier Transformation class
 *
 * \if NODOC
 * $Id: fftfunc.h_v 1.17 2015/09/09 20:10:38 mju Exp $
 *
 * $Log: fftfunc.h_v $
 * Revision 1.17  2015/09/09 20:10:38  mju
 * Add fourierformat enum.
 *
 * Revision 1.16  2015/08/24 20:47:31  mju
 * Move fft1d defns to own headers and rename.
 *
 * Revision 1.15  2015/08/24 19:48:21  mju
 * Move timeseries class to rastanly/rfseries as is only place used.
 *
 * Revision 1.14  2011/09/06 21:10:26  mju
 * Exclude macros and private stuff from dox.
 *
 * Revision 1.13  2008/10/20 20:18:21  scowan
 * Fixed to conform to current standards.
 *
 * Revision 1.12  2007/05/08 15:44:19  dwilliss
 * SML namespace
 *
 * Revision 1.10  2003/02/21 20:36:12  vdronov
 * added CopyGeoreference
 *
 * Revision 1.9  2002/07/22 23:29:39  vdronov
 * added FFT2D
 *
 * Revision 1.7  2002/01/10 20:22:03  vdronov
 * added standard FT and TIMESERIES classes
 *
 * Revision 1.5  2001/11/01 16:26:59  mju
 * Use DOUBLE_ARRAY and 'const' for all input arrays.
 *
 * Revision 1.4  2001/11/01 15:49:48  mju
 * Disable warning about non-DLL base class.
 * Don't include sml.h, forward-declare SML::CONTEXT instead.
 * Return error from Init().
 *
 * Revision 1.3  2001/10/16 16:51:20  vdronov
 * changed DOUBLE_ARRAYS to SIMPLE_ARRAY
 *
 * Revision 1.2  2001/10/16 14:33:40  vdronov
 * added SMLClassInstall static method
 *
 * Revision 1.1  2001/10/15 19:35:44  vdronov
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_FFTFUNC_H
#define	INC_MI32_FFTFUNC_H

#ifndef	INC_MI32_DOUBLEAR_H
	#include <mi32/doublear.h>
#endif

#ifndef	INC_RVC_IMAGE_H
	#include <rvc/image.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class OBJITEM;
	class RASTER;
	}
namespace SML {
	class CONTEXT;
	}
#endif // GENERATING_DOXYGEN_OUTPUT

//! Numeric format for fourier transformations.
enum FOURIERFORMAT {
	FOURIERFORMAT_In_RI =		0x00,	//!< Input Real/Imaginary
	FOURIERFORMAT_In_MP =		0x02,	//!< Input Magnitude/Phase
	FOURIERFORMAT_Out_RI =		0x00,	//!< Output Real/Imaginary
	FOURIERFORMAT_Out_MP =		0x01,	//!< Output Magnitude/Phase
	FOURIERFORMAT_In_MASK =		0x02,
	FOURIERFORMAT_Out_MASK =	0x01,
	FOURIERFORMAT_RI_RI = 0,	  //!< input - RI, output - RI
	FOURIERFORMAT_RI_MP = 1,	  //!< input - RI, output - MP
	FOURIERFORMAT_MP_RI = 2,	  //!< input - MP, output - RI
	FOURIERFORMAT_MP_MP = 3,	  //!< input - MP, output - MP
	};
		

class FFT2D {

	public:

		static ERRVALUE Forward (
			const RVC::RASTER& input,
			RVC::OBJITEM& objitem,
			const RVC::IMAGE::DATATYPE& datatype = RVC::IMAGE::DATATYPE_ComplexRI,
			const LRECT2D rect = LRECT2D(1,1,0,0)
			);

		static ERRVALUE Inverse (
			const RVC::RASTER& input,
			RVC::OBJITEM& objitem,
			const RVC::IMAGE::CELLTYPE& celltype = RVC::IMAGE::CELLTYPE_Float_32,
			const LRECT2D rect = LRECT2D(1,1,0,0)
			);

		static INT32 AdjustToPower2 (
			const INT32 num
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		static ERRVALUE CopyGeoreference (
			const RVC::RASTER& input,
			RVC::RASTER& output,
			const LRECT2D& rect
			);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#endif
