
//! Base for geometric element iterator.
class GEOMITERATOR {
	public:

		//! Pre-increment operator.
		GEOMITERATOR& operator++ (
			) { return (v_Increment()); }

		//! Get current position (0-1) or -1 if cannot determine
		double GetPosition (
			) const { return (v_GetPosition()); }

		//! Determine if done with iteration.
		bool IsDone (
			) const { return (v_IsDone()); }

		//! Restart to beginning (if supported);
		//! @return TRUE if successful, FALSE if not supported, < 0 if error
		int Restart (
			) { return (v_Restart()); }

	private:

		// Overridables.

		//! Get current position.
		//! Default implementation returns -1;
		virtual double v_GetPosition ();

		//! Pre-increment.
		virtual GEOMITERATOR& v_Increment () = 0;

		//! Determine if done.
		virtual bool v_IsDone () = 0;

		//! Restart to beginning.
		//! @return TRUE if successful, FALSE if not supported, < 0 if error
		//! Default returns FALSE.
		virtual ERRVALUE v_Restart ();
	};


//! Single point interator.
class GEOMITERATOR_POINT : public GEOMITERATOR {
	public:

		//! Dereference operator.
		const DPOINT3D& operator* (
			) const { return (m_Point); }

		//! Dereference operator.
		const DPOINT3D& operator-> (
			) const { return (&m_Point); }

	protected:

		DPOINT3D m_Point;
	};


//! Multi-point iterator.
class GEOMITERATOR_POINTLIST : public GEOMITERATOR {
	public:

		//! Dereference operator.
		const POLYLINE& operator* (
			) const { return (m_PointList); }

		//! Dereference operator.
		const POLYLINE& operator-> (
			) const { return (&m_PointList); }

	protected:

		POLYLINE m_PointList;
	};
