/******************************************************************************
 *
 * \file mi32/jobcontrol.h
 * \brief JOB_CONTROL
 *
 * \if NODOC
 * $Log: jobcontrol.h_v $
 * Revision 1.2  2008/11/07 21:12:17  bcolombe
 * *** empty log message ***
 *
 * Revision 1.1  2008/10/08 20:50:38  bcolombe
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_JOBCONTROL_H
#define INC_JOBCONTROL_H

#include <mi32/mithread.h>
#include <mi32/critsec.h>
#include <mi32/stddefns.h>
#include <mi32/milist.h>
#include <mi32/filepath.h>
#include <mi32/filepathlist.h>
#include <mi32/mistring.h>
#include <mi32/datetime.h>
#include <mi32/bitset.h>
#include <mi32/misystem.h>

namespace {
class MyJob;

class JOB_CONTROL {
	public:
		JOB_CONTROL ();
		JOB_CONTROL (MITHREAD::CRITICALSECTION* CritSec);
		virtual ~JOB_CONTROL();

		ERRVALUE AddJobs(const FILEPATH& path, bool clearold = false);
		void RunJobs();
		void StopJobs();
		void ChangePriority(const INT32& oldpriority, const INT32& newpriority, const FILEPATH& path);
		void SetNumRunningJobs(const INT32& NumJobsAllowed);
		void SetStartTime(const DATETIME& datetime) {m_starttime = datetime; m_checkstarttime = true;}
		void StopStartTime() {m_checkstarttime = false;}
		void SetEndTime(const DATETIME& datetime) {m_endtime = datetime; m_checkendtime = true;}
		void StopEndTime() {m_checkendtime = false;}
		void SetDayOfWeekRepeating(const BITSET& DayOfWeekSet) {m_DayOfWeekSet = DayOfWeekSet;}
		void SetNeedRedraw(bool redraw) {m_needredraw = redraw;}

	private:
		class MyJob {
			public:
				MyJob(MITHREAD::CRITICALSECTION* CritSec, const MISTRING& str, const FILEPATH& path);
				~MyJob() {}

				FILEPATH GetPath() {return m_path;}

				void SetDelegateOnRunning (
					FastDelegate<void(MyJob*, const FILEPATH&)> delegate
					) { m_DelegateOnRunning = delegate; }

				void SetDelegateOnCompletion (
					FastDelegate<void(MyJob*, const FILEPATH& , bool)> delegate
					) { m_DelegateOnCompletion = delegate; }

				void StartJob();

				bool HasLock(MyJob* job = 0);

				void SetInputList(const FILEPATHLIST& list) {m_input = list;}

				void SetOutputList(const FILEPATHLIST& list) {m_output = list;}

				const FILEPATHLIST& GetInput() {return m_input;}

				const FILEPATHLIST& GetOutput() {return m_output;}

			private:
				MITHREAD::CRITICALSECTION* m_CritSec;
				MISTRING m_str;
				FILEPATH m_path;
				INT32 m_priority;
				FastDelegate<void(MyJob*, const FILEPATH&)> m_DelegateOnRunning;
				FastDelegate<void(MyJob*, const FILEPATH& , bool)> m_DelegateOnCompletion;
				FILEPATHLIST m_input;
				FILEPATHLIST m_output;

				void RunJob();
				void EndJob(PTRUINT id, MISYSTEM::PROCESSRESULT result);
			};

		void GetNextJob();
		void JobRunning(MyJob*, const FILEPATH& path);
		void JobFinished(MyJob*, const FILEPATH& path, bool HasError);
		virtual void v_JobFinished(const FILEPATH& path, bool HasError);
		virtual void v_JobRunning(const FILEPATH& path);

		MITHREAD::CRITICALSECTION* m_CritSec;
		FILEPATHLIST m_joblist;

		//queue for each priority
		MILIST<MyJob*> m_queue1;
		MILIST<MyJob*> m_queue2;
		MILIST<MyJob*> m_queue3;
		MILIST<MyJob*> m_queue4;
		MILIST<MyJob*> m_queue5;
		MILIST<MyJob*> m_runqueue;
		MILIST<MyJob*> m_lockqueue;

		DATETIME m_starttime;
		DATETIME m_endtime;
		bool m_checkstarttime;
		bool m_checkendtime;
		BITSET m_DayOfWeekSet;

		INT32 m_NumJobs;
		INT32 m_NumJobsAllowed;
		bool m_StopJobs;
		bool m_needredraw;
	};

} // end of namespace

#endif		//  INC_JOBCONTROL_H


