/**
 * \file momentofinertia.h
 * \brief MOMENTOFINERTIA definitions
 *
 * \if NODOC
 * $Id: momentofinertia.h_v 1.1 2014/08/08 21:41:14 mju Exp $
 *
 * $Log: momentofinertia.h_v $
 * Revision 1.1  2014/08/08 21:41:14  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MOMENTOFINERTIA_H
#define	INC_MI32_MOMENTOFINERTIA_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class POLYLINE;
#endif

//=====================================================================================================================

//! Container for moment of inertia for an area defined by a POLYLINE.
class MOMENTOFINERTIA {
	public:

		//! Constructor.
		MOMENTOFINERTIA ();

		//! Destructor.
		~MOMENTOFINERTIA ();

		//! Add values for another area.
		MOMENTOFINERTIA& operator+= (
			const MOMENTOFINERTIA& rhs
			);

		//! Subtract values for another area.
		MOMENTOFINERTIA& operator-= (
			const MOMENTOFINERTIA& rhs
			);

		//! Compute for specified polygon.
		ERRVALUE Compute (
			const POLYLINE& polygon				//!< Polygon, must be closed and non-overlapping
			);

		//! Get area.
		const double GetArea () const
			{ return (m_area); }

		//! Get centroid.
		const DPOINT2D& GetCentroid () const
			{ return (m_centroid); }

		//! Get value (actual moment of inertia).
		double GetValue () const
			{ return (m_I); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DPOINT2D m_centroid;
		double m_area;
		double m_I;
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_MOMENTOFINERTIA_H
