/**
 * \file mi32/mxdialog.h
 * \brief Definitions for Mxd-style dialogs.
 *
 * \if NODOC
 *
 *		If moving prototypes out of this header, DO NOT have this header
 *		include the new header.
 *
 * $Id: mxdialog.h_v 1.54 2014/10/08 14:27:08 mju Exp $
 *
 * $Log: mxdialog.h_v $
 * Revision 1.54  2014/10/08 14:27:08  mju
 * Add prototype for xyplotAssignData using mistrings.
 *
 * Revision 1.53  2014/05/20 19:36:32  mju
 * Remove insprast dialog.
 *
 * Revision 1.52  2014/05/14 13:59:13  mju
 * Remove viewRastHist fns.
 *
 * Revision 1.51  2013/11/01 22:09:05  bcolombe
 * Add RoseChangeNumVars
 *
 * Revision 1.50  2013/01/23 16:27:03  mju
 * Remove more obsolete prototypes.
 *
 * Revision 1.49  2013/01/22 15:15:48  mju
 * Remove unused fns.
 *
 * Revision 1.48  2013/01/16 14:46:14  mju
 * Add mxdlgViewProfileAddObjItems.
 *
 * Revision 1.47  2013/01/15 21:52:31  mju
 * Change parms for dlgViewProfileOpen.
 *
 * Revision 1.46  2013/01/14 17:01:21  mju
 * Add mxdViewProfileHasObject, remove other viewProfile fns.
 *
 * Revision 1.45  2013/01/11 18:08:54  mju
 * Remove profileflag_InMapCoord as always required.
 * Change mxdViewProfileUpdate to use double_array.
 *
 * Revision 1.44  2013/01/10 21:56:54  mju
 * Remove unused mxdlgViewProfile fns.
 *
 * Revision 1.41  2007/09/19 17:55:38  mju
 * Add delegates for profile cursor tracking.
 *
 * Revision 1.18  2001/12/28 21:34:51  mju
 * Add MxEditColorMap using color array.
 *
 * Revision 1.1  1999/05/06  22:14:23  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MXDIALOG_H
#define	INC_MI32_MXDIALOG_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#ifndef INC_MI32_XDEFNS_H
	#include <mi32/xdefns.h>
#endif

#ifndef	INC_MI32_POINT_H
	#include <mi32/point.h>
#endif

#ifndef	INC_MI32_MCB_H
	#include <mi32/mcb.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
	#include <mi32/doublear.h>
#endif

#ifndef _XtIntrinsic_h
	#include <X11/Intrinsic.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class REGION2D;
class TRANS2D_MAPGEN;
namespace RVC {
	class OBJITEM;
	class OBJITEMLIST;
	}
#endif

//!\addtogroup Mxd Mxd-style dialog functions
//!@{

//=====================================================================================================================

struct MxdAnyCallbackStruct {
	UINT32 reason;							//!<  Callback reason 
	MXDHANDLE dhandle;					//!<  Dialog handle 
	};

typedef void (*MxdCallbackFunc)(MxdAnyCallbackStruct*,void*);

			/* Generic flags */
#define	MxdFLAG_SetLastPosition		0x80000000	//!<  Open dialog where it was last used (via MxSetWindowPosition())
#define	MxdFLAG_OpenOnCreate			0x40000000	//!<  Open dialog when created 
#define	MxdFLAG_DestroyOnClose		0x20000000	//!<  Destroy dialog when closed 
#define	MxdFLAG_Modal					0x10000000	//!<  Dialog is modal 
#define	MxdFLAG_ReturnWhenClose		0x08000000	//!<  Don't return from Mxd...Open() until dialog closed 
#define	MxdFLAG_NoClose				0x04000000	//!<  Don't allow dialog to be closed by user or Mxd...Close() 
#define	MxdFLAG_HaveApply				0x02000000	//!<  Include "Apply" button if appropriate 

			/* Dialog-specific flags */

			/* Flags for MxdTipOfDayCreate() */
#define	MxdTIPOFDAY_NoAutoOpen		0x00000001	//!<  Auto open on startup disabled 

			/* Callback reasons common to many dialogs */
#define	MxdCR_Destroy				0x80000000		//!<  Dialog is being destroyed 
#define	MxdCR_Close					0x40000000		//!<  Dialog has been closed 
#define	MxdCR_Open					0x20000000		//!<  Dialog has been opened 
#define	MxdCR_Resize				0x10000000		//!<  Dialog has been resized 
#define	MxdCR_Apply					0x08000000		//!<  User has pressed the "Apply" button 
#define	MxdCR_Modify				0x04000000		//!<  User has modified something in the dialog 
#define	MxdCR_ChangeBegin			0x02000000		//!<  User is about to change something 
#define	MxdCR_ChangeEnd			0x01000000		//!<  User has finished changing something 
#define	MxdCR_ChangeContinue		0x00800000		//!<  User is continuing to change something 
#define	MxdCR_SelectedObject		0x00400000		//!<  User selected or opened different object 
#define	MxdCR_SavedObject			0x00200000		//!<  User saved the object 


//=====================================================================================================================

//! Add callback to list.
int MxdAutoColorMapAddCallback (
	MXDHANDLE dhandle, 
	MxdCallbackFunc, 
	void *data, 
	UINT32 reason
	);

//! Create the dialog.
int MxdAutoColorMapCreate (
	Widget pwidget, 							//!< Parent widget
	MXDHANDLE *dhandle, 						//!< Dialog handle returned
	const RVC::OBJITEM& ObjItemRaster,
	COLOR *colmap,
	int NumColors,
	int dialogflags							//!< MxdFLAG_... flags
	);

//! Open the dialog.
int MxdAutoColorMapOpen (
	MXDHANDLE dhandle							//!< Dialog handle
	);

//! Close the dialog.
int MxdAutoColorMapClose (
	MXDHANDLE dhandle							//!< Dialog handle
	);

//! Destroy the dialog.
int MxdAutoColorMapDestroy (
	MXDHANDLE dhandle							//!< Dialog handle
	);

//! Return whether not dialog is open.
int MxdAutoColorMapIsOpen (
	MXDHANDLE dhandle							//!< Dialog handle
	);

//! Remove callback from list.
int MxdAutoColorMapRemoveCallback (
	MXDHANDLE dhandle, 
	MxdCallbackFunc, 
	void *data, 
	UINT32 reason
	);

//=====================================================================================================================

#define	MxdEDITCOLORMAP_LoadFromObject		0x00000001	//!<  Load color map from file/object provided 
#define	MxdEDITCOLORMAP_InitGrayscale			0x00000002	//!<  Initialize color map to linear grayscale 
#define	MxdEDITCOLORMAP_RetNewObject			0x00000004	//!<  Return new filename/inode if user does save-as 
#define	MxdEDITCOLORMAP_AllowTransparent		0x00000008	//!<  Allow colors to be set to "transparent" 
#define	MxdEDITCOLORMAP_AllowTransparency	0x00000010	//!<  Allow transparency factor to be set 

struct MxdEditColorMapCallbackStruct : McbAnyCallbackStruct {
	MXDHANDLE dhandle;					//!< Dialog handle 
	RVC::OBJITEM m_ObjItem;				//!< Location of object being saved to (for MxdCR_SavedObject only)

	MxdEditColorMapCallbackStruct (
		UINT32 reason,
		MXDHANDLE dhandle
		);
	};

//! Add callback for "color map editor" dialog.
//!
//! Flags:
//!		\li \b MxdFLAG_Modal							Make dialog modal
//!		\li \b MxdEDITCOLORMAP_LoadFromObject	Load color map from filename/inode provided if possible
//!		\li \b MxdEDITCOLORMAP_InitGrayscale		Initialize color map to linear grayscale
//!		\li \b MxdEDITCOLORMAP_RetNewObject		Return new filename/inode if user does save-as (modal only)
int MxdEditColorMapAddCallback (
	MXDHANDLE dhandle, 
	MxdCallbackFunc, 
	void *cbs, 
	UINT32 reasonmask
	);

//! Close the dialog.
int MxdEditColorMapClose (
	MXDHANDLE dhandle
	);

//! Create "color map editor" dialog.
//!
//! Flags:		
//!		\li \b MxdFLAG_Modal							Make dialog modal
//!		\li \b MxdEDITCOLORMAP_LoadFromObject	Load color map from filename/inode provided if possible
//!		\li \b MxdEDITCOLORMAP_InitGrayscale		Initialize color map to linear grayscale
//!		\li \b MxdEDITCOLORMAP_RetNewObject		Return new filename/inode if user does save-as (modal only)
int MxdEditColorMapCreate (
	Widget pwidget, 							//!< Parent widget for dialog
	MXDHANDLE *dhandle, 						//!< Dialog handle returned
	char *dialogname, 						//!< Dialog name for title ("color_palette_editor" if NULL)
	int relposn, 								//!< Position relative to parent's shell (0=none, -1=use previous)
	COLOR *colmap,								//!< Pointer to color palette, data at pointer will be modified by dialog
	int NumColors,								//!< Number of colors in palette
	RVC::OBJITEM& ObjItemPalette,			//!< Location of palette being edited, or set as "new" with parent if none existing
	UINT32 dialogflags						//!< MxdFLAG_OpenOnCreate, MxdFLAG_DestroyOnClose
	);

//! Destroy the dialog. 
int MxdEditColorMapDestroy (
	MXDHANDLE dhandle
	);

//! Return whether not dialog is open.
int MxdEditColorMapIsOpen (
	MXDHANDLE dhandle
	);

//! Open the dialog.
int MxdEditColorMapOpen (
	MXDHANDLE dhandle
	);

//! Remove callback from list.
int MxdEditColorMapRemoveCallback (
	MXDHANDLE dhandle, 
	MxdCallbackFunc, 
	void *cbs, 
	UINT32 reasonmask
	);

//! Update dialog for external changes to color palette.
int MxdEditColorMapUpdate (
	MXDHANDLE dhandle, 						//!< Dialog handle from MxdEditColorMapCreate()
	int notify									//!< Set if want callback list called with MxdCR_Modify
	);

//=====================================================================================================================

#define MxdSmartLine_StartLine		0x00000001	//!<  when start tracing new line 
#define MxdSmartLine_StopLine			0x00000002	//!<  when new line is complete 
#define MxdSmartLine_SaveLine			0x00000004	//!<  when new line is complete same as StopLine 
#define MxdSmartLine_AbortLine		0x00000008	//!<  if line started, but then error occured 
#define MxdSmartLine_NewPoint			0x00000010	//!<  when new point added to line 
#define MxdSmartLine_TraceBegin		0x00000020	//!<  when start tracing procedure 
#define MxdSmartLine_TraceEnd			0x00000040	//!<  when stop tracing procedure 
#define MxdSmartLine_StopReason		0x00000080	//!<  when stop tracing procedure 

struct MxdSmartLineCallbackStruct {
	UINT32 reason;
	MXDHANDLE dhandle;
	INT32 ElemNum;
	LPOINT3D *Line;
	INT32 NumPoints;
	int ElemType;
	};

//! Activate dialog tool.
int MxdSmartLineActivate (
	MXDHANDLE handle
	);

//! Add callback to the dialog.
int MxdSmartLineAddCallback (
	MXDHANDLE handle,													//!< Dialog handle
	void (*func)(MxdSmartLineCallbackStruct *cb, void*),	//! Callback function
	void *udata,														//!< User data to pass into callback
	UINT32 reason														//!< Reason for callback
	);

//! Close the dialog.
int MxdSmartLineClose (
	MXDHANDLE dhandle
	);

//! Create Smart Line Following Dialog.
int MxdSmartLineCreate (
	Widget pwidget,							//!< Parent widget
	MXDHANDLE *handle, 						//!< Handle returned
	char *dialogname, 						//!< Dialog name override 
	int relposn,								//!< Position relative to parent's shell (0=none, -1=use previous)
	void *vview, 								//!< View to operate on
	void *vlp, 									//!< Layer to operate on
	UINT32 dialogflags						//!< Dialog flags
	);

//! Destroy the dialog.
int MxdSmartLineDestroy (
	MXDHANDLE dhandle
	);

//! Return whether not dialog is open.
int MxdSmartLineIsOpen (
	MXDHANDLE dhandle
	);

//! Start a new view.
int MxdSmartLineNewView (
	MXDHANDLE dhandle
	);

//! Open the dialog.
int MxdSmartLineOpen (
	MXDHANDLE dhandle
	);

//! Remove callback from the dialog. 
void MxdSmartLineRemoveCallback (
	MXDHANDLE dhandle, 
	void (*func)(MxdSmartLineCallbackStruct *cb, void*), 
	void *data, 
	UINT32 reason
	);

//! Resume dialog tool.
int MxdSmartLineResume (
	MXDHANDLE handle
	);

//! Suspend dialog tool.
int MxdSmartLineSuspend (
	MXDHANDLE handle
	);

//=====================================================================================================================

//! Add callback for "tip of day" dialog. 
int MxdTipOfDayAddCallback (
	MXDHANDLE dhandle, 
	MxdCallbackFunc, 
	void *data, 
	UINT32 reason
	);

//! Close previously opened "tip of day" dialog.
int MxdTipOfDayClose (
	MXDHANDLE dhandle
	);

//! Create "tip of day" dialog.
//!
//! Flags:		
//!		\li \b MxdTIPOFDAY_NoAutoOpen	Do not perform automatic opening
int MxdTipOfDayCreate (
	Widget pwidget, 							//!< Parent widget for dialog
	MXDHANDLE *dhandle, 						//!< Dialog handle returned
	char *dialogname, 						//!< Dialog name ("mxInspRast" if NULL)
	int relposn, 								//!< Position relative to parent's shell (0=none, -1=use previous)
	UINT32 dialogflags						//!< MxdFLAG_OpenOnCreate, MxdFLAG_DestroyOnClose
	);

//! Destroy previously created "tip of day" dialog.
int MxdTipOfDayDestroy (
	MXDHANDLE dhandle
	);

//! Return true if "tip of day" dialog is open, false if not.
int MxdTipOfDayIsOpen (
	MXDHANDLE dhandle
	);

//! Open previously created "tip of day" dialog.
int MxdTipOfDayOpen (
	MXDHANDLE dhandle
	);

//! Remove callback for "tip of day" dialog.
int MxdTipOfDayRemoveCallback (
	MXDHANDLE dhandle, 
	MxdCallbackFunc, 
	void *data, 
	UINT32 reason
	);

//=====================================================================================================================

//! Add objects to profile.
ERRVALUE MxdViewProfileAddObjItems (
	MXDHANDLE dhandle,
	const RVC::OBJITEMLIST& ObjItemList
	);

//! Close the dialog.
int MxdViewProfileClose (
	MXDHANDLE dhandle
	);

//! Create the dialog.
int MxdViewProfileCreate (
	Widget parent, 							//!< Parent widget
	MXDHANDLE *dhandle, 						//!< Dialog handle returned
	char *dialogname, 						//!< Dialog name override
	MxRELPOSN relposn,						//!< Relative position for dialog to appear
	const RVC::OBJITEMLIST *pObjItemList,			//!< Optional object location
	int dialogflags							//!< MxdFLAG_... flags
	);

//! Destroy the dialog. 
int MxdViewProfileDestroy (
	MXDHANDLE dhandle
	);

//! Determine if specified object is shown in profile view.
bool MxdViewProfileHasObject (
	MXDHANDLE dhandle,
	const RVC::OBJITEM& objitem
	);

//! Open the dialog.
int MxdViewProfileOpen (
	MXDHANDLE dhandle
	);

void MxdViewProfileSetDelegateOnCursorLeave (
	MXDHANDLE dhandle,
	DELEGATE_VOID_NOPARMS delegate
	);
	
void MxdViewProfileSetDelegateOnCursorMotion (
	MXDHANDLE dhandle,
	FastDelegate<void(const DPOINT2D&, const SPATREF::COORDREFSYS&)> delegate
	);
	
//! Set function to call when dialog destroyed.
int MxdViewProfileSetDestroyCallback (
	MXDHANDLE dhandle, 
	void(*)(MXDHANDLE,void*), 
	void *destroydata
	);

enum PROFILEFLAGS {
	PROFILEFLAG_None =					0x00,
	PROFILEFLAG_SeparatePoints =		0x02
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(PROFILEFLAGS)
#endif

//! Notify dialog that area selected for profile has changed. 
int MxdViewProfileUpdate (	
	MXDHANDLE dhandle, 
	const DOUBLE_ARRAY<DPOINT2D>& MapPoints,
	const SPATREF::COORDREFSYS& CoordRefSys,
	PROFILEFLAGS flags
	);

//=====================================================================================================================

//! Change number of variables.  Old data destroyed.
int MxdViewRoseChangeNumVars (
	MXDHANDLE dhandle, 						//!< Dialog handle
	int NumVars 							//!< Number of variables/dimensions 
	);

//! Assign or change variable in rose diagram display.
int MxdViewRoseAddVariable (
	MXDHANDLE dhandle, 						//!< Dialog handle
	int Num, 									//!< Variable number, started from 0
	const MIUNICODE *name,  							//!< Variable name to be displayed (lenght <= 25 char)
	double *Value,  							//!< Address of variable to be assigned 
	double Min, 								//!< Minimum value of this variable
	double Max, 								//!< Maximum value of this variable
	double SelPos								//!< Reserved for future use, any value
	);

//! Add variable using direction.
int MxdViewRoseAddVariableAngle (
	MXDHANDLE dhandle,  
	double Angle, 
	MIUNICODE *name,  
	double *Value,  
	double Min, 
	double Max, 
	double SelPos
	);

//! Close previously opened "rose diagram" dialog. 
int MxdViewRoseClose (
	MXDHANDLE dhandle
	);

//! Create rose diagram dialog.
int MxdViewRoseCreate (
	Widget pwidget, 							//!< Parent widget 
	MXDHANDLE *dhandle, 						//!< Dialog handle returned 
	char *dialogname, 						//!< Dialog name override 
	int relposn, 								//!< Position relative to parent's shell (0=none, -1=use previous)
	int NumVars, 								//!< Number of variables/dimensions 
	int dialogflags							//!< MxdFLAG_... flags 
	);

//! Destroy previously created "rose diagram" dialog.
int MxdViewRoseDestroy (
	MXDHANDLE dhandle
	);

//! Return true if "rose diagram" dialog is open, false if not.
int MxdViewRoseIsOpen (
	MXDHANDLE dhandle
	);

//! Open previously created "rose diagram" dialog.
int MxdViewRoseOpen (
	MXDHANDLE dhandle
	);

//! Function for future use/ Get scale bar position for specified variable.
int MxdViewRoseScaleBar (
	MXDHANDLE dhandle, 						//!< Dialog handle
	int Num, 									//!< Variable number to get scalebar position
	double *Var									//!< Scalebar position returned
	);

//! Set selection callback function.
int MxdViewRoseSetSelectCallback (
	MXDHANDLE dhandle, 
	void(*)(int,  void*), 
	void *data
	);

//! Update rose diagram display / redraw diagram.
int MxdViewRoseUpdate (
	MXDHANDLE dhandle
	);

//! Set function to call when dialog closed.
int MxdViewRoseSetCloseCallback (
	MXDHANDLE dhandle, 
	void(*)(void*,void*), 
	void *closedata
	);

//! Set function to call when dialog destroyed.
int MxdViewRoseSetDestroyCallback (
	MXDHANDLE dhandle, 
	void(*)(void*,void*), 
	void *destroydata
	);

//=====================================================================================================================

//! Assign a callback.
int MxdViewXYPlotAddCallback (
	MXDHANDLE dhandle, 
	MxdCallbackFunc, 
	void *cbdata, 
	UINT32 reasonmask
	);

//! Assign XY data for dialog and provide name of axises.
//!
//! Note 1: If programmer pass NULL for X or Y axis, the default
//!		names will be displayed in dialog: "X" and "Y".
//!
//!	Note 2: Dialog creates own copy of "Points" and you can do
//!		what you want with it, it doesn't affect assigned data.
int MxdViewXYPlotAssignData (
	MXDHANDLE dhandle, 						//!< Dialog handle
	INT32 Num, 									//!< Number of points
	DPOINT2D *Points, 						//!< Array of points
	char *Xname, 								//!< Name of horizontal axis, can be NULL 
	char *Yname									//!< Name of vertical axis, can be NULL
	);

//! Assign XY data for dialog and provide name of axises.
//!
//! Note 1: If programmer pass NULL for X or Y axis, the default
//!		names will be displayed in dialog: "X" and "Y".
//!
//!	Note 2: Dialog creates own copy of "Points" and you can do
//!		what you want with it, it doesn't affect assigned data.
int MxdViewXYPlotAssignData (
	MXDHANDLE dhandle, 						//!< Dialog handle
	INT32 Num, 									//!< Number of points
	DPOINT2D *Points, 						//!< Array of points
	const MISTRING &XAxisName,				//!< Name for horizontal axis
	const MISTRING &YAxisName				//!< Name for vertical axis
	);

//! Close previously opened dialog.
int MxdViewXYPlotClose (
	MXDHANDLE dhandle
	);

//! Create XY-plot dialog.
int MxdViewXYPlotCreate (
	Widget parent, 							//!< Parent widget
	MXDHANDLE *dhandle,						//!< Dialog handle returned
	char *name, 								//!< Dialog name override
	int relposn, 								//!< Position relative to parent's shell (0=none, -1=use previous)
	int dialogflags							//!< MxdFLAG_OpenOnCreate, MxdFLAG_DestroyOnClose
	);

//! Destroy previously created dialog.
int MxdViewXYPlotDestroy (
	MXDHANDLE dhandle
	);

//! Get Motif FormDialog for X-Y plot.
Widget MxdViewXYPlotGetFormDialog (
	MXDHANDLE dhandle
	);

//! Return true if dialog is open.
int MxdViewXYPlotIsOpen (
	MXDHANDLE dhandle
	);

//! Open previously created dialog.
int MxdViewXYPlotOpen (
	MXDHANDLE dhandle
	);

//! Redraw dialog contents.
int MxdViewXYPlotUpdate (
	MXDHANDLE dhandle
	);

//=====================================================================================================================

//!@}

#endif	// INC_MI32_MXDIALOG_H 
