/**
 * \file mi32/mxmulticlick.h
 * \brief MXMULTICLICK class defns
 *
 * \if NODOC
 * $Id: mxmulticlick.h_v 1.2 2013/03/13 15:00:16 mju Exp $
 *
 * $Log: mxmulticlick.h_v $
 * Revision 1.2  2013/03/13 15:00:16  mju
 * Remove checkReset.
 *
 * Revision 1.1  2013/03/13 14:49:37  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MXMULTICLICK_H
#define	INC_MI32_MXMULTICLICK_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef	INC_MI32_XBASE_H
#include <mi32/xbase.h>
#endif

//! Helper to simplify multi-click handling.
class MXMULTICLICK {
	public:

		//! Constructor
		MXMULTICLICK ();

		//! Get multi-click count.
		//! The click count is incremented when the button is "pressed", not when it is released.
		//! Typically the multi-click action should be done on the second (or subsequent) button press.
		//! The count will be reset if the pointer is moved more than a small distance from the location
		//! of the first click or if a different button is pressed.
		UINT8 GetCount (
			const XEvent *xevent					//!< Event to check
			);

		//! Reset.
		void Reset (
			) { m_Count = 0; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 m_LastTime;
		int m_StartX;
		int m_StartY;
		UINT8 m_StartButton;
		UINT8 m_Count;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif	// INC_MI32_MXMULTICLICK_H
