/**
 * \file mi32/rasterfilterparms.h
 *
 * \if NODOC
 * $Id: rasterfilterparms.h_v 1.15 2016/12/19 18:02:58 mju Exp $
 *
 * $Log: rasterfilterparms.h_v $
 * Revision 1.15  2016/12/19 18:02:58  mju
 * Add majority, minimum, maximum types (nfr 24587).
 *
 * Revision 1.14  2016/12/07 17:20:56  mju
 * Add Scharr gradient filter.
 *
 * Revision 1.13  2016/01/14 23:57:44  bcolombe
 * Remove qualification on method
 *
 * Revision 1.12  2014/11/11 18:06:56  mju
 * Make createFilter methods const.
 *
 * Revision 1.11  2014/05/16 20:01:09  bcolombe
 * Get scale when do defaultmatrices
 *
 * Revision 1.10  2014/04/18 17:07:05  bcolombe
 * Add gaussian
 *
 * Revision 1.9  2014/04/07 22:53:17  bcolombe
 * Add GLCM
 *
 * Revision 1.8  2014/02/21 18:13:50  bcolombe
 * Remove threshold stuff
 *
 * Revision 1.7  2014/02/18 17:46:12  bcolombe
 * Add cb func to read old parms
 *
 * Revision 1.6  2014/02/17 22:17:49  bcolombe
 * Add user defined options
 *
 * Revision 1.5  2014/02/17 18:37:01  bcolombe
 * Additional params
 *
 * Revision 1.4  2014/02/15 18:26:45  bcolombe
 * Change to filter type flag
 *
 * Revision 1.3  2008/04/09 16:13:44  scowan
 * Fixed unix warnings.
 *
 * Revision 1.2  2007/12/27 18:39:44  bcolombe
 * Added CreateFilter methods
 *
 * Revision 1.1  2007/11/19 22:03:37  bcolombe
 * Initial revision
 *
 *
 * \endif
****************************************************************************/

#ifndef	INC_MI32_RASTERFILTERPARMS_H
#define	INC_MI32_RASTERFILTERPARMS_H

#ifndef INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#ifndef INC_SPATMOD_IMAGESTAGEAUTOPTR_H
#include <spatmod/imagestageautoptr.h>
#endif

#ifndef INC_SPATMOD_IMAGESTAG_H
#include <spatmod/imagestage.h>
#endif

#ifndef  INC_MI32_SERIALIZERBASE_H
#include <mi32/serializeritem.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class SERIALIZER;
class MIMATRIX;
#endif

//! Class to hold settings for raster spatial filter
class RASTER_FILTER_PARMS {
	public:
		
		enum TYPE {
			TYPE_Sharpening = 1,
			TYPE_HighPass,
			TYPE_HighBoost,
			TYPE_Volterra,
			TYPE_LocalContrast,
			TYPE_LUM,
			TYPE_CS,
			TYPE_WMMR,
			TYPE_SharpeningUserDefined,
			TYPE_SharpeningCount,

			TYPE_EdgeDetection = 100,
			TYPE_StdDev,
			TYPE_Range,
			TYPE_Teager,
			TYPE_Laplacian,
			TYPE_LaplacianDiagonals,
			TYPE_GradientSobel,
			TYPE_GradientRoberts,
			TYPE_GradientPrewitt,
			TYPE_GradientScharr,
			TYPE_GradientUserDefined,
			TYPE_EdgeDetectionUserDefined,
			TYPE_EdgeDetectionCount,

			TYPE_Smoothing = 200,
			TYPE_LowPass,
			TYPE_Median,
			TYPE_Mode,
			TYPE_Majority,
			TYPE_Minimum,
			TYPE_Maximum,
			TYPE_Olympic,
			TYPE_MLM,
			TYPE_PMedian,
			TYPE_AMPM,
			TYPE_Gaussian,
			TYPE_SmoothingUserDefined,
			TYPE_SmoothingCount,

			TYPE_Radar = 300,
			TYPE_Sigma,
			TYPE_Frost,
			TYPE_Lee,
			TYPE_Kuan,
			TYPE_RadarCount,

			TYPE_GLCM = 400,
			TYPE_GLCM_Mean,
			TYPE_GLCM_Contrast,
			TYPE_GLCM_Dissimilarity,
			TYPE_GLCM_Homogeneity,
			TYPE_GLCM_ASM,
			TYPE_GLCM_Entropy,
			TYPE_GLCM_Variance,
			TYPE_GLCM_Correlation,
			TYPE_GLCMCount
			};

		//! Default Constructor
		RASTER_FILTER_PARMS (
			);

		//! Destructor
		~RASTER_FILTER_PARMS (
			);

		//! Return amount of filter to be applied
		double GetAmount() const {return m_Amount;}

		//! Set amount of filter to be applied
		void SetAmount(double Amount) {m_Amount = Amount;}

		//! Get kernel A
		void GetMatrixA(MIMATRIX& Matrix) const;

		//! Set kernel A
		void SetMatrixA(const MIMATRIX& MatrixA);

		//! Get kernel B
		void GetMatrixB(MIMATRIX& Matrix) const;

		//! Set kernel B
		void SetMatrixB(const MIMATRIX& MatrixB);

		//! Get row size of kernel
		UINT32 GetNumRows() const {return m_NumRows;}

		//! Set row size of kernel
		void SetNumRows(UINT32 NumRows) {m_NumRows = NumRows;}

		//! Get column size of kernel
		UINT32 GetNumColumns() const {return m_NumColumns;}

		//! Set column size of kernel
		void SetNumColumns(UINT32 NumColumns) {m_NumColumns = NumColumns;}

		//! Get number of decimal places to show in kernel
		UINT32 GetNumPlaces() const {return m_NumPlaces;}

		//! Set number of decimal places to show in kernel
		void SetNumPlaces(UINT32 NumPlaces) {m_NumPlaces = NumPlaces;}

		//! Get filter type
		TYPE GetFilterType() const {return m_FilterType;}

		//! Set filter type
		void SetFilterType(TYPE FilterType) {m_FilterType = FilterType;}

		//! Get value of slider A
		double GetParamA() const {return m_ParamA;}

		//! Set value of slider A
		void SetParamA(double ParamA) {m_ParamA = ParamA;}

		//! Get value of slider B
		double GetParamB() const {return m_ParamB;}

		//! Set value of slider B
		void SetParamB(double ParamB) {m_ParamB = ParamB;}

		//! Get whether filter is set to be applied
		bool IsEnabled() const {return m_IsEnabled;}

		//! Set wheter filter is to be applied
		void SetEnabled (
			bool bIsEnabled
			) { m_IsEnabled = bIsEnabled; }

		//! Get boost
		double GetBoost() const {return m_Boost;}

		//! Set boost
		void SetBoost(double Boost) {m_Boost = Boost;}

		//! Get offset
		double GetOffset() const {return m_Offset;}

		//! Set offset
		void SetOffset(double Offset) {m_Offset = Offset;}

		//! Get scale
		double GetScale() const {return m_Scale;}

		//! Set scale
		void SetScale(double Scale) {m_Scale = Scale;}

		//! Setup default matrices
		static void GetDefaultMatrices(TYPE FilterType, INT32 Rows, INT32 Columns, MIMATRIX& MatrixA, MIMATRIX& MatrixB, double& Scale, double Param = 0.0);

		//! Get item definition array for serialization.
		static const SERIALIZERITEM* SerialGetItemDef ();

		//! Read the old raster filter parms and convert to new
		static ERRVALUE SerializerCB_ReadOldParms (
			SERIALIZER& serializer,
			const SERIALIZERITEM *itemdef,
			void *vpDispParm,
			SERIALIZERITEM::ACTION action
			);

		//! Return filter_focal stage based on source stage and current parms settings
		ERRVALUE CreateFilter (
			SPATMOD::IMAGE::STAGE *pSourceImage, 
			bool TakeOwnershipSource, 
			SPATMOD::IMAGE::STAGE_AUTOPTR &PtrStage
			) const;

		//! Return filter_focal stage based on source stage, mask image and current parms settings
		ERRVALUE CreateFilter (
			SPATMOD::IMAGE::STAGE *pSourceImage, 
			bool TakeOwnershipSource, 
			SPATMOD::IMAGE::STAGE *pMaskImage, 
			bool TakeOwnershipMask, 
			SPATMOD::IMAGE::STAGE_AUTOPTR &PtrStage
			) const;

	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef RASTER_FILTER_PARMS THISCLASS;

		TYPE m_FilterType; //filter type

		DOUBLE_ARRAY<double> m_ArrayA; //store kernel A as DOUBLE_ARRAY for serialization
		DOUBLE_ARRAY<double> m_ArrayB; //store kernel B as DOUBLE_ARRAY for serialization

		UINT32 m_NumRows; //Number of rows in kernel
		UINT32 m_NumColumns; //Number of columns in kernel
		double m_ParamA; //Value of parameter A
		double m_ParamB; //Value of parameter B
		double m_Amount; //Amount of filter to be applied
		bool m_IsEnabled; //Sets whether filtering is on
		UINT32 m_NumPlaces; //Number of decimal places to show in kernel

		double m_Boost;
		double m_Offset;
		double m_Scale;

		#endif
	};

#endif //INC_MGUI_FORMFILTRAST_H

