/**
 * \file mi32/rvcdbase.h
 *
 * \if NODOC
 * $Id: rvcdbase.h_v 1.402 2015/03/09 15:18:58 mju Exp $
 *
 * $Log: rvcdbase.h_v $
 * Revision 1.402  2015/03/09 15:18:58  mju
 * Change labels to use -1 for internal table like other element types.
 *
 * Revision 1.401  2015/03/06 22:15:16  mju
 * Adjust tablenums for vectorlabel.
 *
 * Revision 1.400  2015/03/05 18:18:06  mju
 * Add tablenum_BuiltIn values.
 *
 * Revision 1.397  2009/01/20 16:10:36  mju
 * Add safield_multi_polygon values.
 *
 * Revision 1.377  2007/01/17 23:43:11  dwilliss
 * added some DBCONV types
 *
 * Revision 1.365  2006/09/26 19:45:38  dwilliss
 * made the attach #defines match the DBTABLE::ATTACHMENT enum
 *
 * Revision 1.361  2006/08/09 16:07:43  dwilliss
 * added a new CAPABILITY enum value
 *
 * Revision 1.350  2006/03/29 16:24:33  scowan
 * \endif
**/

#ifndef INC_MI32_RVCDBASE_H
#define INC_MI32_RVCDBASE_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

//=====================================================================================================================

//! Structure used to maintain list of element to record attachments.
struct DBLISTELMT {
	INT32 table;								//!< Table
	INT32 record;								//!< Record
	// Size of this structure must not change
	};

inline bool operator== (					
	const DBLISTELMT& lhs, 
	const DBLISTELMT& rhs
	) { return (lhs.table == rhs.table && lhs.record == rhs.record); }

//=====================================================================================================================


//! Pseudo fields for DBFIELDINFO.next_field.  Some database schemas
//! require a 2 part key where the primary key is made up of 2 fields
//! but the foreign key is made up of 1 field and the name of the field
//! or one field and a "domid" which is looked up from a 3rd table by
//! using the table/field name.  We look those up on import and record
//! them in the DBFIELDINFO.domid.  I don't know of anything that would
//! use the table name, but I added it for completeness 
#define	FIELDNUM_ThisFieldName	0xFFFF	//!< Name of this field is the key data 
#define	FIELDNUM_ThisTableName	0xFFFE	//!< Name of this table is the key data 
#define	FIELDNUM_DomID				0xFFFD	//!< Key data is the "domid" field of this field 
#define	FIELDNUM_PseudoField		0xFFFD	//!< If next_field is >= this, it's one of these 


//=====================================================================================================================
//		Predefined and special table numbers

#define TABLE_Internal		-1				//!< The "Built-in" table
#define TABLE_InternalNode	-2				//!< Internal table number for vector nodes
#define TABLE_New				-9				//!< New table not yet created
#define TABLE_All				-99			//!< All tables
#define TABLE_Invalid		-98			//!< Internal use only

#define TABLENUM_Internal	-1

#define TABLENUM_BuiltIn_TIN_Node		0
#define TABLENUM_BuiltIn_TIN_Edge		1
#define TABLENUM_BuiltIn_TIN_Triangle	2
#define TABLENUM_BuiltIn_TIN_Hull		3

#define TABLENUM_BuiltIn_VectorLabel_Style	0

//=====================================================================================================================
//		Special record numbers

#define RECORD_New			-1

//=====================================================================================================================
//		Definitions for style-assignment tables

#define	SAFIELD_SINGLE_RelField			0
#define	SAFIELD_SINGLE_StyleName		1
#define	SAFIELD_SINGLE_StyleIndex		2
#define	SAFIELD_SINGLE_DrawFlag			3

#define	SAFIELD_MULTI_RelField				0
#define	SAFIELD_MULTI_DrawFlag				1
#define	SAFIELD_MULTI_PointStyleName		2
#define	SAFIELD_MULTI_PointStyleIndex		3
#define	SAFIELD_MULTI_LineStyleName		4
#define	SAFIELD_MULTI_LineStyleIndex		5
#define	SAFIELD_MULTI_PolygonStyleName	6
#define	SAFIELD_MULTI_PolygonStyleIndex	7
#define	SAFIELD_MULTI_PolyStyleName		SAFIELD_MULTI_PolygonStyleName
#define	SAFIELD_MULTI_PolyStyleIndex		SAFIELD_MULTI_PolygonStyleIndex
#define	SAFIELD_MULTI_TextStyleName		8
#define	SAFIELD_MULTI_TextStyleIndex		9

#define	SAFIELD_CAD_RelField				SAFIELD_MULTI_RelField
#define	SAFIELD_CAD_DrawFlag				SAFIELD_MULTI_DrawFlag
#define	SAFIELD_CAD_PointStyleName		SAFIELD_MULTI_PointStyleName
#define	SAFIELD_CAD_PointStyleIndex	SAFIELD_MULTI_PointStyleIndex
#define	SAFIELD_CAD_LineStyleName		SAFIELD_MULTI_LineStyleName
#define	SAFIELD_CAD_LineStyleIndex		SAFIELD_MULTI_LineStyleIndex
#define	SAFIELD_CAD_PolyStyleName		SAFIELD_MULTI_PolygonStyleName
#define	SAFIELD_CAD_PolyStyleIndex		SAFIELD_MULTI_PolygonStyleIndex
#define	SAFIELD_CAD_TextStyleName		SAFIELD_MULTI_TextStyleName
#define	SAFIELD_CAD_TextStyleIndex		SAFIELD_MULTI_TextStyleIndex

//=====================================================================================================================
//	Definitions for scale-selection tables

#define	SCALESELFIELD_RelField			0
#define	SCALESELFIELD_MinScale			1
#define	SCALESELFIELD_MaxScale			2

//=====================================================================================================================

#endif	// INC_MI32_RVCDBASE_H 
