/******************************************************************
* \file mi32/sensordata.h
* \brief Sensor data lookup
*
* \if NODOC
*
* $Id: sensordata.h_v 1.5 2015/12/07 20:48:30 bcolombe Exp $
*
* $Log: sensordata.h_v $
* Revision 1.5  2015/12/07 20:48:30  bcolombe
* Add band number from import metadata
*
* Revision 1.4  2015/11/20 23:22:07  bcolombe
* Add sun direction
*
* Revision 1.3  2015/10/19 16:05:25  bcolombe
* Add wavelength
*
* Revision 1.2  2015/08/31 19:34:14  bcolombe
* Changes to find metadata and detect bands
*
* Revision 1.1  2015/08/17 17:07:10  bcolombe
* Initial revision
*
* \endif
*******************************************************************/

#ifndef INC_MI32_SENSORDATA_H
#define INC_MI32_SENSORDATA_H

#ifndef INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif

#ifndef INC_MI32_DATETIME_H
#include <mi32/datetime.h>
#endif

#ifndef INC_MI32_XML_H
#include <mi32/xml.h>
#endif

#ifndef INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

class SENSORDATA {
public:

	class BANDDATA {
		public:
			BANDDATA();
			~BANDDATA();

			void Clear();

			ERRVALUE FindBandData (
				const SENSORDATA& sensordata,		//!< Sensor data
				int bandnum,						//!< bandnum if known, -1 if not
				const RVC::OBJITEM& objitem,		//!< Objitem to raster
				const FILEPATH& OriginalFilePath	//!< Provide original file path of linked file if created temporary link in order to separate bands.  Otherwise, same filepath as objitem
				);

			const MISTRING& GetName (
				) const { return m_Name; }

			const MISTRING& GetMICode (
				) const { return m_MICode; }

			const MISTRING& GetBandID (
				) const { return m_BandID; }

			const MISTRING& GetFileName (
				) const { return m_FileName; }

			double GetBandwidth (
				) const { return m_Bandwidth; }

			double GetIrradiance (
				) const { return m_Irradiance; }

			double GetOffset (
				) const { return m_Offset; }

			double GetScale (
				) const { return m_Scale; }

			double GetReflectanceOffset (
				) const { return m_ReflectanceOffset; }

			double GetReflectanceScale (
				) const { return m_ReflectanceScale; }

			double GetLMin (
				) const { return m_LMin; }

			double GetLMax (
				) const { return m_LMax; }

			double GetDNMin (
				) const { return m_DNMin; }

			double GetDNMax (
				) const { return m_DNMax; }

			double GetWavelength (
				) const { return m_Wavelength; }

			void SetBandwidth (
				double Bandwidth
				) { m_Bandwidth = Bandwidth; }

			void SetIrradiance (
				double Irradiance
				) { m_Irradiance = Irradiance; }

			void SetOffset (
				double Offset
				) { m_Offset = Offset; }

			void SetScale (
				double Scale
				) { m_Scale = Scale; }

			void SetReflectanceOffset (
				double Offset
				) { m_ReflectanceOffset = Offset; }

			void SetReflectanceScale (
				double Scale
				) { m_ReflectanceScale = Scale; }

			void SetLMin (
				double LMin
				) { m_LMin = LMin; }

			void SetLMax (
				double LMax
				) { m_LMax = LMax; }

			void SetDNMin (
				double DNMin
				) { m_DNMin = DNMin; }

			void SetDNMax (
				double DNMax
				) { m_DNMax = DNMax; }

			void SetWavelength (
				double Wavelength
				) { m_Wavelength = Wavelength; }

		private:
			friend class SENSORDATA;
			MISTRING m_Name;
			MISTRING m_MICode;
			MISTRING m_BandID;
			MISTRING m_FileName;
			double m_Irradiance;
			double m_Offset;
			double m_Scale;
			double m_ReflectanceOffset;
			double m_ReflectanceScale;
			double m_Bandwidth;
			double m_LMin;
			double m_LMax;
			double m_DNMin;
			double m_DNMax;
			double m_Wavelength;
		};

	// value here matters, matches sensor id number in sensordata.xml
	enum SENSOR {
		SENSOR_Unknown					= 0,
		SENSOR_DigitalGlobe_WorldView2	= 1,
		SENSOR_DigitalGlobe_WorldView3	= 2,
		SENSOR_DigitalGlobe_QuickBird2	= 3,
		SENSOR_DigitalGlobe_GeoEye1		= 4,
		SENSOR_RapidEye					= 5,
		SENSOR_SPOT6					= 6,
		SENSOR_SPOT7					= 7,
		SENSOR_Pleiades					= 8,
		SENSOR_LANDSAT8					= 9,
		SENSOR_LANDSAT7					= 10,
		SENSOR_LANDSAT5					= 11,
		SENSOR_LANDSAT4					= 12,
		SENSOR_IKONOS					= 13,
		SENSOR_ASTER					= 14,
		SENSOR_UKDMC					= 15,
		SENSOR_UKDMC2					= 16,
		SENSOR_KOMPSAT2					= 17,
		SENSOR_KOMPSAT3					= 18,
		SENSOR_ALOSAVNIR2				= 19
		};

	SENSORDATA (
		);

	~SENSORDATA (
		);

	void Clear (
		);

	ERRVALUE FindSensorData (
		SENSOR Sensor
		);

	ERRVALUE FindSensorData (
		const MISTRING& text
		);

	ERRVALUE FindSensorData (
		const RVC::OBJITEM& RasterObjItem,	//!< Objitem to raster
		const FILEPATH& OriginalFilePath	//!< Provide original file path of linked file if created temporary link in order to separate bands.  Otherwise, same filepath as objitem
		);

	ERRVALUE FindSensorData (
		const FILEPATH& MetadataFile
		);

	void GetBandList (
		MILIST<BANDDATA>& BandList
		) const;

	INT32 GetBandNum (
		) const { return m_BandNum; }

	const MISTRING& GetMetadataText (
		) const { return m_MetaDataText; }

	static void GetSensorList (
		const XMLDOC& SensorDataDoc,
		MILIST<SENSORDATA>& SensorList
		);

	static void GetSensorList (
		MILIST<SENSORDATA>& SensorList
		);

	SENSOR GetSensor (
		) const { return m_Sensor; }

	const MISTRING& GetName (
		) const { return m_Name; }

	const DATETIME& GetDateTime (
		) const { return m_DateTime; }

	double GetSunAzimuthAngle (
		) const { return m_SunAzimuthAngle; }

	double GetSunElevationAngle (
		) const { return m_SunElevationAngle; }

	void SetDateTime (
		const DATETIME& datetime
		) { m_DateTime = datetime; }

	void SetSunAzimuthAngle (
		double SunAzimuthAngle
		) { m_SunAzimuthAngle = SunAzimuthAngle; }

	void SetSunElevationAngle (
		double SunElevationAngle
		) { m_SunElevationAngle = SunElevationAngle; }

private:
	#ifndef GENERATING_DOXYGEN_OUTPUT
	
	XMLDOC m_MetaDataDoc;
	MISTRING m_MetaDataText;
	XMLNODE* m_SensorNode;
	SENSOR m_Sensor;
	MISTRING m_Name;
	DATETIME m_DateTime;
	double m_SunAzimuthAngle;
	double m_SunElevationAngle;
	INT32 m_BandNum;

	bool FindNode(XMLNODE* sensornode, const char* name, MISTRING& pathtext, MISTRING& valuetext) const;
	ERRVALUE OpenSensorDoc();
	void SetSensor();

	#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif
