/**
 * \file mi32/voronoi2d.h
 * \brief VORONOI2D class definition
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
 * \endif
**/

#ifndef	INC_MI32_VORONOI2D_H
#define	INC_MI32_VORONOI2D_H

#ifndef	INC_MI32_POINT_H
#include	<mi32/point.h>
#endif

//=====================================================================================================================

class VORONOI2D {
	public:

		class EDGE {
			public:

				EDGE ();

				INT32 GetNodeNumBegin (
					) const { return (m_NodeNumBegin); }

				INT32 GetNodeNumEnd (
					) const { return (m_NodeNumEnd); }

				INT32 GetPolygonNumLeft (
					) const { return (m_PolyNumLeft); }

				INT32 GetPolygonNumRight (
					) const { return (m_PolyNumRight); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				INT32 m_NodeNumBegin;
				INT32 m_NodeNumEnd;
				INT32 m_PolyNumLeft;
				INT32 m_PolyNumRight;
				friend class VORONOI2D;
				#endif
			};

		//! Constructor.
		VORONOI2D ();

		//! Destructor.
		~VORONOI2D ();

		//! Compute voronoi graph from points
		ERRVALUE Compute (
			const DPOINT2D* points,
			INT32 NumPoints,
			double extfactor = 0.1
			);

		const EDGE& GetEdge (
			INT32 EdgeNum
			) const;

		const SIMPLE_ARRAY<INT32>& GetNodeEdgeList (
			INT32 NodeNum
			) const;

		const DPOINT2D& GetNodePoint (
			INT32 NodeNum
			) const;

		INT32 GetNumEdges (
			) const;

		INT32 GetNumNodes (
			) const;

		INT32 GetNumPolygons (
			) const;

		const SIMPLE_ARRAY<INT32>& GetPolygonNodeList (
			INT32 PolygonNum
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_VORONOI2D_H
