/**
 * \file rvc/cameracalib.h 
 * \brief interface for the RVC::CAMERACALIB class.
 *
 * \if NODOC
 * $Id: cameracalib.h_v 1.1 2017/02/24 14:52:55 mju Exp $
 *
 * $Log: cameracalib.h_v $
 * Revision 1.1  2017/02/24 14:52:55  mju
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_CAMERACALIB_H
#define INC_RVC_CAMERACALIB_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_POINT_H
	#include <mi32/point.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class CAMERAINTRINSICS;
class CAMERA3D;
#endif

namespace RVC {

//=====================================================================================================================

//! Container for camera intrinsic and extrinsic values.
//! Object Type: OBJTYPE_CameraCalib
//! Parent Type: Georeference
class CAMERACALIB : public RVC::SIMPLE {
	public:

		CAMERACALIB ();

		CAMERACALIB (
			const RVC::CAMERACALIB &rhs					
			);

		virtual ~CAMERACALIB ();

		RVC::CAMERACALIB& operator= (
			const RVC::CAMERACALIB &rhs
			);

		void GetValues (
			CAMERAINTRINSICS &CameraIntrinsics,
			CAMERA3D &Camera3D
			) const;

		void SetValues (
			const CAMERAINTRINSICS &CameraIntrinsics,
			const CAMERA3D &Camera3D
			);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		void Copy (const CAMERACALIB &rhs);

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT &parent, DESCRIPTOR &Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS &ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS &ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS &ObjOpenParms);

		LPOINT2D m_ImageSize;
		DPOINT2D m_PrincipalPoint;
		DPOINT2D m_PixelSizeMM;
		DOUBLE m_FocalLengthMM;
		DOUBLE m_FocalLengthPixels;
		DOUBLE m_Skew;
		DOUBLE m_DistortionCoeff[15];
		DPOINT3D m_Rotation;
		DPOINT3D m_Position;
		MISTRING m_CameraMake;
		MISTRING m_CameraModel;
		MISTRING m_CameraSerialNumber;
		MISTRING m_LensMake;
		MISTRING m_LensModel;
		MISTRING m_LensSerialNumber;

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}		//! End of RVC namespace

#endif		// INC_RVC_CAMERACALIB_H
