/**
 * \file rvc/cmap.h 
 * \brief interface for the RVC::COLORMAP class.
 *
 * \if NODOC
 * $Id: cmap.h_v 1.19 2013/05/08 13:05:22 mju Exp $
 *
 * $Log: cmap.h_v $
 * Revision 1.19  2013/05/08 13:05:22  mju
 * Remove unused getobjfilter.
 *
 * Revision 1.18  2007/09/06 17:18:35  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.16  2007/01/18 14:43:51  mju
 * Use color_array.
 *
 * Revision 1.1  2000/09/29 15:37:49  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_CMAP_H
#define INC_RVC_CMAP_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_COLORARRAY_H
	#include <mi32/colorarray.h>
#endif

namespace RVC {

//=====================================================================================================================
//! Manages RVC::OBJTYPE_ColorPalette objects.  
//!
//!	RVC Object Types that this class supports: OBJTYPE_ColorPalette.
//!
//!	Valid parents for this class:
//!		- OBJTYPE_File
//!		- OBJTYPE_Folder
//!		- OBJTYPE_Raster
class COLORMAP : public RVC::SIMPLE {
	public:

		COLORMAP ();

		COLORMAP (
			const RVC::COLORMAP& rhs
			);

		virtual ~COLORMAP ();

		RVC::COLORMAP& operator= (
			const RVC::COLORMAP& rhs
			);

		//! Generate the default 80 standard color colormap
		//! Can be used to Make() an object with the 80 std colors
		ERRVALUE GenerateStandardColors ();

		//! Obtain color map from the instance, not useful until Read() method is performed
		//! @return Constant Reference to colormap array.
		const COLOR_ARRAY& GetColorMap () const;

		//! Set the internal colormap array storage for later writing
		ERRVALUE SetColorMap (
			const COLOR_ARRAY& ColorMap
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		COLOR_ARRAY m_ColorMap;

		// Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		// Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}		// End of RVC namespace

#endif			// INC_RVCCMAP_H
