/**
 * \file rvc/dbfieldhelper.h
 * \brief DBFIELDHELPER class definitions
 *
 * \if NODOC
 * $Id: dbfieldhelper.h_v 1.4 2014/12/30 14:43:53 mju Exp $
 *
 * $Log: dbfieldhelper.h_v $
 * Revision 1.4  2014/12/30 14:43:53  mju
 * Doxygen fix.
 *
 * Revision 1.3  2014/10/13 20:20:19  mju
 * Fix missing comparison on isFloat/String.
 *
 * Revision 1.2  2014/10/02 13:45:44  mju
 * Add NumTypeFromFieldType and FieldTypeFromNumType.
 *
 * Revision 1.1  2014/10/01 21:15:31  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_DBFIELDHELPER_H
#define	INC_RVC_DBFIELDHELPER_H

#ifndef	INC_RVC_DBFIELDTYPE_H
#include <rvc/dbfieldtype.h>
#endif

#ifndef	INC_MI32_NUMTYPE_H
#include <mi32/numtype.h>
#endif


#ifdef RVCSYSDLL
	#define CLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSEXPORT MI_DLLCLASSIMPORT
#endif

namespace RVC {

//=====================================================================================================================

//! Helper for database fields.
//! Contains only static methods.
class CLASSEXPORT DBFIELDHELPER {
	public:

		//! Get DBFIELDTYPE from NUMTYPE.
		static DBFIELDTYPE GetFieldTypeFromNumType (
			NUMTYPE NumType,
			bool bExtendForNull = false	//!< Whether to extend 8/16 bit types for "null" value
			);

		//! Get maximum allowed value for specified field type.
		static double GetMaximumForType (
			DBFIELDTYPE fieldtype,			//!< Field type
			bool bExcludeNull = false		//!< Whether to exclude "null" value
			);

		//! Get minimum allowed value for specified field type.
		static double GetMinimumForType (
			DBFIELDTYPE fieldtype,			//!< Field type
			bool bExcludeNull = false		//!< Whether to exclude "null" value
			);

		//! Get NUMTYPE from DBFIELDTYPE.
		static NUMTYPE GetNumTypeFromFieldType (
			DBFIELDTYPE fieldtype			//!< Field type
			);
			
		//! Determine if field type is floating-point numeric.
		static bool IsFloat (
			DBFIELDTYPE fieldtype			//!< Field type
			) { return (fieldtype == DBFIELDTYPE_Double || fieldtype == DBFIELDTYPE_Float); }

		//! Determine if field type is numeric.
		static bool IsNumeric (
			DBFIELDTYPE fieldtype			//!< Field type
			);

		//! Determine if field type is text string.
		//! Note, does not include "memo" types.
		static bool IsString (
			DBFIELDTYPE fieldtype			//!< Field type
			) { return (fieldtype == DBFIELDTYPE_String || fieldtype == DBFIELDTYPE_Unicode); }
	};

//=====================================================================================================================

}	// End namespace RVC

#undef CLASSEXPORT

#endif	// INC_RVC_DBFIELDHELPER_H
