/**
 * \file rvc/dbfieldtype.h
 * \brief RVC::DBFIELDTYPE enumeration
 *
 * \if NOLOG
 * $Id: dbfieldtype.h_v 1.2 2014/09/30 20:51:38 mju Exp $
 *
 * $Log: dbfieldtype.h_v $
 * Revision 1.2  2014/09/30 20:51:38  mju
 * Add op_increment.
 *
 * Revision 1.1  2014/09/29 19:20:34  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_DBFIELDTYPE_H
#define	INC_RVC_DBFIELDTYPE_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

namespace RVC {

//=====================================================================================================================

//! Database field types.
enum DBFIELDTYPE {
	DBFIELDTYPE_Invalid = 0,		//!< Invalid field type
	DBFIELDTYPE_String = 1,			//!< Encoded character string, null-terminated 
	DBFIELDTYPE_Char = 2,			//!< Single character 
	DBFIELDTYPE_UINT8 = 3,			//!< Unsigned 1-byte integer (0-254)
	DBFIELDTYPE_SINT8 = 4,			//!< Signed 1-byte integer (-128-126)
	DBFIELDTYPE_UINT16 = 5,			//!< Unsigned 2-byte integer (0-65534)
	DBFIELDTYPE_SINT16 = 6,			//!< Signed 2-byte integer (-32768-32766)
	DBFIELDTYPE_UINT32 = 7,			//!< Unsigned 4-byte integer (0 - 2^32-2)
	DBFIELDTYPE_SINT32 = 8,			//!< Signed 4-byte integer (-2^31 - 2^31-2)
	DBFIELDTYPE_Float = 9,			//!< 4-byte floating-point number
	DBFIELDTYPE_Double = 10,		//!< 8-byte floating-point number
	DBFIELDTYPE_Logical = 11,		//!< 1-byte logical (?/Y/y/N/n/T/t/F/f/0/1)
	DBFIELDTYPE_Date = 12,			//!< 4-byte integer date YYYYMMDD
	DBFIELDTYPE_UINT64 = 13,		//!< Unsigned 8-byte integer (0 - 2^64-1)
	DBFIELDTYPE_SINT64 = 14,		//!< Signed 8-byte integer (-2^63 - 2^63-1)
	DBFIELDTYPE_Color = 15,			//!< Color spec. Always 8 bytes (COLOR)
	DBFIELDTYPE_Binary = 16,		//!< Binary field
	DBFIELDTYPE_Memo = 17,			//!< Memo field
	DBFIELDTYPE_Unicode = 18,		//!< MIUNICODE string, null-terminated
	DBFIELDTYPE_DateTime = 19,		//!< Date/Time field, 100-nanosecond intervals from 1-Jan-1601
	DBFIELDTYPE_NUMTYPES = 20,		//!< Number of defined field types

	// Be sure to update "alignment" and "size" arrays in rvcdbase.c when adding field types 
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_INCREMENT(DBFIELDTYPE)
#endif

//=====================================================================================================================

}	// End namespace RVC

#endif	// INC_RVC_DBFIELDTYPE_H
