/**
 * \file rvc/dbfieldusage.h
 * \brief RVC::DBFIELDUSAGE enumeration
 *
 * \if NODOC
 * $Id: dbfieldusage.h_v 1.2 2015/05/28 13:26:32 mju Exp $
 *
 * $Log: dbfieldusage.h_v $
 * Revision 1.2  2015/05/28 13:26:32  mju
 * Add elementModTime.
 *
 * Revision 1.1  2014/11/17 15:55:10  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_DBFIELDUSAGE_H
#define	INC_RVC_DBFIELDUSAGE_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

namespace RVC {

//=====================================================================================================================

//! Database field "usages".
enum DBFIELDUSAGE {
	DBFIELDUSAGE_Undefined = 0,
	DBFIELDUSAGE_XCoord = 1,					//!< X coordinate
	DBFIELDUSAGE_YCoord = 2,					//!< Y coordinate
	DBFIELDUSAGE_ZCoord = 3,					//!< Z coordinate
	DBFIELDUSAGE_Label = 4,						//!< Label
	DBFIELDUSAGE_Filename = 5,					//!< File name
	DBFIELDUSAGE_ObjName = 6,					//!< Object name
	DBFIELDUSAGE_ImageData = 7,				//!< Image data (in binary field, format setting applies)
	DBFIELDUSAGE_ImageFile = 8,				//!< Filename pointing to an image, format setting applies
	DBFIELDUSAGE_ImageURL = 9,					//!< URL pointing to an image, format setting applies
	DBFIELDUSAGE_ImageName = 10,				//!< Image name
//	DBFIELDUSAGE_Spatial = 11,
	DBFIELDUSAGE_URL = 12,						//!< A URL to anything
	DBFIELDUSAGE_DataTip = 13,					//!< DataTip
	DBFIELDUSAGE_Heading = 14,					//!< Heading (north 0 increasing clockwise)
	DBFIELDUSAGE_Velocity = 15,				//!< Velocity
	DBFIELDUSAGE_CRS = 16,						//!< Coordinate reference system (format applies)
	DBFIELDUSAGE_DateTime = 17,				//!< Date and Time
	DBFIELDUSAGE_ImageThumbnail = 18,		//!< Image thumbnail in a blob.  Thumbnail is for preceding DBFIELDUSAGE_ImageBLOB or DBFIELDUSAGE_ImageFile field
	DBFIELDUSAGE_ImageDescription = 19,		//!< Fields holds the description of the image
	DBFIELDUSAGE_RecordModTime = 20,			//!< Time the record was last written -- automatically updated
	DBFIELDUSAGE_EnumLabel = 21,				//!< Enumeration label
	DBFIELDUSAGE_EnumToolTip = 22,			//!< Enumeration ToolTip
	DBFIELDUSAGE_ElementID = 23,				//!< Initially from GML 2.0.  Each element can have a built in name, description and ID values
	DBFIELDUSAGE_ElementName = 24,			//!< Element "name"
	DBFIELDUSAGE_ElementDesc = 25,			//!< Element "description"
	DBFIELDUSAGE_SegmentStart = 26,			//!< Start of "segment" (portion of polyline) the record applies to
	DBFIELDUSAGE_SegmentEnd = 27,				//!< End of "segment" (portion of polyline) the record applies to
	DBFIELDUSAGE_SegmentLength = 28,			//!< Length of "segment" (portion of polyline) the record applies to
	DBFIELDUSAGE_TotalLength = 29,			//!< Total length, may not match computed length, used to scale segment start/end/length values to geometric element
	DBFIELDUSAGE_ElementModTime = 30,		//!< Time element was modified
	DBFIELDUSAGE_NUMTYPES
	// Values are stored in UINT8 so maximum allowed is 255
	};

//=====================================================================================================================

}	// End namespace RVC

#endif	// INC_RVC_DBFIELDUSAGE_H
