/**
 * \file rvc/dbtabsty.h 
 * \brief interface for the RVC::DBTABLE_STYLE class.
 *
 * \if NODOC
 * $Id: dbtabsty.h_v 1.11 2013/05/09 22:19:22 mju Exp $
 *
 * $Log: dbtabsty.h_v $
 * Revision 1.11  2013/05/09 22:19:22  mju
 * Remove unused getobjfilter defns.
 *
 * Revision 1.10  2005/12/23 22:31:47  scowan
 * Added two get object selection filters.
 *
 * Revision 1.9  2004/05/27 17:57:11  scowan
 * Added convert style assign table to method.
 *
 * Revision 1.4  2002/05/24 17:22:49  scowan
 * Added set style link method.
 *
 * Revision 1.1  2002/04/05 18:48:21  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_DBTABSTY_H
#define INC_RVC_DBTABSTY_H

#ifndef INC_RVC_DBTABLE_H
	#include <rvc/dbtable.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class STYLE;		// Forward declarations
#endif

//! Manage RVC::OBJTYPE_Table objects with RVC::DBTABLE::Usage_XXXStyle. 
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Table.
//!
//!	RVC Database Table Usage that this class supports: 
//!		- RVC::DBTABLE::USAGE_PointStyle
//!		- RVC::DBTABLE::USAGE_LineStyle
//!		- RVC::DBTABLE::USAGE_PolyStyle
//!		- RVC::DBTABLE::USAGE_CADStyle
//!
//!	Valid parents for this class:
//!		RVC::OBJTYPE_Database
//!		RVC::OBJTYPE_Database_Raster
//!		RVC::OBJTYPE_Database_VectorPoint
//!		RVC::OBJTYPE_Database_VectorLine
//!		RVC::OBJTYPE_Database_VectorPolygon
//!		RVC::OBJTYPE_Database_CAD
//!		RVC::OBJTYPE_Database_TIN
//!
class DBTABLE_STYLE : public RVC::DBTABLE {
	public:

		//! Default constructor
		DBTABLE_STYLE ();

		//! Copy constructor
		DBTABLE_STYLE (
			const RVC::DBTABLE_STYLE& rhs
			);

		//! Destructor
		virtual ~DBTABLE_STYLE ();

		//! Assignment
		RVC::DBTABLE_STYLE& operator= (
			const RVC::DBTABLE_STYLE& rhs
			);
			
		//! Convert style assignment table from 'this' to 'DestStyleTable' based on the 
		//! table usage types.
		ERRVALUE ConvertTableTo (
			DBTABLE_STYLE& DestStyleTable				//!< Must be created outside this function and be open
			);

		//! Locate style object referenced by this style assignment table
		//! 'StyleObj' will be attached if one is found, unattached if not
		ERRVALUE FindStyleObject (
			RVC::STYLE& StyleObj
			);

		//! Set the style object that the style assignment table links to			
		ERRVALUE SetStyleLink (
			const RVC::STYLE& StyleObj			//!< Style object to link to
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
	
		ERRVALUE MakeStyleLink (STYLE& StyleObj);

		virtual bool v_IsValidTable (const RVC::DBTABLE& table);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_DBTABSTY_H
