/**
 * \file rvc/dbtnode.h
 * \brief interface for the RVC::DBASE_TINNODE class.
 *
 * \if NODOC
 * $Id: dbtnode.h_v 1.14 2015/05/12 21:44:01 mju Exp $
 *
 * $Log: dbtnode.h_v $
 * Revision 1.14  2015/05/12 21:44:01  mju
 * Use dbtabletrans.
 *
 * Revision 1.13  2015/05/08 19:09:29  mju
 * Use elemnumtrans.
 *
 * Revision 1.12  2010/05/12 21:09:00  mju
 * Simplify and specify parent objtype when construct base.
 *
 * Revision 1.10  2006/06/02 22:23:45  dwilliss
 * Changes to Join
 *
 * Revision 1.1  2001/05/25 19:22:33  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_DBTNODE_H
#define INC_RVC_DBTNODE_H

#ifndef INC_RVC_DBELEM_H
	#include <rvc/dbelem.h>
#endif

namespace RVC {

//! Manage RVC::OBJTYPE_Database_TIN objects.
class DBASE_TINNODE : public RVC::DBASE_ELEMENT {
	public:

		//! Default constructor
		DBASE_TINNODE (
			): DBASE_ELEMENT(OBJTYPE_TIN,ELEMTYPE_Node) { }

		//! Copy constructor
		DBASE_TINNODE (
			const RVC::DBASE_TINNODE& rhs
			): DBASE_ELEMENT(rhs) { }

		//! Destructor
		virtual ~DBASE_TINNODE (
			);

		//! Assignment
		RVC::DBASE_TINNODE& operator= (
			const RVC::DBASE_TINNODE& rhs
			);

		//! Determine that the node has attributes
		//! @return 'True' if the node is considered a point, false if not
		bool IsNodeAPoint (
			INT32 NodeNum
			);

		//! Join another database to this one and deal with posible nodes
		ERRVALUE JoinPointDatabaseTo (
			RVC::DBASE_ELEMENT& DestDBase,
			const ELEMNUMTRANS& PointTransList,
			const ELEMNUMTRANS& NodeTransList,
			RVC::DBASE::COPYFLAG CopyFlags = RVC::DBASE::COPYFLAG_None,
			RVC::DBASE::JOINFLAGS JoinFlags = RVC::DBASE::JOINFLAG_None,
			DBTABLETRANSLIST *pTableTransList = 0,		//!< Optional table translation list, NULL to copy all user tables
			INT32* pNextElemID = 0,		//! Pointer to INT32 return next ElemID in
			ELEMTYPE SrcElemType = ELEMTYPE_Empty,  //! Will default to the default element type of the source database
			ELEMTYPE DestElemType = ELEMTYPE_Empty  //! Will default to the default element type of the destination database
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DBASE_ELEMENT::DBLIST m_DBList;		//!< Used for IsNodeAPoint()

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		// End of RVC namespace

#endif 	// INC_RVC_DBTNODE_H
