/**
 * \file rvc/elemset.h
 * \brief RVC::ELEMSET definitions
 *
 * \if NODOC
 * $Id: elemset.h_v 1.7 2015/04/30 14:02:49 mju Exp $
 *
 * $Log: elemset.h_v $
 * Revision 1.7  2015/04/30 14:02:49  mju
 * Use elemidx.
 *
 * Revision 1.6  2015/04/30 13:36:31  mju
 * Use elemidx hdr.
 *
 * Revision 1.5  2014/04/07 15:56:16  mju
 * Add clearByType.
 *
 * Revision 1.4  2014/04/01 20:30:41  mju
 * Add isEmpty and getElemTypeName.
 *
 * Revision 1.3  2014/04/01 17:33:04  mju
 * Add getNumItems.
 *
 * Revision 1.2  2014/03/28 21:37:29  mju
 * Change isSingle to check elemnum.
 *
 * Revision 1.1  2014/03/28 21:28:59  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_ELEMSET_H
#define	INC_RVC_ELEMSET_H

#ifndef	INC_RVC_ELEMIDX_H
#include <rvc/elemidx.h>
#endif

#ifndef	INC_MI32_UNSIGNEDSET_H
#include <mi32/unsignedset.h>
#endif

namespace RVC {

//=====================================================================================================================

//! Set of ELEMIDXs.
class ELEMSET {
	public:

		//! Iterator for ELEMIDXs in set.
		class ITERATOR {
			public:

				//! Default constructor.
				ITERATOR (
					) : m_ElemIdx(ELEMTYPE_NumTypes), m_pElemSet(0) { }

				//! Copy constructor.
				ITERATOR (
					const ITERATOR &rhs
					): m_ElemIdx(rhs.m_ElemIdx), m_pElemSet(rhs.m_pElemSet), m_Iterator(rhs.m_Iterator) { }

				#ifndef GENERATING_DOXYGEN_OUTPUT
				// Internal constructor.
				ITERATOR (
					const ELEMSET &elemset
					);
				#endif

				//! Assignment.
				ITERATOR& operator= (
					const ITERATOR &rhs
					) {
					if (this != &rhs) { m_ElemIdx = rhs.m_ElemIdx; m_pElemSet = rhs.m_pElemSet; m_Iterator = rhs.m_Iterator; }
					return (*this);
					}

				//! Return reference to ELEMIDX.
				const ELEMIDX& operator* () const
					{ return (m_ElemIdx); }

				//! Return pointer to ELEMIDX.
				const ELEMIDX* operator-> () const
					{ return (&m_ElemIdx); }

				//! Pre-increment operator.
				ITERATOR& operator++ ();

				//! Equality operator
				bool operator== (
					const ITERATOR &rhs
					) const { return (m_ElemIdx == rhs.m_ElemIdx); }

				//! Inequality operator
				bool operator!= (
					const ITERATOR &rhs
					) const { return (!(*this == rhs)); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				ELEMIDX m_ElemIdx;
				const ELEMSET *m_pElemSet;
				UNSIGNEDSET::ITERATOR64 m_Iterator;
				#endif
			};

		//! Default constructor, creates empty set.
		ELEMSET ();

		//! Implicit constructor for single element
		ELEMSET (
			const ELEMIDX &elemidx
			);

		//! Destructor.
		~ELEMSET ();

		//! Access set by type, will create empty set if none.
		UNSIGNEDSET* AccessByType (
			ELEMTYPE ElemType
			);

		//! Access single-element.
		ELEMIDX& AccessSingle ()
			{ return (m_ElemIdxSingle); }

		//! Add range of element numbers to set.
		ERRVALUE AddRange (
			ELEMTYPE ElemType,
			INT32 ElemNumMin,
			INT32 ElemNumMax
			) { return (AccessByType(ElemType)->AddRange(ElemNumMin,ElemNumMax)); }

		//! Add single element number to set.
		ERRVALUE AddSingle (
			ELEMTYPE ElemType,
			INT32 ElemNum
			) { return (AccessByType(ElemType)->AddSingle(ElemNum)); }

		//! Begin iterator.
		ITERATOR Begin () const
			{ return (ITERATOR(*this)); }

		//! Clear set of all element numbers.
		void Clear ();

		//! Clear set for specified element type.
		void ClearByType (
			ELEMTYPE ElemType
			);

		//! End iterator.
		ITERATOR End () const
			{ return (ITERATOR()); }

		//! Get set for ELEMTYPE, if any.
		//! @return Pointer to set, NULL if none.
		const UNSIGNEDSET* GetByType (
			ELEMTYPE ElemType
			) const { return (m_pSets[ElemType]); }

		//! Get name to display for type of elements in set.
		TEXTID GetElemTypeName (
			ELEMTYPETEXTID NameType
			) const;

		//! Get number of items in set.
		UINT64 GetNumItems () const;

		//! Get single-element instance.
		const ELEMIDX& GetSingle () const
			{ return (m_ElemIdxSingle); }

		//! Determine if set is empty.
		bool IsEmpty () const;

		//! Determine if contains single-element instance.
		bool IsSingle () const
			{ return (m_ElemIdxSingle.GetNumber() != ELEMENTNUM_None); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ELEMIDX m_ElemIdxSingle;
		UNSIGNEDSET *m_pSets[ELEMTYPE_NumTypes];
		mutable UINT64 m_NumItems;
		#endif
	};

//=====================================================================================================================

}	// End namespace RVC

#endif	// INC_RVC_ELEMSET_H
