/**
 * \file rvc/geometrc.h
 * \brief interface for the RVC::GEOMETRIC class.
 *
 * \if NODOC
 * $Id: geometrc.h_v 1.125 2015/05/08 19:09:32 mju Exp $
 *
 * $Log: geometrc.h_v $
 * Revision 1.125  2015/05/08 19:09:32  mju
 * Use elemnumtrans, simplify elemtranslist.
 *
 * Revision 1.124  2015/05/01 19:07:03  mju
 * Add textanchor for label.
 * Add readLabel/Line/Point using region.
 *
 * Revision 1.123  2015/04/30 13:36:34  mju
 * Use elemidx hdr.
 *
 * Revision 1.122  2015/04/29 22:11:03  mju
 * Change virtuals for reading line/point/label, use elemidx.
 *
 * Revision 1.121  2015/04/27 22:07:10  mju
 * Add v_componentToElemType.
 *
 * Revision 1.120  2015/04/10 17:20:34  mju
 * Add extractflag_IgnoreLabelTables.
 *
 * Revision 1.119  2015/01/14 22:50:17  mju
 * Remove getElementBitsetDeque.
 *
 * Revision 1.118  2015/01/14 17:16:07  mju
 * Change elemtranslist::getElementBitset to getElementSet and use unsignedset.
 *
 * Revision 1.117  2014/12/29 17:55:41  mju
 * Add doxygen exclusions for deprecated methods.
 *
 * Revision 1.116  2014/11/21 20:53:04  mju
 * Add getPointTypeTextID.
 *
 * Revision 1.115  2014/05/20 16:52:48  mju
 * Add extractflag_CopyElementIDTables.
 *
 * Revision 1.114  2014/04/07 17:59:04  mju
 * Use rvc::element for getExtents and readElement.
 *
 * Revision 1.113  2013/10/24 21:20:35  mju
 * Change v_readRegion to use unsignedset.
 *
 * Revision 1.112  2013/05/13 17:36:15  mju
 * Remove getobjfilter_isselectable.
 *
 * Revision 1.111  2013/05/07 19:47:47  mju
 * Remove getobjfilter_hasnoimpliedgeoref as unused.
 *
 * Revision 1.110  2013/05/02 21:52:25  mju
 * Add extentsObjItem to elemselectparms.
 *
 * Revision 1.109  2013/01/10 16:22:52  mju
 * Add defaults for findClosestElement.
 *
 * Revision 1.108  2013/01/08 16:04:59  mju
 * Add extractflag_IgnoreLineTables.
 *
 * Revision 1.107  2012/10/09 15:03:11  mju
 * Add "separate" to setupElemTransList.
 *
 * Revision 1.106  2012/07/25 17:30:15  mju
 * Add elemselectparms::addBorder member.
 *
 * Revision 1.105  2012/07/25 17:12:44  mju
 * Add elemselectparms::separation.
 *
 * Revision 1.102  2012/05/14 22:37:21  mju
 * Add labelgenerator.
 * Change label to not have to contain textstyle / frameparms and add elemtype.
 *
 * Revision 1.101  2012/05/11 17:49:53  mju
 * Change label class to have access/set methods, separate baseline and defriend cad/shape/vector.
 *
 * Revision 1.99  2011/12/23 15:36:55  mju
 * Use labelframeparms hdr and remove others that are not needed for this hdr.
 *
 * Revision 1.96  2010/04/22 20:03:50  mju
 * Add findElementSet using unsignedset.
 *
 * Revision 1.95  2010/04/19 16:20:45  mju
 * Add searchfilter_unsignedset.
 * Add searchresult::getElemSet using unsignedset.
 *
 * Revision 1.93  2008/10/01 20:01:14  scowan
 * Added multi table support cap flag.
 *
 * Revision 1.85  2006/11/01 22:58:47  dfriberg
 * Make extents calculation include leader lines as an option for labels
 *
 * Revision 1.77  2005/10/20 17:00:36  scowan
 * Quadtree and geometric interface changes supported.
 * Support for new quadtree system required changes to virtual method tables.
 *
 * Revision 1.1  2000/10/02 17:39:43  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_GEOMETRC_H
#define INC_RVC_GEOMETRC_H

#ifndef INC_RVC_GEOSPATL_H
	#include <rvc/geospatl.h>
#endif

#ifndef INC_RVC_STYLETYPE_H
	#include <rvc/styletype.h>
#endif

#ifndef INC_MI32_LABELFRAMEPARMS_H
	#include <mi32/labelframeparms.h>
#endif

#ifndef INC_RVC_ELEMIDX_H
	#include <rvc/elemidx.h>
#endif

#ifndef INC_RVC_QUADTREE_H
	#include <rvc/quadtree.h>
#endif

#ifndef INC_MI32_REGION2D_H
	#include <mi32/region2d.h>
#endif

#ifndef INC_MI32_TRANS2D_AFFINE_H
	#include <mi32/trans2daffine.h>
#endif

#ifndef INC_MI32_BITSET_H
	#include <mi32/bitset.h>
#endif

#ifndef INC_MI32_ELEMNUMTRANS_H
	#include <mi32/elemnumtrans.h>
#endif

#ifndef INC_MI32_ELEMTYPESET_H
	#include <mi32/elemtypeset.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
class TRANS2D_MAPGEN;
class UNSIGNEDSET;
namespace MGD {
	class CONTEXT;
	}
namespace SML {
	class CONTEXT;
	}
namespace RVC {
	struct ANYCALLBACKSTRUCT;
	class CAD;
	class SHAPE;
	class SPATIALTREE;
	class STYLE;
	class VECTOR;
	}
#endif


namespace RVC {

//=====================================================================================================================

//! Base for "geometric" objects.
class GEOMETRIC : public GEOSPATIAL {
	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ELEMSELECTFILTER;
		class ELEMSELECTPARMS;
		class ELEMTRANSLIST;
		class GETOBJFILTER_HASMINIMUMELEMENTS;
		class GETOBJFILTER_ISDISPLAYABLE;
		class GETOBJFILTER_ISNOTEMPTY;
		class MAKEPARMSBASE;
		class LABEL;
		class OBSERVER;
		class SEARCHFILTER;
		class SEARCHFILTER_BITSET;
		class SEARCHFILTER_UNSIGNEDSET;
		class SEARCHFILTER_ELEMTRANSLIST;
		class SEARCHRESULT;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! General label container.
		class LABEL {
			public:

				//! Default constructor.
				LABEL ();

				//! Copy constructor.
				LABEL (
					const LABEL &rhs
					);

				//! Destructor.
				~LABEL ();

				//! Assignment operator.
				LABEL& operator= (
					const LABEL &rhs
					);

				//! Access leader line list (non-const).
				//! \internal
				POLYLINELIST& AccessLeaderLineList ()
					{ return (m_LeaderLineList); }

				//! Access frame parameters (non-const).
				//! \internal
				LABELFRAMEPARMS& AccessFrameParms ();

				//! Access text style (non-const).
				//! \internal
				TEXTSTYLE& AccessTextStyle ();

				//! Add leader line.
				void AddLeaderLine (
					const POLYLINE &LeaderLine
					) { m_LeaderLineList.PushBack(LeaderLine); }

				//! Attach frame parameters.
				void AttachFrameParms (
					LABELFRAMEPARMS *& pFrameParms,		//!< Pointer to frame parameters, if take ownership will be cleared, otherwise must remain valid
					bool TakeOwnership						//!< Whether to take ownership of pointer
					);

				//! Attach text style.
				void AttachTextStyle (
					TEXTSTYLE *& pTextStyle,				//!< Pointer to text style, if take ownership will be cleared, otherwise must remain valid
					bool TakeOwnership						//!< Whether to take ownership of pointer
					);

				//! Clear label contents.
				void Clear ();

				//! Compute the extents of this label.
				//! If the height specified in the text style is not in SCREENUNIT_Object units, 
				//! then this method will return invalid extents for the label.
				ERRVALUE ComputeExtents (
					DRECT3D &extents,
					bool IncludeLeaders = true
					) const;
					
				//! Get where text is anchored at.
				//! Only applies if "anchor to" is TEXTANCHOR_TO_Point.
				TEXTANCHOR_AT GetAnchorAt () const
					{ return (static_cast<TEXTANCHOR_AT>(m_TextAnchorAt)); }

				//! Get anchor point.
				//! Only valid if "anchor to" is TEXTANCHOR_TO_Point.
				DPOINT2D GetAnchorPoint () const;

				//! Get what text is anchored to.
				TEXTANCHOR_TO GetAnchorTo () const
					{ return (static_cast<TEXTANCHOR_TO>(m_TextAnchorTo)); }

				//! Get label baseline.
				//! @return Label baseline points
				const POLYLINE& GetBaseline () const
					{ return (m_BaseLine); }

				//! Get general element type being labeled.
				ELEMTYPE GetElemType () const
					{ return (static_cast<ELEMTYPE>(m_ElemType)); }

				//! Get the label frame parameters.
				//! @return The label frame parameters
				const LABELFRAMEPARMS& GetFrameParms () const
					{ return (*m_pFrameParms); }

				//! Get number of leader lines.
				INT32 GetNumLeaderLines () const
					{ return (m_LeaderLineList.GetNumItems()); }

				#ifndef GENERATING_DOXYGEN_OUTPUT
			#ifndef NO_DEPRECATED
				//! Get label text style.
				//! \deprecated
				DEPRECATED_MSG("Use GetTextStyle instead") const TEXTSTYLE &GetStyle () const
					{ return (*m_pTextStyle); }
			#endif
				#endif

				//! Get label text string.
				//! @return Label string
				const MISTRING& GetText () const
					{ return (m_Text); }

				//! Get text height.
				double GetTextHeight () const
					{ return (m_TextHeight); }

				//! Get label text style.
				const TEXTSTYLE& GetTextStyle () const
					{ return (*m_pTextStyle); }

				//! Get (const) iterator at beginning of leader list.
				POLYLINELIST::CONST_ITERATOR LeaderLineBegin () const
					{ return (m_LeaderLineList.Begin()); }
			
				//! Get (const) iterator at end of leader list.
				POLYLINELIST::CONST_ITERATOR LeaderLineEnd () const
					{ return (m_LeaderLineList.End()); }

				//! Set where text is anchored at.
				//! Only applies if "anchor to" is TEXTANCHOR_TO_Point.
				void SetAnchorAt (
					TEXTANCHOR_AT AnchorAt
					) { m_TextAnchorAt = static_cast<UINT8>(AnchorAt); }

				//! Set what text is anchored to.
				void SetAnchorTo (
					TEXTANCHOR_TO AnchorTo
					) { m_TextAnchorTo = static_cast<UINT8>(AnchorTo); }

				//! Set baseline points.
				void SetBaseLine (
					const POLYLINE &BaseLine
					) { m_BaseLine = BaseLine; }

				//! Set general element type being labeled.
				void SetElemType (
					ELEMTYPE ElemType					//!< Element type, typically Point, Line or Polygon
					) { m_ElemType = static_cast<UINT8>(ElemType); }

				//! Set frame parameters.
				void SetFrameParms (
					const LABELFRAMEPARMS &FrameParms
					) { AccessFrameParms() = FrameParms; }

				//! Set text string.
				void SetText (
					const MISTRING &Text
					) { m_Text = Text; }

				//! Set text height.
				void SetTextHeight (
					double TextHeight
					) { m_TextHeight = TextHeight; }

				//! Set text style.
				void SetTextStyle (
					const TEXTSTYLE &TextStyle
					) { AccessTextStyle() = TextStyle; }
			
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MISTRING m_Text;
				POLYLINE m_BaseLine;
				POLYLINELIST m_LeaderLineList;
				TEXTSTYLE *m_pTextStyle;
				LABELFRAMEPARMS *m_pFrameParms;
				double m_TextHeight;
				UINT8 m_TextAnchorTo;
				UINT8 m_TextAnchorAt;
				UINT8 m_ElemType;
				bool m_bOwnTextStyle;
				bool m_bOwnFrameParms;

				static LABELFRAMEPARMS* GetSharedFrameParms ();
				static TEXTSTYLE* GetSharedTextStyle ();

				void Release ();
				#endif
			};	// End LABEL
		typedef MILIST<LABEL> LABELLIST;

		//! This is the base class for the LINE, POINT, POLYGON, and LABEL iterators		
		class ITERATOR {
			public:
				//! Default constructor
				ITERATOR (
					) : m_pObjGeometric(0) {}
					
				//! Internal constructor
				ITERATOR (
					RVC::GEOMETRIC *pObjGeometric
					) : m_pObjGeometric(pObjGeometric) {}
					
				//! Copy constructor
				ITERATOR (				
					const ITERATOR &rhs
					) : m_pObjGeometric(rhs.m_pObjGeometric), m_Element(rhs.m_Element) {}

				//! Dereference operator, returns reference to templated item					
				const RVC::ELEMIDX& operator* (
					) const { return (m_Element); }

				//! Dereference operator, returns pointer to templated item
				const RVC::ELEMIDX* operator-> (
					) const { return (&**this); }

				bool operator== (				//! Equality operator
					const ITERATOR &rhs
					) const { return (m_pObjGeometric == rhs.m_pObjGeometric && m_Element == rhs.m_Element); }

				bool operator!= (				//! Inequality operator
					const ITERATOR &rhs
					) const { return (!(*this == rhs)); }

			protected:
				GEOMETRIC *m_pObjGeometric;
				RVC::ELEMIDX m_Element;
			};

		class LABELITERATOR : public ITERATOR {
			public:
				//! Default constructor
				LABELITERATOR (
					) : ITERATOR() {}

				//! Internal constructor
				LABELITERATOR (
					RVC::GEOMETRIC *pObjGeometric,
					DIMENSION dim,
					STYLE *pObjStyle
					) : ITERATOR(pObjGeometric), m_dim(dim), m_pObjStyle(pObjStyle) {
					m_pObjGeometric->ReadNextLabel(m_Element, m_LabelList, m_dim, m_pObjStyle);
					}
					
				//! Internal constructor
				LABELITERATOR (
					RVC::GEOMETRIC *pObjGeometric
					) : ITERATOR(pObjGeometric) {}
					
				//! Copy constructor
				LABELITERATOR (				
					const LABELITERATOR &rhs
					) : 
					ITERATOR(rhs), 
					m_LabelList(rhs.m_LabelList), 
					m_dim(rhs.m_dim),
					m_pObjStyle(rhs.m_pObjStyle)
					{}

				//! Pre-increment operator.
				LABELITERATOR& operator++ (
					) {
					m_pObjGeometric->ReadNextLabel(m_Element, m_LabelList, m_dim, m_pObjStyle);
					return (*this);
					}
					
				const LABELLIST& GetLabelList (
					) const { return (m_LabelList); }
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				LABELLIST m_LabelList;
				DIMENSION m_dim;
				STYLE *m_pObjStyle;
				#endif
			};
			
		class LINEITERATOR : public ITERATOR {
			public:
				//! Default constructor
				LINEITERATOR (
					) : ITERATOR() {}

				//! Internal constructor
				LINEITERATOR (
					RVC::GEOMETRIC *pObjGeometric,
					DIMENSION dim
					) : ITERATOR(pObjGeometric), m_dim(dim) {
					m_pObjGeometric->ReadNextLine(m_Element, m_LineList, m_dim);
					}
					
				//! Internal constructor
				LINEITERATOR (
					RVC::GEOMETRIC *pObjGeometric
					) : ITERATOR(pObjGeometric) {}
					
				//! Copy constructor
				LINEITERATOR (				
					const LINEITERATOR &rhs
					) : 
					ITERATOR(rhs), 
					m_LineList(rhs.m_LineList), 
					m_dim(rhs.m_dim)
					{}

				//! Pre-increment operator.
				LINEITERATOR& operator++ (
					) {
					m_pObjGeometric->ReadNextLine(m_Element, m_LineList, m_dim);
					return (*this);
					}
					
				const POLYLINELIST& GetLineList (
					) const { return (m_LineList); }
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				POLYLINELIST m_LineList;
				DIMENSION m_dim;
				#endif
			};
			
		class NODEITERATOR : public ITERATOR {
			public:
				//! Default constructor
				NODEITERATOR (
					) : ITERATOR() {}

				//! Internal constructor
				NODEITERATOR (
					RVC::GEOMETRIC *pObjGeometric,
					DIMENSION		// dim
					) : ITERATOR(pObjGeometric) {
					m_pObjGeometric->ReadNextNode(m_Element, m_Node);
					}
					
				//! Internal constructor
				NODEITERATOR (
					RVC::GEOMETRIC *pObjGeometric
					) : ITERATOR(pObjGeometric) {}
					
				//! Copy constructor
				NODEITERATOR (				
					const NODEITERATOR &rhs
					) : 
					ITERATOR(rhs), 
					m_Node(rhs.m_Node) 
					{}

				//! Pre-increment operator.
				NODEITERATOR& operator++ (
					) {
					m_pObjGeometric->ReadNextNode(m_Element, m_Node);
					return (*this);
					}
					
				const DPOINT3D& GetNode (
					) const { return (m_Node); }
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DPOINT3D m_Node;
				#endif
			};
			
		class POINTITERATOR : public ITERATOR {
			public:
				//! Default constructor
				POINTITERATOR (
					) : ITERATOR() {}

				//! Internal constructor
				POINTITERATOR (
					RVC::GEOMETRIC *pObjGeometric,
					DIMENSION dim
					) : ITERATOR(pObjGeometric), m_dim(dim) {
					m_pObjGeometric->ReadNextPoint(m_Element, m_PointList, m_dim);
					}
					
				//! Internal constructor
				POINTITERATOR (
					RVC::GEOMETRIC *pObjGeometric
					) : ITERATOR(pObjGeometric) {}
					
				//! Copy constructor
				POINTITERATOR (				
					const POINTITERATOR &rhs
					) : 
					ITERATOR(rhs), 
					m_PointList(rhs.m_PointList), 
					m_dim(rhs.m_dim)
					{}

				//! Pre-increment operator.
				POINTITERATOR& operator++ (
					) {
					m_pObjGeometric->ReadNextPoint(m_Element, m_PointList, m_dim);
					return (*this);
					}
					
				const POLYLINE& GetPointList (
					) const { return (m_PointList); }
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				POLYLINE m_PointList;
				DIMENSION m_dim;
				#endif
			};
			
		class POLYGONITERATOR : public ITERATOR {
			public:
				//! Default constructor
				POLYGONITERATOR (
					) : ITERATOR() {}

				//! Internal constructor
				POLYGONITERATOR (
					RVC::GEOMETRIC *pObjGeometric,
					DIMENSION dim
					) : ITERATOR(pObjGeometric), m_dim(dim) {
					m_pObjGeometric->ReadNextRegion(m_Element, m_Region, m_dim);
					}
					
				//! Internal constructor
				POLYGONITERATOR (
					RVC::GEOMETRIC *pObjGeometric
					) : ITERATOR(pObjGeometric) {}
					
				//! Copy constructor
				POLYGONITERATOR (				
					const POLYGONITERATOR &rhs
					) : 
					ITERATOR(rhs), 
					m_Region(rhs.m_Region), 
					m_dim(rhs.m_dim)
					{}

				//! Pre-increment operator.
				POLYGONITERATOR& operator++ (
					) {
					m_pObjGeometric->ReadNextRegion(m_Element, m_Region, m_dim);
					return (*this);
					}
					
				const REGION2D& GetRegion (
					) const { return (m_Region); }
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				REGION2D m_Region;
				DIMENSION m_dim;
				#endif
			};
	
		enum CAPABILITY {
			CAPABILITY_None = 				0,
			CAPABILITY_OwnSearchMethods = 512,	//!< Object has its own search methods (Oracle, MySQL, ...)
			CAPABILITY_CanAddToEnd,					//!< Object can be modified by adding elements to the end of the object
			CAPABILITY_CanChangeElements,			//!< Existing object elements can be modified
			CAPABILITY_HasStyleByElement,			//!< Each element can have its own style settings
			CAPABILITY_ElementHasExtents,			//!< Each element has extents stored with it, except labels
			CAPABILITY_HasLabels,		  			//!< Object has labels
			CAPABILITY_LabelHasExtents,  			//!< Each label has extents stored with it
			CAPABILITY_CanDeleteLastElement,		//!< Can delete the last element in the set
			CAPABILITY_CanDeleteAnyElement,		//!< Can delete any element in the set
			CAPABILITY_MultipleTableSupport,		//!< Format can support any number of tables
			CAPABILITY_HasSpatialTrees = 766,	//!< Object can support spatial tree indexes
			CAPABILITY_HasQuadTrees = 767			//!< Object can support quad tree indexes
			};

		enum COMPONENT {
			COMPONENT_Point = 		0x01,
			COMPONENT_Line = 			0x02,
			COMPONENT_Polygon = 		0x04,
			COMPONENT_Label = 		0x08,
			COMPONENT_Node =	 		0x10,
			COMPONENT_All = 			(COMPONENT_Point + COMPONENT_Line + COMPONENT_Polygon + COMPONENT_Label + COMPONENT_Node)
			};

		enum EXTRACTFLAGS {
			EXTRACTFLAG_None =						0,
			EXTRACTFLAG_RegionMethod =				0x00000001,		//!< SelectMethod is a REGION2D
			EXTRACTFLAG_RemExRecords =				0x00000002,		//!< Remove Excess (Unattached) database records
			EXTRACTFLAG_RemStdAttTables =			0x00000004,		//!< Remove Standard Attribute database tables
			EXTRACTFLAG_IgnoreMissingLines =		0x00000008,		//!< Ignore missing lines from extraction operation, polygon and node line lists
			EXTRACTFLAG_TableJoinNameStruct =	0x00000010,		//!< Join tables in Append functions if names alike and compatable structures
			EXTRACTFLAG_TableJoinStruct =			0x00000020,		//!< Join tables in Append functions if compatable structures
			EXTRACTFLAG_TableRemoveDupRec =		0x00000040,		//!< Do not copy over records if exists in dest table
			EXTRACTFLAG_IgnorePolygonTables =	0x00000080,		//!< Do not copy over polygon databases
			EXTRACTFLAG_RemoveTables =				0x00000100,		//!< Remove destination tables
			EXTRACTFLAG_CopyHyperIndexSub =		0x00000200,		//!< Copy Hyper index subobjects
			EXTRACTFLAG_DensifyPoints =			0x00000400,		//!< Densify line points using transformation parameters
			EXTRACTFLAG_CreateNewTables =			0x00000800,		//!< Copy records and tables instead of maintaining links.
			EXTRACTFLAG_IgnoreLineTables =		0x00001000,		//!< Don't copy line databases
			EXTRACTFLAG_CopyElementIDTables =	0x00002000,		//!< Copy element ID tables if available
			EXTRACTFLAG_IgnoreLabelTables =		0x00004000,		//!< Don't copy label databases
			};

		//! Flags for FindElements.
		enum FINDELEMENT {
			FINDELEMENT_All =					0x0001,	//!< All element within specified distance
			FINDELEMENT_Closest =			0x0002,	//!< Only closest element(s) within specified distance
			FINDELEMENT_Equivalent =		0x0003,	//!< Element equivalent to search region within specified distance
			FINDELEMENT_Overlap =			0x0004,	//!< Elements overlap search region
			FINDELEMENT_ContainedBy =		0x0005,	//!< Elements contained by search region (element inside region polygon, line subset of region lines)
			FINDELEMENT_Containing =		0x0006,	//!< Elements containing search region (region inside polygon element, region line subset of line element)
			FINDELEMENT_MASK_Mode =			0x000F,	//!< Mask for criteria mode
			FINDELEMENT_Force2D =			0x0010,	//!< Force to use 2D distance if both region and object are 3D
//			FINDELEMENT_IgnoreIslands =	0x0020,	//!< Ignore islands (holes) in elements for equivalence test
//			FINDELEMENT_DistanceInside =	0x0040,	//!< Return distance "inside" (polygon) element as negative distance to nearest boundary (returns 0 otherwise)
			};

		enum POINTTYPE {
			POINTTYPE_2DXY = 1,				//!< 2-D coordinates, X and Y given (default)
			POINTTYPE_3DXY = 4,				//!< 3-D coordinates, X, Y given, Z constant
			POINTTYPE_3DXYZ = 7				//!< 3-D coordinates, X, Y and Z given
			};

		//! Get geometric object referred to by 'object'
		//! @return Allocated instance of RVC::GEOMETRIC object (must be 'delete'd) or 0 if error
		static RVC::GEOMETRIC* GetGeometric (
			const RVC::OBJECT &object
			);

		//! Get geometric object referred to by 'ObjItem'
		//! @return Allocated instance of RVC::GEOMETRIC object (must be 'delete'd) or 0 if error
		static RVC::GEOMETRIC* GetGeometric (
			const RVC::OBJITEM &ObjItem
			);

		//! Get correct OBJTYPESET for all geometric OBJTYPE's
		//! @return OBJTYPESET for all geometric OBJTYPE's
		static RVC::OBJTYPESET GetObjTypeSet ();

		//! Get TEXTID for point type.
		static TEXTID GetPointTypeTextID (
			POINTTYPE PointType
			);

		//! Get region from label list.
		static void GetRegionFromLabels (
			REGION2D &region,
			const LABELLIST &labels
			);

		//! Determine if the object type is a geometric object
		static bool IsGeometric (
			RVC::OBJTYPE ObjType
			);

		//! Default constructor
		GEOMETRIC ();

		//! Copy constructor
		GEOMETRIC (
			const RVC::GEOMETRIC &rhs
			);

		//! Destructor
		virtual ~GEOMETRIC () = 0;

		//! Assignment
		RVC::GEOMETRIC& operator= (
			const RVC::GEOMETRIC &rhs
			);

		//! Add an element to the search tree
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE AddSearchTreeElement (
			INT32 elemnum,
			ELEMTYPE ElemType,
			const DRECT2D &Extents
			);

		//! Add an element to the search tree
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE AddSearchTreeElement (
			INT32 elemnum,
			ELEMTYPE ElemType,
			const DPOINT2D &Point
			);

		//! Populate a search tree given existing elements using the extents and number of elements.
		//! currently in the object.
		//! This will replace any existing search tree for this type with a new search tree.
		ERRVALUE BuildSearchTree (
			ELEMTYPE ElemType
			) const;

		//! Populate a search tree given existing elements using the extents and number of elements
		//! given by the QSEARCHTREE::MAKEPARMS.
		//! This will replace any existing search tree for this type with a new search tree.
		ERRVALUE BuildSearchTree (
			const QSEARCHTREE::MAKEPARMS &MakeParms
			) const;

		//! Clear a search tree using the extents and number of initial elements given by the QSEARCHTREE::MAKEPARMS.
		//! Does not add known elements to the search tree.
		ERRVALUE ClearSearchTree (
			const QSEARCHTREE::MAKEPARMS &MakeParms
			) const;

		//! Combine styles from source object 'this' to the destination object 'DestObj'.
		//! this method deals with style links from source tables
		ERRVALUE CombineStylesTo (
			RVC::GEOMETRIC &DestObj
			) const;

		//! Combine styles from source object 'this' to the destination object 'DestObj'.
		//! this method deals with style links from source tables
		//! Used for objects with direct style attachments (a.k.a CAD)
		ERRVALUE CombineStylesTo (
			RVC::GEOMETRIC &DestObj,
			SIMPLE_ARRAY<INT32> &FontTrans,
			SIMPLE_ARRAY<INT32> PattTrans[PATTERNTYPE_NumTypes]
			) const;

		//! Convert geometric "component" to ELEMTYPE.
		ELEMTYPE ComponentToElemType (
			GEOMETRIC::COMPONENT component		//!< Component
			) const { return (v_ComponentToElemType(component)); }

		//! Copy an element from one part of the search tree to another (atomic method)
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE CopySearchTreeElement (
			INT32 SourceElement,
			INT32 DestElement,
			ELEMTYPE ElemType,
			const DRECT2D &Extents
			);

		//! Copy an element from one part of the search tree to another (atomic method)
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE CopySearchTreeElement (
			INT32 SourceElement,
			INT32 DestElement,
			ELEMTYPE ElemType,
			const DPOINT2D &Point
			);

		//! Create an SML context for the given derived class instance
		ERRVALUE CreateSMLContext (
			SML::CONTEXT **pcontext					//!< SML context allocated and RETURNED
			) const;

		//! Delete the search tree
		ERRVALUE DeleteSearchTree (
			ELEMTYPE ElemType
			);

		//! Delete an element from the search tree
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE DeleteSearchTreeElement (
			INT32 elemnum,
			ELEMTYPE ElemType,
			const DRECT2D &Extents
			);

		//! Delete an element from the search tree
		//! Need to call OpenSearchTree() before using this method
		ERRVALUE DeleteSearchTreeElement (
			INT32 elemnum,
			ELEMTYPE ElemType,
			const DPOINT2D &Point
			);

		//! Extract elements from source 'this' object to existing destination object
		//! Deals with database attachments and element transfers
		//! Not implemented TIN yet
		ERRVALUE ExtractTo (
			RVC::GEOMETRIC &DestObj,
			RVC::GEOMETRIC::ELEMTRANSLIST &TransList,
			const TRANS2D_MAPGEN &tmg,
			RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags = EXTRACTFLAG_None
			) const { return (v_ExtractTo(DestObj, TransList, tmg, ExtractFlags)); }

		//! Determine closest element based on point and search distance
		//! @return Element found, -1 if no elements are close enough, < -1 if error
		INT32 FindClosestElement (
			ELEMTYPE ElemType,						//!< Element type to search for
			const DPOINT2D &SearchPt,				//!< Search point in object coordinates
			double SearchDist = 0,					//!< Search distance in object coordinates
			double *pDistFromElem = 0				//!< Returned distance to element found
			) const { return (v_FindClosestElementLow(ElemType, SearchPt, 0, SearchDist, pDistFromElem)); }

		//! Determine closest element based on point and search distance with filter
		//! @return Element found, -1 if no elements are close enough, < -1 if error
		INT32 FindClosestElement (
			ELEMTYPE ElemType,						//!< Element type to search for
			const DPOINT2D &SearchPt,				//!< Search point in object coordinates
			SEARCHFILTER &SearchFilter,			//!< Search filter
			double SearchDist = 0,					//!< Search distance in object coordinates
			double *pDistFromElem = 0				//!< Returned distance to element found
			) const { return (v_FindClosestElementLow(ElemType, SearchPt, &SearchFilter, SearchDist, pDistFromElem)); }

		//! Determine set of elements that are within the 'SearchDist' of the 'SearchPt'
		ERRVALUE FindClosestElements (
			const DPOINT2D &SearchPt,
			double SearchDist,
			SIMPLE_ARRAY<RVC::ELEMIDX> &ElementList,
			COMPONENT Component = COMPONENT_All
			) const { return (v_FindClosestElementsLow(SearchPt, SearchDist, 0, ElementList, Component)); }

		//! Determine set of elements that are within the 'SearchDist' of the 'SearchPt' with filter
		ERRVALUE FindClosestElements (
			const DPOINT2D &SearchPt,
			double SearchDist,
			SEARCHFILTER &SearchFilter,
			SIMPLE_ARRAY<RVC::ELEMIDX> &ElementList,
			COMPONENT Component = COMPONENT_All
			) const { return (v_FindClosestElementsLow(SearchPt, SearchDist, &SearchFilter, ElementList, Component)); }

		//! Obtain set of elements that fall within the rectangle, return list in 'SearchResult'
		//! This method will NOT build search trees to determine the elements that fall within the rectangle.
		//! If a search or spatial tree is not available to limit the search, all elements will be returned
		ERRVALUE FindElementDrawSet (
			ELEMTYPE ElemType,
			const DRECT2D &SearchExtents,
			SEARCHRESULT &SearchResult
			) const;

		ERRVALUE FindElements (
			const REGION2D &SearchRegion,						//!< Search region, may contain points, lines or subregions
			double SearchDistance,								//!< Search distance in object coordinate space
			COMPONENT component,									//!< Component element type
			SIMPLE_ARRAY<RVC::ELEMIDX> &ElementsFound,	//!< Array of elements found returned
			FINDELEMENT flags,									//!< Flags for how to perform search
			DOUBLE_ARRAY<double> *pDistances = 0,			//!< Distances to elements returned (optional)
			DELEGATE_BOOL_ELEMIDX filter = 0					//!< Filter delegate for whether element usable (optional)
			) const;

		//! Obtain set of elements that fall within the rectangle, return list in 'SearchResult'
		//! This method will build search trees, if needed, to determine the elements that fall within the rectangle.
		ERRVALUE FindElementSearchSet (
			ELEMTYPE ElemType,
			const DRECT2D &SearchExtents,
			SEARCHRESULT &SearchResult
			) const;

		//! Obtain set of elements that fall within specified extent, optionally building index/quadtree if needed.
		ERRVALUE FindElementSet (
			ELEMTYPE ElemType,						//!< Element type
			const DRECT2D &Extents,					//!< Extents
			UNSIGNEDSET &ElemSet,					//!< Element set returned
			bool BuildIndex = false					//!< Build search index (quadtree) if does not exist or is outdated
			) const;

		//! Obtain the DBASE_ELEMENT Attachment index from the element specified
		//! @return '-1' if element does not have a database attachment, >= 0 if it does.
		ELEMNUM GetAttachmentIndex (
			const RVC::ELEMIDX &elemidx
			) const { return (v_GetAttachmentIndex(elemidx)); }

		//! Obtain DIMENSION type
		//! @return DIMENSION type
		DIMENSION GetDimension (
			) const { return ((GetPointType() == POINTTYPE_3DXYZ) ? DIMENSION_3D : DIMENSION_2D); }

		//! Obtain element extents.
		ERRVALUE GetElementExtents (
			const RVC::ELEMIDX &elemidx,
			DRECT3D &Extents					//!< Element extents RETURNED, set to invalid if error or element is deleted
			) const { return (v_GetElementExtents(elemidx, Extents)); }

		#ifndef GENERATING_DOXYGEN_OUTPUT
	#ifndef NO_DEPRECATED
		//! Obtain element extents.
		DEPRECATED ERRVALUE GetElementExtents (
			ELEMTYPE ElemType,
			INT32 ElemNum,
			DRECT3D &Extents					//!< Element extents RETURNED, set to invalid if error or element is deleted
			) const { return (GetElementExtents(RVC::ELEMIDX(ElemNum,ElemType), Extents)); }
	#endif
		#endif

		//! Obtain next element ID depending on element type
		//! @return Element ID to be used for the next element
		INT32 GetNextElementID (
			ELEMTYPE ElemType
			) const { return (v_GetNextElementID(ElemType)); }

		//! Obtain number of elements based on element type
		//! Specifying ELEMTYPE_All will return the total combination of elements in the object
		//! @return Number of elements if valid ELEMTYPE for object, -1 if not
		INT32 GetNumElements (
			ELEMTYPE ElemType
			) const { return (v_GetNumElements(ElemType)); }

		//! Obtain number of point coordinate dimensions
		//! @return Number of dimensions
		INT32 GetPointDimension (
			) const { return (GetPointSize() / sizeof(double)); }

		//! Obtain point coordinate type
		//! @return POINTTYPE coordinate type
		POINTTYPE GetPointType (
			) const { return (v_GetPointType()); }

		//! Obtain size of point coordinate
		//! @return Size of point in bytes
		INT32 GetPointSize (
			) const { return (v_GetPointSize()); }

		//! Obtain the total line length of all of the lines in the geometric object combined
		//! @return Total length of all of the lines combined or zero if not computed
		double GetTotalLineLength (
			) const { return (v_GetTotalLineLength()); }

		//! Determine if the element in this object is deleted.
		//! @return 'true' if the element is deleted, 'false' if not.
		bool IsElementDeleted (
			const RVC::ELEMIDX &elemidx
			) const { return (v_IsElementDeleted(elemidx)); }

		//! Determine if the object has any elements.
		//! @return 'true' if the object is empty, 'false' if not.
		bool IsEmpty (
			) const { return (v_IsEmpty()); }

		//! Obtain a start iterator for geometric labels
		//! @return a LABELITERATOR instance that contains the first label or LabelEnd()			
		LABELITERATOR LabelBegin (
			DIMENSION dim = DIMENSION_2D,
			STYLE *pObjStyle = 0
			) { return (LABELITERATOR(this, dim, pObjStyle)); }

		//! Obtain the end iterator for geometric labels
		LABELITERATOR LabelEnd (
			) { return (LABELITERATOR(this)); }

		//! Obtain a start iterator for geometric lines
		//! @return a LINEITERATOR instance that contains the first line or LineEnd()			
		LINEITERATOR LineBegin (
			DIMENSION dim = DIMENSION_2D
			) { return (LINEITERATOR(this, dim)); }

		//! Obtain the end iterator for geometric lines
		LINEITERATOR LineEnd (
			) { return (LINEITERATOR(this)); }

		//! Obtain a start iterator for geometric nodes
		//! @return a NODEITERATOR instance that contains the first node or NodeEnd()			
		NODEITERATOR NodeBegin (
			DIMENSION dim = DIMENSION_3D
			) { return (NODEITERATOR(this, dim)); }

		//! Obtain the end iterator for geometric nodes
		NODEITERATOR NodeEnd (
			) { return (NODEITERATOR(this)); }

		//! Obtain a start iterator for geometric points
		//! @return a POINTITERATOR instance that contains the first point set or PointEnd()
		POINTITERATOR PointBegin (
			DIMENSION dim = DIMENSION_2D
			) { return (POINTITERATOR(this, dim)); }

		//! Obtain the end iterator for geometric points
		POINTITERATOR PointEnd (
			) { return (POINTITERATOR(this)); }

		//! Obtain a start iterator for geometric polygons
		//! @return a POLYGONITERATOR instance that contains the first polygon set or PolygonEnd()
		POLYGONITERATOR PolygonBegin (
			DIMENSION dim = DIMENSION_2D
			) { return (POLYGONITERATOR(this, dim)); }

		//! Obtain the end iterator for geometric polygons
		POLYGONITERATOR PolygonEnd ()
			{ return (POLYGONITERATOR(this)); }

		//! Read general element as region.
		//! Note that for labels, the region represents the location(s) being labeled. If the label has leader lines,
		//! the region will contain a point for each leader endpoint. If no leaders are present, and the label is
		//! anchored at a single point, the region will contain the anchor point. Otherwise the region will contain
		//! the label baseline. If additional information about the label is needed, use ReadLabel instead.
		ERRVALUE ReadElement (
			const ELEMIDX &elemidx,				//!< Element specifier
			REGION2D &region,						//!< Element region returned, will only be empty if element cannot be represented as region
			DIMENSION dim							//!< Dimension
			) const { return (v_ReadElement(elemidx, region, dim)); }
		
		#ifndef GENERATING_DOXYGEN_OUTPUT
	#ifndef NO_DEPRECATED
		//! Read "line" element.
		DEPRECATED_MSG("Use ReadLine instead") ERRVALUE ReadElement (
			const ELEMIDX &elemidx,				//!< Element specifier
			POLYLINELIST &LineList,				//!< Element line(s) returned, will be empty if element is not 'line' type. 
			DIMENSION dim							//!< Dimension
			) const { return (v_ReadLine(elemidx, LineList, dim)); }
	#endif
		#endif
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
	#ifndef NO_DEPRECATED
		//! Read "point" element.
		DEPRECATED_MSG("Use ReadPoint instead") ERRVALUE ReadElement (
			const ELEMIDX &elemidx,				//!< Element specifier
			POLYLINE &PointList,					//!< Element point(s) returned, will be empty if element is not 'point' type.
			DIMENSION dim							//!< Dimension
			) const { return (v_ReadPoint(elemidx, PointList, dim)); }
	#endif
		#endif
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
	#ifndef NO_DEPRECATED
		//! Read an element into 'LineList'.  LineList will be empty if the element is not a 'line' type.
		DEPRECATED_MSG("Use ReadLine instead") ERRVALUE ReadElement (
			INT32 ElemNum, 
			POLYLINELIST &LineList, 
			DIMENSION dim
			) const { return (v_ReadLine(ELEMIDX(ElemNum), LineList, dim)); }
	#endif
		#endif
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
	#ifndef NO_DEPRECATED
		//! Read an element into 'PointList'.  PointList will be empty if the element is not a 'point' type.
		DEPRECATED_MSG("Use ReadPoint instead") ERRVALUE ReadElement (
			INT32 ElemNum, 
			POLYLINE &PointList, 
			DIMENSION dim
			) const { return (v_ReadPoint(ELEMIDX(ElemNum), PointList, dim)); }
	#endif
		#endif
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
	#ifndef NO_DEPRECATED
		//! Read an element into 'Region'.  Region will be empty if the element is not a 'polygon' type.
		DEPRECATED ERRVALUE ReadElement (
			INT32 ElemNum, 
			REGION2D &Region, 
			DIMENSION dim
			) const { return (v_ReadElement(ELEMIDX(ElemNum), Region, dim)); }
	#endif
		#endif

		//! Read "label" element.
		ERRVALUE ReadLabel (
			const ELEMIDX &elemidx,				//!< Element specifier
			LABELLIST &label,						//!< Label list returned, will be empty if element is not a 'label' type.
			DIMENSION dim = DIMENSION_2D,		//!< Dimension
			STYLE *pObjStyle = 0					//!< Style object (optional)
			) const { return (v_ReadLabel(elemidx,label,dim,pObjStyle)); }
		
		//! Read "label" element as REGION2D.
		//! Note that for labels, the region represents the location(s) being labeled. If the label has leader lines,
		//! the region will contain a point for each leader endpoint. If no leaders are present, and the label is
		//! anchored at a single point, the region will contain the anchor point. Otherwise the region will contain
		//! the label baseline. If additional information about the label is needed, use other ReadLabel instead.
		ERRVALUE ReadLabel (
			const ELEMIDX &elemidx,				//!< Element specifier
			REGION2D &region,						//!< Element region returned
			DIMENSION dim = DIMENSION_2D		//!< Dimension
			) const;
			
		//! Read "line" element as POLYLINELIST.
		ERRVALUE ReadLine (
			const ELEMIDX &elemidx,				//!< Element specifier
			POLYLINELIST &LineList,				//!< Element line(s) returned, will be empty if element is not 'line' type. 
			DIMENSION dim							//!< Dimension
			) const { return (v_ReadLine(elemidx, LineList, dim)); }
			
		//! Read "line" element as REGION2D.
		ERRVALUE ReadLine (
			const ELEMIDX &elemidx,				//!< Element specifier
			REGION2D &region,						//!< Element region returned
			DIMENSION dim							//!< Dimension
			) const;
			
		//! Read "point" element as POLYLINE.
		ERRVALUE ReadPoint (
			const ELEMIDX &elemidx,				//!< Element specifier
			POLYLINE &PointList,					//!< Element point(s) returned, will be empty if element is not 'point' type.
			DIMENSION dim							//!< Dimension
			) const { return (v_ReadPoint(elemidx, PointList, dim)); }
			
		//! Read "point" element as REGION2D.
		ERRVALUE ReadPoint (
			const ELEMIDX &elemidx,				//!< Element specifier
			REGION2D &region,						//!< Element region returned
			DIMENSION dim							//!< Dimension
			) const;
			
		//! Read specified set of elements as a REGION2D.
		//! Resulting region may contain subregion, line and point components.
		//! The region returned will be empty if the virtual method is not supported.
		ERRVALUE ReadRegion (
			ELEMTYPE ElemType,					//!< Element type if applicable
			const UNSIGNEDSET &ElemSet,		//!< Set of elements
			REGION2D &region,						//!< Region returned
			DIMENSION dim = DIMENSION_2D,		//!< Dimension requested
			bool bExcludeIslands = false		//!< Whether to exclude islands
			) const;
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
	#ifndef NO_DEPRECATED
		//! Generate a region given a bitset of polygonal elements to use.
		//! The 'Region' will be empty if the virtual method is not supported.
		DEPRECATED_MSG("Use version with ELEMTYPE/UNSIGNEDSET parameters") ERRVALUE ReadRegion (
			const BITSET_UNOWNED &ElemSet,	//!< Set of polygonal elements to generate region from
			REGION2D &Region,						//!< Region to set
			DIMENSION dim,							//!< Set to dimension needed
			bool NoIslands = false				//!< 'True' of no islands wanted
			) const;
	#endif
		#endif
			
		//! Set extents of an object
		//! Only use if you have a write lock on the entire object
		ERRVALUE SetExtents (
			const DRECT3D &extents
			) { return (v_SetExtents(extents)); }

		//! Set next element ID depending on element type
		ERRVALUE SetNextElementID (
			ELEMTYPE ElemType,
			INT32 NextElementID
			) { return (v_SetNextElementID(ElemType, NextElementID)); }

		//! Set total line length of all of the lines in the geometric object combined
		ERRVALUE SetTotalLineLength (
			double length
			) { return (v_SetTotalLineLength(length)); }

		//! Generate element translation lists dependent on the selection parameters.
		//! @return > 0 if elements are found to meet the selection parameters, '0' if no elements found, < 0 error
		int SetupElemTransList (
			const RVC::GEOMETRIC::ELEMSELECTPARMS &SelectParms,
			RVC::GEOMETRIC::ELEMTRANSLIST &TransList,
			bool separate = false								//!< Separate components if used in more than one element
			) const { return (SetupElemTransList(SelectParms, TransList, 0, separate)); }

		//! Generate element translation lists dependent on the selection parameters.
		//! @return > 0 if elements are found to meet the selection parameters, '0' if no elements found, < 0 error
		int SetupElemTransList (
			const RVC::GEOMETRIC::ELEMSELECTPARMS &SelectParms,
			RVC::GEOMETRIC::ELEMTRANSLIST &TransList,
			RVC::GEOMETRIC::ELEMSELECTFILTER &SelectFilter,
			bool separate = false								//!< Separate components if used in more than one element
			) const { return (SetupElemTransList(SelectParms, TransList, &SelectFilter, separate)); }

		//! Test element against region
		//! The REGION2D needs to be in 'object' coordinates, no conversion is done internally
		//! @return True if test against region was sucessful
		bool TestElementAgainstRegion (
			const RVC::ELEMIDX &elemidx,
			const REGION2D &region,
			TESTCRITERIA TestMode
			) const { return (v_TestElementAgainstRegion(elemidx, region, TestMode)); }

		//! Update the extents of an object, this is a thread safe method
		ERRVALUE UpdateExtents (
			const DRECT3D &extents
			) { return (v_UpdateExtents(extents)); }

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		static void GeometricCallbackFunc (ANYCALLBACKSTRUCT *cb, void *data);
		#endif // GENERATING_DOXYGEN_OUTPUT

		virtual ERRVALUE v_FindElements (const REGION2D &SearchRegion, double SearchDistance, COMPONENT component, SIMPLE_ARRAY<RVC::ELEMIDX> &ElementsFound, FINDELEMENT flags, DOUBLE_ARRAY<double> *pDistances, DELEGATE_BOOL_ELEMIDX filter) const;

	private:

		//! Read the next label in order based on the previous label given
		//! The 'Element' field determines which element was previously read and will be set to the next element
		//! that was read on this method's return.  If the ELEMIDX::GetNumber() returns -1, the end of the 
		//! element list has been reached.
		ERRVALUE ReadNextLabel (
			ELEMIDX &elemidx,
			LABELLIST &Label,
			DIMENSION dim = DIMENSION_2D,
			STYLE *pObjStyle = 0
			) const { return (v_ReadNextLabel(elemidx, Label, dim, pObjStyle)); }

		//! Read the next point in order based on the previous point given
		//! The 'Element' field determines which element was previously read and will be set to the next element
		//! that was read on this method's return.  If the ELEMIDX::GetNumber() returns -1, the end of the 
		//! element list has been reached.
		ERRVALUE ReadNextLine (
			ELEMIDX &elemidx,
			POLYLINELIST &LineList,
			DIMENSION dim = DIMENSION_2D
			) const { return (v_ReadNextLine(elemidx, LineList, dim)); }

		//! Read the next point in order based on the previous point given
		//! The 'elemidx' field determines which element was previously read and will be set to the next element
		//! that was read on this method's return.  If the ELEMIDX::GetNumber() returns -1, the end of the 
		//! element list has been reached.
		ERRVALUE ReadNextNode (
			ELEMIDX &elemidx,
			DPOINT3D &Node
			) const { return (v_ReadNextNode(elemidx, Node)); }

		//! Read the next point in order based on the previous point given
		//! The 'elemidx' field determines which element was previously read and will be set to the next element
		//! that was read on this method's return.  If the ELEMIDX::GetNumber() returns -1, the end of the 
		//! element list has been reached.
		ERRVALUE ReadNextPoint (
			ELEMIDX &elemidx,
			POLYLINE &PointList,
			DIMENSION dim = DIMENSION_2D
			) const { return (v_ReadNextPoint(elemidx, PointList, dim)); }

		//! Read the next point in order based on the previous point given
		//! The 'elemidx' field determines which element was previously read and will be set to the next element
		//! that was read on this method's return.  If the ELEMIDX::GetNumber() returns -1, the end of the 
		//! element list has been reached.
		ERRVALUE ReadNextRegion (
			ELEMIDX &elemidx,
			REGION2D &Region,
			DIMENSION dim = DIMENSION_2D
			) const { return (v_ReadNextRegion(elemidx, Region, dim)); }

		// Pure virtual methods
		virtual ELEMTYPE v_ComponentToElemType (GEOMETRIC::COMPONENT component) const = 0;
		virtual ERRVALUE v_ExtractTo (RVC::GEOMETRIC &DestObj, RVC::GEOMETRIC::ELEMTRANSLIST &TransList, const TRANS2D_MAPGEN &tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const = 0;
		virtual INT32 v_FindClosestElementLow (ELEMTYPE ElemType, const DPOINT2D &SearchPt, SEARCHFILTER *SearchFilter, double SearchDist, double *DistFromElem) const = 0;
		virtual ERRVALUE v_GetElementExtents (const RVC::ELEMIDX &elemidx, DRECT3D &Extents) const = 0;
		virtual INT32 v_GetNextElementID (ELEMTYPE ElemType) const = 0;
		virtual INT32 v_GetNumElements (ELEMTYPE ElemType) const = 0;
		virtual POINTTYPE v_GetPointType () const = 0;
		virtual INT32 v_GetPointSize () const = 0;
		virtual QSEARCHTREE* v_GetSearchTree (ELEMTYPE ElemType) const = 0;
		virtual double v_GetTotalLineLength () const = 0;
		virtual bool v_IsElementDeleted (const RVC::ELEMIDX &elemidx) const = 0;
		virtual bool v_IsEmpty () const = 0;
		virtual ERRVALUE v_SetExtents (const DRECT3D &extents) = 0;
		virtual ERRVALUE v_SetNextElementID (ELEMTYPE ElemType, INT32 NextElementID) = 0;
		virtual void v_SetSearchTree (ELEMTYPE ElemType, QSEARCHTREE *SearchTree) const = 0;
		virtual ERRVALUE v_SetTotalLineLength (double length) = 0;
		virtual int v_SetupElemTransList (const RVC::GEOMETRIC::ELEMSELECTPARMS &SelectParms, RVC::GEOMETRIC::ELEMTRANSLIST &TransList, RVC::GEOMETRIC::ELEMSELECTFILTER *SelectFilter, bool separate) const = 0;
		virtual bool v_TestElementAgainstRegion (const RVC::ELEMIDX &elemidx, const REGION2D &region, TESTCRITERIA TestMode) const = 0;
		virtual ERRVALUE v_UpdateExtents (const DRECT3D &extents) = 0;

		// Virtual methods
		virtual ERRVALUE v_FindClosestElementsLow (const DPOINT2D &SearchPt, double SearchDist, SEARCHFILTER *SearchFilter, SIMPLE_ARRAY<RVC::ELEMIDX> &ElementList, COMPONENT Component) const;
		virtual ERRVALUE v_FindElementDrawSet (ELEMTYPE ElemType, const DRECT2D &SearchExtents, SEARCHRESULT &SearchResult) const;
		virtual ERRVALUE v_FindElementSearchSet (ELEMTYPE ElemType, const DRECT2D &SearchExtents, SEARCHRESULT &SearchResult) const;
		virtual ERRVALUE v_FindElementSet (ELEMTYPE ElemType, const DRECT2D &SearchExtents, UNSIGNEDSET &ElemSet, bool BuildIndex) const;
		virtual ELEMNUM v_GetAttachmentIndex (const RVC::ELEMIDX &elemidx) const;
		virtual SPATIALTREE* v_GetSpatialTree () const;
		virtual ERRVALUE v_ReadElement (const ELEMIDX &elemidx, REGION2D &region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadLabel (const ELEMIDX &elemidx, LABELLIST &labels, DIMENSION dim, STYLE *pObjStyle) const;
		virtual ERRVALUE v_ReadLine (const ELEMIDX &elemidx, POLYLINELIST &lines, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextLabel (ELEMIDX &elemidx, LABELLIST &Label, DIMENSION dim, STYLE *pObjStyle) const;
		virtual ERRVALUE v_ReadNextLine (ELEMIDX &elemidx, POLYLINELIST &LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextNode (ELEMIDX &elemidx, DPOINT3D &Node) const;
		virtual ERRVALUE v_ReadNextPoint (ELEMIDX &elemidx, POLYLINE &PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextRegion (ELEMIDX &elemidx, REGION2D &Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadPoint (const ELEMIDX &elemidx, POLYLINE &points, DIMENSION dim) const;
		virtual ERRVALUE v_ReadRegion (ELEMTYPE ElemType, const UNSIGNEDSET &ElemSet, REGION2D &region, DIMENSION dim, bool bExcludeIslands) const;
		virtual void v_SetSpatialTree (SPATIALTREE*) const;

		#ifndef GENERATING_DOXYGEN_OUTPUT
		ERRVALUE DeleteSearchTrees (OBJTYPE ObjTypeSearchTree, ELEMTYPE ElemType, bool ErrorIfCantDelete = false) const;
		ERRVALUE FillInSearchTree (ELEMTYPE ElemType) const;
		ERRVALUE OpenSearchTree (ELEMTYPE ElemType, OPENMODE mode, bool DoNotCreate = false) const;
		ERRVALUE OpenFoundSearchTree (ELEMTYPE ElemType, OPENMODE mode, bool &NeedTempTree) const;
		int SetupElemTransList (const RVC::GEOMETRIC::ELEMSELECTPARMS &SelectParms, RVC::GEOMETRIC::ELEMTRANSLIST &TransList, RVC::GEOMETRIC::ELEMSELECTFILTER *SelectFilter, bool separate) const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GEOMETRIC::COMPONENT)
DEFINE_ENUM_OP_BITWISE(GEOMETRIC::EXTRACTFLAGS)
DEFINE_ENUM_OP_BITWISE(GEOMETRIC::FINDELEMENT)
#endif

//---------------------------------------------------------------------------------------------------------------------

class GEOMETRIC::ELEMSELECTFILTER {
	public:
		ELEMSELECTFILTER () {}

		//! Filter elements based on some external criteria
		//! @return '1' for element acceptance, '0' to reject element, < 0 for an error
		int FilterElement (
			RVC::ELEMIDX &elemidx
			) { return (v_FilterElement(elemidx)); }

	private:
		virtual int v_FilterElement (RVC::ELEMIDX &elemidx) = 0;
	};

//---------------------------------------------------------------------------------------------------------------------

//! Parameters for element selection/extraction.
class GEOMETRIC::ELEMSELECTPARMS {
	public:

		//! Extraction operation by extents.
		enum EXTRACT {
			EXTRACT_All,
			EXTRACT_PartInside,
			EXTRACT_FullInside,
			EXTRACT_ClipInside,
			EXTRACT_PartOutside,
			EXTRACT_FullOutside,
			EXTRACT_ClipOutside,
			EXTRACT_ClipByPolygons
			};

		//! Item defining how component elements are selected.
		struct ITEM {

			enum SELECT {
				SELECT_None = 0,
				SELECT_All,
				SELECT_ByRecord,
				SELECT_ByScript,
				SELECT_ByElement,
				SELECT_Last
				};

			//! Default constructor.
			ITEM ();

			//! Get set of selected elements, indexed by element number.
			const BITSET& GetElementSet () const
				{ return (m_ElemList); }

			//! Get query string.
			const MISTRING& GetQueryStr () const
				{ return (m_QueryStr); }

			//! Get set of selected records, indexed by record number.
			const BITSET& GetRecordSet () const
				{ return (m_RecordList); }

			SELECT GetSelection () const
				{ return (m_SelectType); }

			//! Get table name corresponding to record set.
			const RVC::OBJECTNAME& GetTableName () const
				{ return (m_TableName); }

			SELECT m_SelectType;
			MISTRING m_QueryStr;
			RVC::OBJECTNAME m_TableName;		//!< Table that 'RecordList' is selected from
			BITSET m_RecordList;					//!< List of records in 'TableNum' that the user has selected (bit array)
			BITSET m_ElemList;					//!< List of elements (bit array)
			};

		//! Parameters for output to separate object/files by input.
		class SEPARATION {
			public:

				//! Default constructor.
				SEPARATION ();

				//! Destructor.
				~SEPARATION ();

				//! Clear separation.
				void Clear ();

				//! Get element type to base separation on, ELEMTYPE_None if not separating.
				ELEMTYPE GetElemType () const
					{ return (m_ElemType); }

				//! Get field name.
				const MISTRING& GetFieldName () const
					{ return (m_FieldName); }

				//! Get file/object name prefix.
				const MISTRING& GetPrefix () const
					{ return (m_Prefix); }

				//! Get file/object name suffix.
				const MISTRING& GetSuffix () const
					{ return (m_Suffix); }

				//! Get table name.
				const MISTRING& GetTableName () const
					{ return (m_TableName); }

				//! Determine whether each output is to own file or all in same file.
				bool IsEachToOwnFile () const
					{ return (m_IsEachToOwnFile); }

				//! Set element type and table/field.
				void Set (
					ELEMTYPE ElemType,					//!< Element type to base separation on
					const MISTRING &TableName,			//!< Table name to separate by
					const MISTRING &FieldName			//!< Field name to separate by
					) { m_ElemType = ElemType; m_TableName = TableName; m_FieldName = FieldName; }

				//! Set whether each output is to own file or all in same file.
				void SetEachToOwnFile (
					bool IsEachToOwnFile
					) { m_IsEachToOwnFile = IsEachToOwnFile; }

				//! Set file/object name prefix.
				void SetPrefix (
					const MISTRING &Prefix
					) { m_Prefix = Prefix; }

				//! Set file/object name suffix.
				void SetSuffix (
					const MISTRING &Suffix
					) { m_Suffix = Suffix; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				ELEMTYPE m_ElemType;
				MISTRING m_TableName;
				MISTRING m_FieldName;
				MISTRING m_Prefix;
				MISTRING m_Suffix;
				bool m_IsEachToOwnFile;
				#endif
			};

		//! Default constructor.
		ELEMSELECTPARMS ();

		//! Access (non-const) selection parameter item by element type.
		ITEM& AccessSelectParmItem (
			ELEMTYPE ElemType
			);

		//! Access (non-const) separation parameters.
		SEPARATION& AccessSeparation ()
			{ return (m_Separation); }

		//! Clear all selection values.
		void Clear ();

		//! Get whether to add border when extracting by region.
		bool GetAddBorder () const
			{ return (m_bAddBorder); }

		//! Get OBJITEM for extents.
		const RVC::OBJITEM& GetExtentsObjItem () const
			{ return (m_ExtentsObjItem); }

		//! Get extraction region.
		const REGION2D& GetRegion2D () const
			{ return (m_Region); }

		//! Get extraction mode for use with region.
		EXTRACT GetRegionExtractMode () const
			{ return (m_ExtractMode); }

		//! Get region test criteria.
		TESTCRITERIA GetRegionTestCriteria () const
			{
			TESTCRITERIA TestCriteria = TESTCRITERIA_PartInside;
			switch (m_ExtractMode) {
				case EXTRACT_FullOutside:	TestCriteria = TESTCRITERIA_FullOutside; break;
				case EXTRACT_PartOutside:	TestCriteria = TESTCRITERIA_PartOutside; break;
				case EXTRACT_ClipOutside:	TestCriteria = TESTCRITERIA_PartOutside; break;
				case EXTRACT_FullInside:	TestCriteria = TESTCRITERIA_FullInside; break;
				case EXTRACT_PartInside:	TestCriteria = TESTCRITERIA_PartInside; break;
				case EXTRACT_ClipInside:	TestCriteria = TESTCRITERIA_PartInside; break;
				}
			return (TestCriteria);
			}

		//! Get selection parameter item by element type.
		const ITEM& GetSelectParmItem (
			ELEMTYPE ElemType
			) const;

		//! Access (non-const) selection parameter item by element type.
		ITEM& GetSelectParmItem (
			ELEMTYPE ElemType
			);

		//! Get separation parameters.
		const SEPARATION& GetSeparation () const
			{ return (m_Separation); }

		//! Determine if has non-empty region for possible extraction by extent.
		bool HasRegion () const
			{ return (!m_Region.IsEmpty()); }

		//! Set whether to add border when extracting by region.
		void SetAddBorder (
			bool bAddBorder
			) { m_bAddBorder = bAddBorder; }

		//! Set OBJITEM for extents.
		void SetExtentsObjItem (
			const RVC::OBJITEM &objitem
			) { m_ExtentsObjItem = objitem; }

		//! Set extent region.
		void SetRegion (
			const REGION2D &Region
			) { m_Region = Region; }

		//! Set region extraction mode.
		void SetRegionExtractMode (
			EXTRACT ExtractMode
			) { m_ExtractMode = ExtractMode; }

		//! Set to select all elements.
		void SetSelectAll ();

		//! Set to select no elements.
		void SetSelectNone ();

		//! Set selection parameter item by element type.
		void SetSelectParmItem (
			ELEMTYPE ElemType,
			const ITEM &item
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ITEM m_SelectParm[6];
		SEPARATION m_Separation;
		REGION2D m_Region;
		EXTRACT m_ExtractMode;
		OBJITEM m_ExtentsObjItem;
		bool m_bAddBorder;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Class to manage element translation lists from source to destination elements
//! Eg: 'DestElemNum = TransList[SourceElemNum]' where 'TransList' is an array of INT32's
//! The value '-1' states that the source element does not translate to a destination number.
class GEOMETRIC::ELEMTRANSLIST {
	public:

		//! Default constructor.
		ELEMTRANSLIST ();

		//! Copy constructor
		ELEMTRANSLIST (
			const ELEMTRANSLIST &rhs
			);

		//! Destructor
		~ELEMTRANSLIST ();

		//! Assignment Operator
		ELEMTRANSLIST& operator= (
			const ELEMTRANSLIST &rhs
			);

		//! Access element translation (non-const).
		//! @return Element Translation List
		ELEMNUMTRANS& AccessElementTransList (
			ELEMTYPE ElemType
			);

		//! Get base table name to use for extraction.  Currently used the extract to shape objects only.
		//! @return The name of the base table for extraction.
		const DESCRIPTOR& GetBaseTableName () const
			{ return (m_BaseTableName); }

		//! Get set of elements for type.
		ERRVALUE GetElementSet (
			ELEMTYPE ElemType,				//!< Element type
			UNSIGNEDSET &ElemSet				//!< Set of elements returned
			) const;

		//! Obtain the element translation array for a particular element type (ELEMTYPE)
		//! @return Element Number Translation
		const ELEMNUMTRANS& GetElementTransList (
			ELEMTYPE ElemType
			) const;

		//! Set base table name to use for extraction.  Currently used the extract to shape objects only.
		void SetBaseTableName (
			const DESCRIPTOR &BaseTableName
			) { m_BaseTableName = BaseTableName; }

		//! Set an element translation list entry to a pre-built list.
		void SetElementTransList (
			const ELEMNUMTRANS &TransList,
			ELEMTYPE ElemType
			);

		//! Set translation list to a one-to-one (Identity) mapping for all components.
		void SetIdentity (
			const RVC::GEOMETRIC &ObjGeometric			//!< Geometric object used to set identity arrays using the object type and number of elements per element type
			);

		//! Set an element translation list entry to a one-to-one (Identity) mapping
		void SetIdentity (
			const RVC::GEOMETRIC &ObjGeometric,		//!< Geometric object used to set identity arrays using the object type
			ELEMTYPE ElemType								//!< Element Type used to set the identity array entry using the number of elements for this type
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ELEMNUMTRANS m_TransList[6];
		DESCRIPTOR m_BaseTableName;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! GETOBJFILTER derived class to determine if the geometric object has the minimum number of elements of the specified types
class GEOMETRIC::GETOBJFILTER_HASMINIMUMELEMENTS : public GETOBJFILTER {
	public:
		GETOBJFILTER_HASMINIMUMELEMENTS (
			const ELEMTYPESET &ElemTypeSet,
			INT32 MinElements = 1
			) : m_ElemTypeSet(ElemTypeSet), m_MinElements(MinElements) {}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ELEMTYPESET m_ElemTypeSet;
		INT32 m_MinElements;

		virtual bool v_IsSelectable (const RVC::OBJITEM &ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Geometric object selection filter to disallow selection of empty and invalid extent geometric objects
class GEOMETRIC::GETOBJFILTER_ISDISPLAYABLE : public GETOBJFILTER {
	public:
		GETOBJFILTER_ISDISPLAYABLE () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool v_IsSelectable (const RVC::OBJITEM &ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Geometric object selection filter to disallow selection of empty geometric objects
class GEOMETRIC::GETOBJFILTER_ISNOTEMPTY : public GETOBJFILTER {
	public:
		GETOBJFILTER_ISNOTEMPTY () {}
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual bool v_IsSelectable (const RVC::OBJITEM &ObjItem, int position);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------------------------

//! Make parameter base class for RVC::GEOMETRIC objects.
class GEOMETRIC::MAKEPARMSBASE {
	public:

		//! Default constructor
		MAKEPARMSBASE (
			POINTTYPE PointType
			) :
			m_PointType(PointType),
			m_XScale(1.0),
			m_YScale(1.0),
			m_ZScale(1.0)
			{}

		//! Get vector object extents
		//! @return Extents of the vector object
		const DRECT3D& GetExtents (
			) const {return (m_Extents);}

		//! Obtain point coordinate type for vector creation
		//! @return RVCGEOMETRIC::POINTTYPE coordinate type
		POINTTYPE GetPointType (
			) const {return (m_PointType);}

		//! Get vector x scale value
		//! @return Vector X scale value
		double GetXScale (
			) const {return (m_XScale);}

		//! Get vector y scale value
		//! @return Vector Y scale value
		double GetYScale (
			) const {return (m_YScale);}

		//! Get vector z scale value
		//! @return Vector Z scale value
		double GetZScale (
			) const {return (m_ZScale);}

		//! Set vector object extents
		void SetExtents (
			const DRECT3D &rect
			) {m_Extents = rect;}

		//! Set coordinate point type for vector creation
		void SetPointType (
			POINTTYPE PointType
			) {m_PointType = PointType;}

		//! Set vector x scale value
		void SetXScale (
			double xscale
			) {m_XScale = xscale;}

		//! Set vector y scale value
		void SetYScale (
			double yscale
			) {m_YScale = yscale;}

		//! Set vector x scale value
		void SetZScale (
			double zscale
			) {m_ZScale = zscale;}

	protected:

		DRECT3D m_Extents;
		POINTTYPE m_PointType;
		double m_XScale;
		double m_YScale;
		double m_ZScale;
	};

//---------------------------------------------------------------------------------------------------------------------

//! OBSERVER class for RVC::GEOMETRIC objects.
class GEOMETRIC::OBSERVER : public RVC::OBJECTOBSERVER {
	public:
		enum TYPE {
			TYPE_ExtentsChange = 	0x00000008,
			TYPE_PointTypeChange = 	0x00000010
			};

		explicit OBSERVER (
			RVC::GEOMETRIC &object
			) : OBJECTOBSERVER(object) {}

		virtual ~OBSERVER (
			) {}

		//! Called when the geoemtric object extents has changed
		virtual void OnExtentsChanged (
			const DRECT3D&								//!< NewExtents
			) { return; }

		//! Called when the geoemtric object extents has changed
		virtual void OnPointTypeChanged (
			GEOMETRIC::POINTTYPE						//!< NewPointType
			) { return; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual void OnNotifyRaw (UINT32 reason, void *data);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Filter search class for FindClosestElement() method
class GEOMETRIC::SEARCHFILTER {
	public:
		//! Element filter for locating correct elements.
		//! @return True to use element in search, False to not use element in search
		virtual bool Filter (
			const RVC::GEOMETRIC &object,
			ELEMTYPE ElemType,
			INT32 ElemNum
			) = 0;

	protected:
		//! Destructor
		virtual ~SEARCHFILTER () {}
	};

//---------------------------------------------------------------------------------------------------------------------

//! Filter search class for FindClosestElement() method supporting bit-set.
class GEOMETRIC::SEARCHFILTER_BITSET : public GEOMETRIC::SEARCHFILTER {
	public:

		//! Construct from BITSET_UNOWNED.
		SEARCHFILTER_BITSET (
			const BITSET_UNOWNED &bitset
			) : m_bitset(bitset) { }

		//! Construct from bit array and count.
		SEARCHFILTER_BITSET (
			const UINT8 *set,
			UINT32 NumEntries
			) : m_bitset(const_cast<UINT8*>(set),NumEntries) { }

		//! Destructor
		virtual ~SEARCHFILTER_BITSET () {}

	protected:

		//! Element filter for locating correct elements.
		//! @return True if element in set, false if not.
		virtual bool Filter (
			const RVC::GEOMETRIC &object,
			ELEMTYPE ElemType,
			INT32 ElemNum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		BITSET_UNOWNED m_bitset;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Filter search class for FindClosestElement() method supporting bit-set.
class GEOMETRIC::SEARCHFILTER_UNSIGNEDSET : public GEOMETRIC::SEARCHFILTER {
	public:

		//! Construct from UNSIGNEDSET_UNOWNED.
		SEARCHFILTER_UNSIGNEDSET (
			const UNSIGNEDSET &ElemSet			//!< Element set, must not be compressed
			) : m_ElemSet(ElemSet) { }

		//! Destructor
		virtual ~SEARCHFILTER_UNSIGNEDSET () {}

	protected:

		//! Element filter for locating correct elements.
		//! @return True if element in set, false if not.
		virtual bool Filter (
			const RVC::GEOMETRIC &object,
			ELEMTYPE ElemType,
			INT32 ElemNum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		const UNSIGNEDSET &m_ElemSet;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

//! Filter search class for FindClosestElement() method supporting bit-set.
class GEOMETRIC::SEARCHFILTER_ELEMTRANSLIST : public GEOMETRIC::SEARCHFILTER {
	public:

		//! Construct from ELEMTRANSLIST
		SEARCHFILTER_ELEMTRANSLIST (
			const GEOMETRIC::ELEMTRANSLIST &ElemTransList
			) : m_ElemTransList(ElemTransList) { }

		//! Destructor
		virtual ~SEARCHFILTER_ELEMTRANSLIST () {}

	protected:

		//! Element filter for locating correct elements.
		//! @return True if element in set, false if not.
		virtual bool Filter (
			const RVC::GEOMETRIC &object,
			ELEMTYPE ElemType,
			INT32 ElemNum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		GEOMETRIC::ELEMTRANSLIST m_ElemTransList;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//---------------------------------------------------------------------------------------------------------------------

class GEOMETRIC::SEARCHRESULT {
	public:

		class CONST_ITERATOR;
		friend class CONST_ITERATOR;

		class CONST_ITERATOR {
			public:

				//! Default constructor
				CONST_ITERATOR (
					) :
					m_IsRange(false),
					m_CurElemNum(0),
					m_CurIndex(0),
					m_sr(0)
					{}

				//! Internal constructor, used by container.
				CONST_ITERATOR (
					const SEARCHRESULT *sr,
					bool CreateEnd = false
					);

				//! Copy constructor
				CONST_ITERATOR (
					const CONST_ITERATOR &rhs
					) :
					m_IsRange(rhs.m_IsRange),
					m_CurElemNum(rhs.m_CurElemNum),
					m_CurIndex(rhs.m_CurIndex),
					m_sr(rhs.m_sr)
					{}

				//! Dereference operator, returns current element number.
				const INT32& operator* (
					) const { return (m_CurElemNum); }

				//! Pre-increment operator.
				CONST_ITERATOR& operator++ (
					);

				//! Post-increment operator.
				CONST_ITERATOR operator++(int
					) {
					CONST_ITERATOR temp = *this;
					++*this;
					return (temp);
					}

				bool operator== (				//! Equality operator
					const CONST_ITERATOR &rhs
					) const { return (m_CurElemNum == rhs.m_CurElemNum); }

				bool operator!= (				//! Inequality operator
					const CONST_ITERATOR &rhs
					) const { return (!(*this == rhs)); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				bool m_IsRange;
				INT32 m_CurElemNum;
				INT32 m_CurIndex;
				const SEARCHRESULT *m_sr;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		SEARCHRESULT (
			);

		SEARCHRESULT (
			const SEARCHRESULT &rhs
			);

		~SEARCHRESULT (
			);

		SEARCHRESULT& operator= (
			const SEARCHRESULT &rhs
			);

		CONST_ITERATOR Begin (
			) const {return (CONST_ITERATOR(this));}

		void Clear (
			);

		CONST_ITERATOR End (
			) const {return (CONST_ITERATOR(this, true));}

		ERRVALUE FindElements (
			const DRECT2D &SearchExtents,
			RVC::QSEARCHTREE &QTreeObj,
			INT32 MaxElemNum
			);

		ERRVALUE FindElements (
			ELEMTYPE ElemType,
			const DRECT2D &SearchExtents,
			RVC::SPATIALTREE &STreeObj
			);

		ERRVALUE GetElementSet (
			UNSIGNEDSET &ElemSet
			) const;

		ERRVALUE GetElementSet (
			BITSET &SelectSet
			) const;

		ERRVALUE GetElementSet (
			SIMPLE_ARRAY<INT32> &SelectList
			) const;

		INT32 GetNumItems () const;

		bool IsEmpty () const;

		//! Set search result for the all elements set
		void SetAll (
			INT32 NumElements
			);

		//! Set search result for array passed in
		void SetArray (
			const SIMPLE_ARRAY<INT32> &ElemSet
			);

		//! Sort the elements in the list using the elements extents from smallest to largest
		void SortItemsByExtentArea (
			const RVC::GEOMETRIC &pObjGeometric,
			ELEMTYPE ElemType
			);

		//! Sort the elements in the list using the elements extents from smallest to largest
		void SortItemsByExtentArea (
			const RVC::GEOMETRIC &pObjGeometric,
			ELEMTYPE ElemType,
			const DRECT2D &OverlapFilterRect		//!< Filter out those entries that do not overlap this rectangle
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<INT32> m_list;
		SIMPLE_ARRAY<INT32_RANGE> m_range;
		INT32 m_MaxElemNum;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}		//! End of RVC namespace

#endif 	//!< INC_RVC_GEOMETRC_H
