/**
 * \file <rvc/opvector2dto3d.h>
 * \brief RVC::OP::VECTOR_2D_TO_3D class definition
 *
 * \if NODOC
 * $Id: opvector2dto3d.h_v 1.1 2017/01/16 16:18:56 mju Exp $
 *
 * $Log: opvector2dto3d.h_v $
 * Revision 1.1  2017/01/16 16:18:56  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPVECTOR2DTO3D_H
#define	INC_RVC_OPVECTOR2DTO3D_H

#ifndef	INC_RVC_NULLMASK_H
#include <rvc/nullmask.h>
#endif

#ifndef	INC_RVC_VECTOR_H
#include <rvc/vector.h>
#endif

#ifndef	INC_TRANS2DMAPGEN_H
#include <mi32/trans2dmapgen.h>
#endif

namespace RVC {
namespace OP {

//=====================================================================================================================
//		VECTOR_2D_TO_3D - Begin

//! Convert 2D vector to 3D X-Y-Z vector by assigning values from surface terrain raster to point and vertex Z values.
//! Note that no additional vertices are inserted by this process.
class VECTOR_2D_TO_3D {
	public:

		//! Constructor.
		VECTOR_2D_TO_3D ();

		//! Destructor.
		~VECTOR_2D_TO_3D ();

		//! Run operation.
		ERRVALUE Run (
			RVC::OBJITEM &ObjItemTgtVector,				//!< Target vector location
			const RVC::OBJITEM &ObjItemSrcVector,		//!< Source vector location
			const RVC::OBJITEM &ObjItemRefRaster		//!< Reference terrain raster location
			);

		//! Set value to use outside reference terrain raster.
		void SetValueOutside (
			double ValueOutside								//!< Value to use outside reference raster or for null cells
			) { m_ValueOutside = ValueOutside; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::VECTOR m_ObjSrcVector;
		RVC::VECTOR m_ObjTgtVector;
		RVC::RASTER m_ObjRefRaster;
		RVC::NULLMASK m_ObjRefNullMask;
		TRANS2D_MAPGEN m_TransVtoR;
		DRECT3D m_TgtExtents;
		double m_ValueOutside;

		ERRVALUE ComputePointZ (const DPOINT2D &pt, double &z);
		ERRVALUE ComputePolyLineZ (POLYLINE &polyline, double &minz, double &maxz);

		VECTOR_2D_TO_3D (const VECTOR_2D_TO_3D &);
		VECTOR_2D_TO_3D& operator= (const VECTOR_2D_TO_3D &);
		#endif
	};

//=====================================================================================================================

} // End namespace OP
} // End namespace RVC

#endif	// INC_RVC_OPVECTOR2DTO3D_H
