/**
 * \file rvc/rasterthemeparm.h
 * \brief RVC::RASTERTHEMEPARM class definition
 *
 * \if NODOC
 * $Id: rasterthemeparm.h_v 1.7 2016/01/18 16:15:26 bcolombe Exp $
 *
 * $Log: rasterthemeparm.h_v $
 * Revision 1.7  2016/01/18 16:15:26  bcolombe
 * Remove extra qualification on method
 *
 * Revision 1.6  2015/06/11 21:45:12  mju
 * Change to use gre::rasterthememap class.
 *
 * Revision 1.5  2012/04/24 17:25:34  mju
 * Inline get and most set methods.
 * Don't return struct/class instance by value.
 *
 * Revision 1.1  2007/07/05 15:39:36  honerkamp
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_RASTERTHEMEPARM_H
#define INC_RVC_RASTERTHEMEPARM_H

#ifndef INC_RVC_SERIALZE_H
	#include <rvc/serialze.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace GRE {
	class RASTERTHEMEMAP;
	}
#endif

namespace RVC {

//=====================================================================================================================

//! Raster Theme Mapping parameters.
class RASTERTHEMEPARM : public RVC::SERIALIZE {
	public:

		//! Default constructor
		RASTERTHEMEPARM ();

		//! Copy constructor
		RASTERTHEMEPARM (
			const RVC::RASTERTHEMEPARM& rhs
			);

		//! Destructor
		virtual ~RASTERTHEMEPARM ();

		//! Assignment
		RVC::RASTERTHEMEPARM& operator= (
			const RVC::RASTERTHEMEPARM& rhs
			);

		// Make object and write values.
		ERRVALUE Make (
			OBJITEM &ObjItem,								//!< Object location passed/returned, can specify existing object to overwrite
			const GRE::RASTERTHEMEMAP &thememap		//!< Values to write
			);

		//! Read values.
		//! Will also read palette from subobject if any.
		ERRVALUE Read (
			GRE::RASTERTHEMEMAP &RasterThemeMap
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef SERIALIZE BASECLASS;

		virtual ERRVALUE v_GetDftNameDesc (const OBJECT &parent, DESCRIPTOR &descriptor);
		virtual OBJTYPE v_GetDftObjectType () const;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}		// End of RVC namespace

#endif 	// INC_RVC_RASTTHEMEPRM_H
