/** 
 * \file rvc/uieditrastervalue.h
 * \brief Definitions for RVC::UI::FORM_EDIT_RASTER_VALUE
 *
 * \if NODOC
 * $Id: uieditrastervalue.h_v 1.8 2015/01/29 21:16:38 mju Exp $
 *
 * $Log: uieditrastervalue.h_v $
 * Revision 1.8  2015/01/29 21:16:38  mju
 * Make internals private, change create parms to use style flags.
 *
 * Revision 1.5  2008/04/18 22:25:24  scowan
 * Added component labels and a style to allow them.
 *
 * Revision 1.4  2007/12/12 18:23:46  scowan
 * Added orientation parm to create method.
 *
 * Revision 1.1  2006/06/28 16:27:59  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIEDITRASTERVALUE_H
#define	INC_RVC_UIEDITRASTERVALUE_H

#ifndef	INC_MGUI_FORM_H
	#include <mgui/form.h>
#endif

#ifndef	INC_RVC_IMAGE_H
	#include <rvc/image.h>
#endif

#ifndef	INC_MI32_ANYRASTV_H
	#include <mi32/anyrastv.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace MGUI {
	class CTRL_LABEL;
	}
#endif

namespace RVC {
namespace UI {

//=====================================================================================================================

//! Form consisting of label and edit field(s) for specified image cell type.
class FORM_EDIT_RASTER_VALUE : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum STYLE {
			STYLE_Default =				0x00,		//!< Default style
			STYLE_ComponentLabels =		0x01,		//!< Include component name labels
			STYLE_Vertical =				0x02,		//!< Use vertical orientation
			STYLE_MinMaxButtons =		0x04,		//!< Include buttons to set minimum and maximum values
			STYLE_AllowedCellTypes =	0x08,		//!< Cell type specifies all allowed types (can be a single type)
			};
	
		//! Constructor.
		FORM_EDIT_RASTER_VALUE ();

		//! Destructor.
		virtual ~FORM_EDIT_RASTER_VALUE ();

		//! Clear current value.
		void ClearValue (
			bool notify = true							//!< Send notification that value changed
			);

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE &ParentPane,		//!< Parent pane
			const MISTRING &label,						//!< Label
			RVC::IMAGE::CELLTYPE CellType,			//!< Initial type or allowed cell types
			STYLE Style = STYLE_Default,				//!< Form style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Retrieve label control for form.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel ();

		//! Get current raster value.
		void GetValue (
			ANYRASTVALUE &value
			);
			
		//! Determine if value has been set.
		bool HasValue () const;

		//! Set cell type.			
		void SetCellType (
			RVC::IMAGE::CELLTYPE CellType
			);

		//! Set delegate to call when value changed.
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set current value with validation.
		//! If value specified is not valid for current raster type the value will not be updated.
		//! No comparison is made with the current value.
		void SetValue (
			const ANYRASTVALUE &value,				//!< New value to set
			bool notify = true						//!< Notify delegate of change
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_EDIT_RASTER_VALUE::STYLE)
#endif

//=====================================================================================================================

}	// End of UI namespace
}	// End of RVC namespace

#endif	// INC_RVC_UIEDITRASTERVALUE_H
