/**
 * \file rvc/uifiltergetobjectdbase.h
 * \brief RVC::UI::FILTER_GETOBJECT_DBASE definitions
 *
 * \if NODOC
 * $Id: uifiltergetobjectdbase.h_v 1.2 2014/10/09 14:27:00 mju Exp $
 *
 * $Log: uifiltergetobjectdbase.h_v $
 * Revision 1.2  2014/10/09 14:27:00  mju
 * Fix return type of setAllowEmpty.
 *
 * Revision 1.1  2013/05/08 14:30:21  mju
 * Initial revision
 *
 * Revision 1.1  2013/05/07 19:59:33  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIFILTERGETOBJECTDBASE_H
#define	INC_RVC_UIFILTERGETOBJECTDBASE_H

#ifndef	INC_MGUI_DLGGETOBJECTFILTER_H
#include <mgui/dlggetobjectfilter.h>
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================

//! Filter for selecting RVC "dbase" object.
class FILTER_GETOBJECT_DBASE : public MGUI::DLG_GETOBJECT_FILTER {
	public:

		//! Default constructor, allows all "dbase" object types.
		FILTER_GETOBJECT_DBASE ();

		//! Construct with specific set of allowed object types.
		//! If any non-dbase types are specified they will always be included by this class.
		FILTER_GETOBJECT_DBASE (
			const RVC::OBJTYPESET& ObjTypeSetSelectable		//!< Set of allowed object types
			);

		//! Destructor.
		virtual ~FILTER_GETOBJECT_DBASE ();

		//! Set whether to allow "empty" database.
		//! Default is to not allow.
		void SetAllowEmpty (
			bool bAllowEmpty = true
			) { m_bAllowEmpty = bAllowEmpty; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef MGUI::DLG_GETOBJECT_FILTER BASECLASS;
		typedef FILTER_GETOBJECT_DBASE THISCLASS;

		bool m_bAllowEmpty;

		int IsSelectableDbase (const RVC::OBJITEM& objitem, MISTRING& msg, int position) const;
		int IsValidParentDbase (const RVC::OBJITEM& objitem, MISTRING& msg, int position) const;
		#endif
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIFILTERGETOBJECTDBASE_H
