/**
 * \file rvc/uifiltergetobjectgeospatial.h
 * \brief RVC::UI::FILTER_GETOBJECT_GEOSPATIAL definitions
 *
 * \if NODOC
 * $Id: uifiltergetobjectgeospatial.h_v 1.2 2014/05/29 13:33:31 mju Exp $
 *
 * $Log: uifiltergetobjectgeospatial.h_v $
 * Revision 1.2  2014/05/29 13:33:31  mju
 * Add needgeoref_Local.
 *
 * Revision 1.1  2013/05/07 19:59:33  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIFILTERGETOBJECTGEOSPATIAL_H
#define	INC_RVC_UIFILTERGETOBJECTGEOSPATIAL_H

#ifndef	INC_MGUI_DLGGETOBJECTFILTER_H
#include <mgui/dlggetobjectfilter.h>
#endif

#ifndef	INC_RVC_GEOSPATIAL_H
#include <rvc/geospatl.h>
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================

//! Filter for selecting RVC "geospatial" object.
class FILTER_GETOBJECT_GEOSPATIAL : public MGUI::DLG_GETOBJECT_FILTER {
	public:

		//! Kind of georeference needed.
		enum NEEDGEOREF {
			NEEDGEOREF_AnyOrNone = 0,		//!< Don't check georeference status
			NEEDGEOREF_Any,					//!< Disallow if not georeferenced
			NEEDGEOREF_NonLocal,				//!< Allow only if georeferenced to non-local CRS
			NEEDGEOREF_Local					//!< Allow only if georeferenced to local CRS
			};

		//! Default constructor, allows all "geospatial" object types.
		FILTER_GETOBJECT_GEOSPATIAL ();

		//! Construct with specific set of allowed object types.
		//! If any non-geospatial types are specified they will always be included by this class.
		FILTER_GETOBJECT_GEOSPATIAL (
			const RVC::OBJTYPESET& ObjTypeSetSelectable		//!< Set of allowed object types
			);

		//! Destructor.
		virtual ~FILTER_GETOBJECT_GEOSPATIAL ();

		//! Set kind of georeference needed.
		void SetNeedGeoref (
			NEEDGEOREF NeedGeoref
			) { m_NeedGeoref = NeedGeoref; }

		//! Set to check if overlaps specified object.
		ERRVALUE SetNeedOverlap (
			const OBJITEM *pObjItemCheck		//!< Object to check if overlapping, NULL for none
			);

		//! Set to check if overlaps specified region.
		void SetNeedOverlap (
			const REGION2D& region				//!< Region to check if overlapping
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef MGUI::DLG_GETOBJECT_FILTER BASECLASS;
		typedef FILTER_GETOBJECT_GEOSPATIAL THISCLASS;

		REGION2D *m_pRegionOverlap;
		NEEDGEOREF m_NeedGeoref;

		int IsGeospatialSelectable (const RVC::OBJITEM& objitem, MISTRING& msg, int position) const;
		#endif
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIFILTERGETOBJECTGEOSPATIAL_H
