/**
 * \file rvc/uifiltergetobjectstyle.h
 * \brief RVC::UI::FILTER_GETOBJECT_STYLE definitions
 *
 * \if NODOC
 * $Id: uifiltergetobjectstyle.h_v 1.1 2013/05/08 13:32:31 mju Exp $
 *
 * $Log: uifiltergetobjectstyle.h_v $
 * Revision 1.1  2013/05/08 13:32:31  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIFILTERGETOBJECTSTYLE_H
#define	INC_RVC_UIFILTERGETOBJECTSTYLE_H

#ifndef	INC_MGUI_DLGGETOBJECTFILTER_H
#include <mgui/dlggetobjectfilter.h>
#endif

#ifndef	INC_RVC_STYLE_H
#include <rvc/style.h>
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================

//! Filter for selecting RVC "style" object.
//! By default will allow browsing into other objects that contain style objects.
class FILTER_GETOBJECT_STYLE : public MGUI::DLG_GETOBJECT_FILTER {
	public:

		//! Default constructor, allows OBJTYPE_Style only.
		FILTER_GETOBJECT_STYLE ();

		//! Construct with specific set of allowed object types.
		//! If other types are specified they will always be included by this class.
		FILTER_GETOBJECT_STYLE (
			const RVC::OBJTYPESET& ObjTypeSetSelectable		//!< Set of allowed object types
			);

		//! Destructor.
		virtual ~FILTER_GETOBJECT_STYLE ();

		//! Set specific pattern type needed.
		void SetNeedPatternType (
			PATTERNTYPE PatternType
			) { m_NeedPatternType = PatternType; }

		//! Set specific style type needed.
		void SetNeedStyleType (
			STYLETYPE StyleType
			) { m_NeedStyleType = StyleType; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef MGUI::DLG_GETOBJECT_FILTER BASECLASS;
		typedef FILTER_GETOBJECT_STYLE THISCLASS;

		PATTERNTYPE m_NeedPatternType;
		STYLETYPE m_NeedStyleType;

		int IsStyleSelectable (const RVC::OBJITEM& objitem, MISTRING& msg, int position) const;
		#endif
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIFILTERGETOBJECTSTYLE_H
