/**
 * \file sml/cartoscript.h
 * \brief SML::CARTOSCRIPT definitions
 *
 * \if NODOC
 * $Id: cartoscript.h_v 1.2 2013/04/05 15:28:53 mju Exp $
 *
 * $Log: cartoscript.h_v $
 * Revision 1.2  2013/04/05 15:28:53  mju
 * Make cartoscript a class instead of a namespace.
 * Add setDrawingContext, setLinePoints, setupStyleDraw methods.
 *
 * Revision 1.1  2010/04/21 18:07:47  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_SML_CARTOSCRIPT_H
#define	INC_SML_CARTOSCRIPT_H

#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
class UNSIGNEDSET;
class TRANS2D_MAPGEN;
namespace SML {
	class CONTEXT;
	class SYMBOL;
	}
namespace MGD {
	class CONTEXT;
	}
#endif

namespace SML {

//=====================================================================================================================

class CARTOSCRIPT {

	public:

		//! Draw "optimized" labels for specified set of elements.
		static ERRVALUE DrawOptimizedLabels (
			SML::CONTEXT *context,
			ELEMTYPE ElemType,
			const UNSIGNEDSET& ElemSet
			);

		//! Set drawing context and transformation.
		static void SetDrawingContext (
			SML::CONTEXT *context,
			MGD::CONTEXT *gc,
			const TRANS2D_MAPGEN& tmg
			);

		//! Cartoscript line setup function
		static ERRVALUE SetLinePoints (
			SML::CONTEXT *context,
			const DPOINT2D *pts,
			INT32 NumPts,
			int CloseMode
			);

		//! Cartoscript setup function
		static void SetupStyleDraw (
			SML::CONTEXT *context,
			MGD::CONTEXT *ggc,
			const TRANS2D_MAPGEN& tmg,				//!< Coordinate and clipping transformation
			bool Is3D,
			SML::SYMBOL *DrawDoneSym				//!< SML symbol to indicate that drawing was done, NULL if none
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		#endif
	};

//=====================================================================================================================

}	// End namespace SML

#endif	// INC_SML_CARTOSCRIPT_H
