/**
 * \file sml/parm.h
 * \brief SML PARM
 *
 * \if NODOC
 * $Log: parm.h_v $
 * Revision 1.4  2015/02/27 20:15:06  mju
 * Make flags const.
 *
 * Revision 1.3  2014/11/12 22:32:29  mju
 * Add parmflag enum and parmflag_IndexBase0/1 values.
 *
 * Revision 1.2  2007/05/08 15:46:11  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 22:46:55  dwilliss
 * Initial revision
 * \endif
**/

#ifndef INC_SML_PARM_H
#define INC_SML_PARM_H

#ifndef	INC_SML_SMLSTYPE_H
	#include <sml/stype.h>
#endif

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif


namespace SML {

class FLAG;

/*****************************************************************************/
/*
** This class is used to define the parameters to an SML::FUNC.  See SML::FUNC for
** an example.
**
** This structure is designed to be statically defined, and as such strings MUST
** be (char*) (at least until all compilers have a way to define Unicode string
** constants ).
**
** @see SML::FUNC
*/
class PARM {
	public:
		const char *name;			//!< Parameter name
		STYPE type;					//!< STYPE_Xxxxx : See <mi32/sml.h>
		UINT32 flags;				//!< PARM_Optional, et.al.
		const char *desc;			//!< default description of parm - only used to create smlfuncs.txt  (And not really used for that anymore either)
		const FLAG *flag;			//!< (SML::FLAG*) array
		const char *ClassName;	//!< Name of parameter class (if STYPE_Class)
	};

enum PARMFLAG {
	PARMFLAG_Required =			0x0000,	//!< Parameter required (default)
	PARMFLAG_Optional =			0x0001,
	PARMFLAG_ByRef =				0x0002,
	PARMFLAG_UndefOK =			0x0004,	//!< Parameter can be undefined
	// Don't reuse 0x0008 as used elsewhere
	PARMFLAG_NeedNext =			0x0010,	//!< Need next parameter (for optional)
	PARMFLAG_IndexBase0 =		0x0020,	//!< "Index" starting at 0
	PARMFLAG_IndexBase1 =		0x0040,	//!< "Index" starting at 1
	};

} // end of SML namespace

/* Flags in SMLSYMBOL.flags (only a short) */

#define PARM_Optional			SML::PARMFLAG_Optional
#define PARM_ByRef				SML::PARMFLAG_ByRef
#define PARM_UndefOK				SML::PARMFLAG_UndefOK

typedef SML::PARM SMLPARM;

#endif		//  INC_SML_PARM_H
