/**
 * \file spatmod/imagefiltercomplextype.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_COMPLEXTYPE definition.
 *
 * \if NODOC
 * $Id: imagefiltercomplextype.h_v 1.2 2015/09/08 18:59:32 mju Exp $
 *
 * $Log: imagefiltercomplextype.h_v $
 * Revision 1.2  2015/09/08 18:59:32  mju
 * Support both single and component-pair conversions.
 *
 * Revision 1.1  2015/08/28 17:48:50  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERCOMPLEXTYPE_H
#define	INC_SPATMOD_IMAGEFILTERCOMPLEXTYPE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to convert between Complex forms (Real/Imaginary and Magnitude/Phase) or select single complex component.
//! If a single component is selected its sample type will become SAMPLE::TYPE_Gray.
//! All non-complex samples will be passed through unchanged in all modes.
class FILTER_COMPLEXTYPE : public FILTER {
	public:

		//! Mode for operation.
		enum MODE {
			MODE_None =				0,
			MODE_R =					0x01,		//!< Real component
			MODE_I =					0x02,		//!< Imaginary component
			MODE_RI =				0x03,		//!< Both Real and Imaginary components
			MODE_M =					0x04,		//!< Magnitude component
			MODE_P =					0x08,		//!< Phase component
			MODE_MP =				0x0C,		//!< Both Magnitude and Phase components
			};

		//! Constructor.
		FILTER_COMPLEXTYPE (
			STAGE *pSourceImage,						//!< Source image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			MODE Mode									//!< Mode for component(s) to output
			);

		//! Destructor.
		virtual ~FILTER_COMPLEXTYPE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MODE m_Mode;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);

		// Unimplemented
		FILTER_COMPLEXTYPE (const FILTER_COMPLEXTYPE &);
		FILTER_COMPLEXTYPE& operator= (const FILTER_COMPLEXTYPE &);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERCOMPLEXTYPE_H
