/**
 * \file spatmod/imagefilterlogarithm.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_LOGARITHM definition.
 *
 * \if NODOC
 * $Id: imagefilterlogarithm.h_v 1.1 2015/09/29 20:55:05 mju Exp $
 *
 * $Log: imagefilterlogarithm.h_v $
 * Revision 1.1  2015/09/29 20:55:05  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERLOGARITHM_H
#define	INC_SPATMOD_IMAGEFILTERLOGARITHM_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to compute logarithm of image sample values.
class FILTER_LOGARITHM : public FILTER {
	public:

		//! Constructor.
		FILTER_LOGARITHM (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership						//!< Filter takes ownership of previous stage and will delete when needed
			);

		//! Destructor.
		virtual ~FILTER_LOGARITHM ();

		//! Set logarithm base.
		//! Default base is "e" (base of natural logarithm).
		void SetBase (
			double base							//!< Base, must be > 1
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_Scale;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		FILTER_LOGARITHM (const FILTER_LOGARITHM&);
		FILTER_LOGARITHM& operator= (const FILTER_LOGARITHM&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERLOGARITHM_H

