/**
 * \file spatmod/imagemrustagelist.h
 * \brief SPATMOD::IMAGE::MRUSTAGELIST
 *
 * \if NODOC
 * $Id: imagemrustagelist.h_v 1.1 2016/01/18 21:11:52 mju Exp $
 *
 * $Log: imagemrustagelist.h_v $
 * Revision 1.1  2016/01/18 21:11:52  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEMRUSTAGELIST_H
#define	INC_SPATMOD_IMAGEMRUSTAGELIST_H

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class STAGE;
class SECTION_ITERATOR;

//=====================================================================================================================

//! Most-Recently-Used (MRU) Stage List.
class MRUSTAGELIST {
	public:

		//! Construct with optional maximum size
		explicit MRUSTAGELIST (
			UINT32 MaxItems = 32
			);

		//! Destructor.
		~MRUSTAGELIST ();

		//! Add item to front of list, calling PullBegin with specified iterator.
		//! Last item will be removed from list if exceeds maximum allowed size.
		ERRVALUE Add (
			STAGE *pStage,							//!< Item to add to front of list
			SECTION_ITERATOR &iterator,		//!< Iterator to specify in PullBegin on stage
			bool bAllowGrow2X = true			//!< Allow to grow to 2X maximum size, typically during a call to PullSubSection.
			);

		//! Clear the list.
		void Clear ();

		//! Find whether item is in list and move to front if found.
		//! @return true if found, false if not.
		bool Find (
			STAGE *pStage,
			bool bMoveToFront = true
			);

		//! Get maximum size.
		UINT32 GetMaxItems () const
			{ return (m_MaxItems); }

		//! Reduce to maximum allowed number of items.
		void ReduceToMaxItems ()
			{ ReduceToMaxItems(m_MaxItems); }

		//! Reduce list size to specified number of items.
		//! Will call PullEnd on each item removed from list.
		void ReduceToMaxItems (
			UINT32 NumItems
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MILIST<STAGE*> m_list;
		UINT32 m_MaxItems;

		void RemoveLast ();
		#endif
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEMRUSTAGELIST_H
