/**
 * \file imagesourcetext.h <spatmod/imagesourcetext.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_TEXT definition.
 *
 * \if NODOC
 * $Id: imagesourcetext.h_v 1.1 2014/06/16 16:29:54 bcolombe Exp $
 *
 * $Log: imagesourcetext.h_v $
 * Revision 1.1  2014/06/16 16:29:54  bcolombe
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCETEXT_H
#define	INC_SPATMOD_IMAGESOURCETEXT_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

#ifndef INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

class SOURCE_TEXT : public SOURCE {
	public:

		//! Constructor.
	 	SOURCE_TEXT (const FILEPATH& path);
			
		//! Destructor.
		virtual ~SOURCE_TEXT ();

		//! Read pixel range, requires source be initialized
		ERRVALUE ReadPixelRange(double& MinValue, double& MaxValue);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
	
		FILEPATH m_path;
		MISTRING m_text;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_TEXT (const SOURCE_TEXT&);
		SOURCE_TEXT& operator= (const SOURCE_TEXT&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCETEXT_H

