/**
 * \file spatmod/imagesourcetilesetpublic.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_TILESET_PUBLIC definition.
 *
 * \if NODOC
 * $Id: imagesourcetilesetpublic.h_v 1.6 2016/10/27 20:00:08 mju Exp $
 *
 * $Log: imagesourcetilesetpublic.h_v $
 * Revision 1.6  2016/10/27 20:00:08  mju
 * Remove mapquest as discontinued.
 *
 * Revision 1.5  2014/12/11 22:29:44  bcolombe
 * Add GetBingURL methods
 *
 * Revision 1.4  2014/05/21 23:23:33  ldyas
 * added Open Street
 *
 * Revision 1.3  2013/08/30 20:49:07  bcolombe
 * Add First,Last type
 *
 * Revision 1.2  2013/08/29 15:13:10  bcolombe
 * Add IsTileValid
 *
 * Revision 1.1  2013/08/26 19:52:45  bcolombe
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCETILESET_PUBLIC_H
#define	INC_SPATMOD_IMAGESOURCETILESET_PUBLIC_H

#ifndef	INC_SPATMOD_IMAGESOURCETILESET_H
#include <spatmod/imagesourcetileset.h>
#endif

#ifndef INC_SPATMOD_IMAGESAMPLEITERATOR_H
#include <spatmod/imagesampleiterator.h>
#endif

#ifndef INC_SPATMOD_IMAGESTAGEAUTOPTR_H
#include <spatmod/imagestageautoptr.h>
#endif

#ifndef INC_GRE_LAYERDC_H
#include <gre/layerdc.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

class SOURCE_TILESET_PUBLIC : public SOURCE_TILESET {
	public:
		
		enum MAPSTYLE {
			MAPSTYLE_Bing =						0x0001,
			MAPSTYLE_OpenStreet =				0x0004,
			MAPSTYLE_Aerial =						0x0100,
			MAPSTYLE_AerialWithLabels =		0x0200,
			MAPSTYLE_Road =						0x0400,
			MAPSTYLE_Bing_Aerial =				(MAPSTYLE_Bing|MAPSTYLE_Aerial),
			MAPSTYLE_Bing_AerialWithLabels =	(MAPSTYLE_Bing|MAPSTYLE_AerialWithLabels),
			MAPSTYLE_Bing_Road =					(MAPSTYLE_Bing|MAPSTYLE_Road),
			};

		// Constructor.
		SOURCE_TILESET_PUBLIC (MAPSTYLE MapStyle);

		// Destructor.
		virtual ~SOURCE_TILESET_PUBLIC ();

		ERRVALUE DrawOverlay (const GRE::LAYERDC& layerdc);

		bool IsTileValid (const FILEPATH& PathTileFile) const;

		void SetMapStyle (MAPSTYLE MapStyle);

		void SetResampleMethod (const SPATMOD::IMAGE::RESAMPLEMETHOD& ResampleMethod) {m_ResampleMethod = ResampleMethod;}

	protected:
		virtual ERRVALUE v_GetDatatip (const DPOINT2D& point, MISTRING& string, INT32 ZoomLevelToUse, SOURCE_TILESET::DATATIPFLAG flags);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MAPSTYLE m_MapStyle;
		MISTRING m_CopyrightStringRight;
		MISTRING m_CopyrightStringLeft;
		MISTRING m_LogoURL;
		FILEPATH m_LogoPath;
		SPATMOD::IMAGE::RESAMPLEMETHOD m_ResampleMethod;
		
		void GetMapStyleString (MISTRING& MapStyleStr);
		void GetBingURL (MISTRING& xmlstr);
		void GetBingURL (MISTRING& xmlstr, INT32 ZoomLevelToUse, const LPOINT2D& TileIdx);
		ERRVALUE SetupURLs();

		// SOURCE_TILESET overrides.
		virtual void v_GetCacheName (MISTRING& CacheName);
		virtual ERRVALUE v_ProcessBottomTile (
			SPATMOD::IMAGE::STAGE*& source, 
			INT32 ZoomLevelToUse, 
			const LPOINT2D& TileIdx, 
			const FILEPATH& PathTileFile);

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		SOURCE_TILESET_PUBLIC (const SOURCE_TILESET_PUBLIC&);
		SOURCE_TILESET_PUBLIC& operator= (const SOURCE_TILESET_PUBLIC&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEGOOGLEMAPS_H
