/**
 * \file spatmod/imagetargetcalcstatistics.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_CALC_STATISTICS definition.
 *
 * \if NODOC
 * $Id: imagetargetcalcstatistics.h_v 1.1 2015/01/05 15:43:17 mju Exp $
 *
 * $Log: imagetargetcalcstatistics.h_v $
 * Revision 1.1  2015/01/05 15:43:17  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETNULL_H
#define	INC_SPATMOD_IMAGETARGETNULL_H

#ifndef	INC_SPATMOD_IMAGETARGET_H
#include <spatmod/imagetarget.h>
#endif

#ifndef	INC_MI32_MIMATRIX_H
#include <mi32/mimatrix.h>
#endif

namespace SPATMOD {
namespace IMAGE {

#ifndef GENERATING_DOXYGEN_OUTPUT
class SAMPLEITERATOR_CONST;
#endif

//=====================================================================================================================

//! Target to calculate multivariate statistics.
//! Will compute covariance, correlation and mean statistics.
class TARGET_CALC_STATISTICS : public TARGET {
	public:

		//! Construct target with specified input stage
		explicit TARGET_CALC_STATISTICS (
			STAGE *pInputStage							//!< Input stage
			);

		//! Destructor.
		virtual ~TARGET_CALC_STATISTICS ();

		//! Get correlation matrix.
		const MIMATRIX& GetCorrelation () const
			{ return (m_Correlation); }

		//! Get covariance matrix.
		const MIMATRIX& GetCovariance () const
			{ return (m_Covariance); }

		//! Get mean matrix.
		const MIMATRIX& GetMeans () const
			{ return (m_Mean); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MIMATRIX m_Correlation;
		MIMATRIX m_Covariance;
		MIMATRIX m_Mean;
		DOUBLE_ARRAY<double> m_sum;
		DOUBLE_ARRAY<double> m_sum2;
		SIMPLE_ARRAY<const double *> m_pValues;
		DOUBLE_ARRAY<double> m_values;
		SAMPLEITERATOR_CONST *m_itValues;
		UINT64 m_StatCells;
		bool m_bAllDouble;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR &iterator) const;

		// TARGET overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR &iterator);
		virtual ERRVALUE v_PushEnd ();
		virtual ERRVALUE v_PushSubSection (const SECTION &SubSection, const BUFFER &buffer);

		// Unimplemented
		TARGET_CALC_STATISTICS (const TARGET_CALC_STATISTICS &);
		TARGET_CALC_STATISTICS& operator= (const TARGET_CALC_STATISTICS &);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETNULL_H

